\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
read("c-curve-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


\\-----------------------------------------------------------------------------
\\ right boundary sqrt2 for zig-zag parts

R_by_sums(k) =
{
  if(k%2==0,
     \\ k=4 is sides RS 0 to 1, RZ 0 to 1, middle RS(2)
     my(h=k/2);
     RS(h) + 2*sum(i=0,h-1, RS(i) + RZ(i)),
     \\ k=3 is sides RS 0 to 1, RZ 0 to 0, middle RZ(1)
     my(h=k\2);
     RZ(h) + 2*sum(i=0,h, RS(i)) + 2*sum(i=0,h-1, RZ(i)));
}

check_exact(vector(100,k,k--; R_by_sums(k)), \
            vector(100,k,k--; R(k)))

R_with_sqrt_ref(k) =
{
  if(k%2==0,
     \\ k=4 is sides RS 0 to 1, RZ 0 to 1, middle RS(2)
     my(h=k/2);
     RS(h) + 2*sum(i=0,h-1, RS(i) + sqrt2/2*RZ(i)),
     \\ k=3 is sides RS 0 to 1, RZ 0 to 0, middle RZ(1)
     my(h=k\2);
     sqrt2/2*RZ(h) + 2*sum(i=0,h, RS(i)) + 2*sum(i=0,h-1, sqrt2/2*RZ(i)));
}
\\vector(100,k,k--; R_with_sqrt_ref(k))

R_with_sqrt_samples = [ 1+0*sqrt2, 2+0*sqrt2, 4+0*sqrt2, 6+1*sqrt2, 10+2*sqrt2, 14+5*sqrt2, 22+8*sqrt2, 30+15*sqrt2, 46+22*sqrt2, 62+37*sqrt2, 94+52*sqrt2, 126+83*sqrt2, 190+114*sqrt2, 254+177*sqrt2, 382+240*sqrt2 ];
check_exact(vector(#R_with_sqrt_samples,k,k--; R_with_sqrt_ref(k)), \
            R_with_sqrt_samples, \
            "R_with_sqrt_samples")


R_with_sqrt(k) =
{
  (3 + 2*sqrt2)* (sqrt2)^k  - sqrt2 * k  - 2 - 2*sqrt2;
}
check_exact(vector(100,k,k--; R_with_sqrt(k)), \
            vector(100,k,k--; R_with_sqrt_ref(k)), \
            "R_with_sqrt")

my(k=10000); R_with_sqrt(k)/sqrt2^k*1.0
3 + 4*sqrt2 * 1.0
my(k=10001); R_with_sqrt(k)/sqrt2^k*1.0
6 + 2*sqrt2 * 1.0