\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


read("c-curve-defines.gp");

check_count = 0;
check_exact(got,want, name="") =
{
  check_count++;
  if(got!= want,
     print("\nwrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

\\-----------------------------------------------------------------------------

r=[-1, 3, -3, 3, -2, -4, +8, -8, 16, -8];

vector_count_nonzeros(v) = sum(i=1,#v, v[i]!=0);
vector_no_negatives(v) = for(i=1,#v, if(v[i]<0,return(0))); 1;
vector_first_nonzero(v) = for(i=1,#v, if(v[i]!=0,return(v[i]))); 0;
vector_addmul_at(v, a, mul, pos) = for(i=1,#a, v[pos+i-1] += mul*a[i]); v;

apply_factors(f) =
{
  my(v = vector(#r+#f));
  v[1] = 1;
  for(i=1,#f,
      for(j=1,#r,
         v[i+j-1] += f[i]*r[j]);

     \\ v = vector_addmul_at(v, r, f[i], i);
  );
  v;
}

print(apply_factors([1]));
print(apply_factors([1,1]));

weight(v) = sum(i=1,#v, 1/2^(i-1)*max(0,v[i]));

found = 0;
min_weight = 9999;
compare(f) =
{
  v = apply_factors(f);
  if(vector_no_negatives(v) && v[1]==0,
     print("all positive ",v);
     quit();
    );
  my(w = weight(v));

  x = vector_count_nonzeros(v);
  if(vector_first_nonzero(f)
     \\ && w < 1 && x < min_weight
     && vector_no_negatives(v)
     ,
     found = 1;
     print("weight ",w,"  at ",f," = ",w*1.0,"  min ",x);
     print("v ",v);
     if(w<1,
       \\ quit();
       );
     min_weight = x);
}

limit = 12;
{
  \\ print("try ",f,"  (limit ",limit,")");
  \\ print("weight ",w,"  at ",f);

  my(pending = List([ [] ]));
  while(#pending && ! found,
        print("pending count ",#pending);
        if(#pending, print("length ",length(pending[1])));
        \\ print("pending ",(pending));
        my(new_pending=List([]));

        while(#pending,
              my(f = pending[#pending]);
              listpop(pending);
              \\ print("f = "f);
              compare(f);
              
              if(#f < limit,
                 v = apply_factors(f);
                 forstep(m=vector_first_nonzero(v),0,-1,  \\ descending
                 \\ for(m=0,vector_first_nonzero(v),       \\ ascending
                        listput(new_pending, concat(f,[m]));
                 )));
        \\ print(new_pending);
        pending = new_pending);
}


