#!/usr/bin/perl -w

# Copyright 2015, 2016, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'xy_rotate_plus90','xy_rotate_minus90',
  'transform_to_horizontal','rect_corners_sort', '@dir4_to_dx','@dir4_to_dy';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 14;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::CCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $check_xydxdy = sub {
  my ($x,$y, $dx,$dy) = @_;
  my $n = $path->xyxy_to_n($x,$y, $x+$dx,$y+$dy);
  return defined $n && $n < $n_hi;
};
my $check_xydxdy_both = sub {
  my ($x,$y, $dx,$dy) = @_;
  my @n_list = $path->xyxy_to_n_list_either($x,$y, $x+$dx,$y+$dy);
  @n_list = grep {$_ < $n_hi} @n_list;
  return @n_list == 2;
};
sub xydxdy_is_full {
  my ($x,$y, $dx,$dy) = @_;

  my ($nx,$ny) = (-$dx, -$dy);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  my ($lx,$ly) = xy_rotate_plus90($dx,$dy);

  # x,y, dx,dy  part 1
  $check_xydxdy_both->($x,$y, $rx,$ry) || return 0;       # part 10, 11
  $check_xydxdy_both->($x,$y, $nx,$ny) || return 0;       # part 12, 13
  $check_xydxdy_both->($x,$y, $lx,$ly) || return 0;       # part 14, 15

  $check_xydxdy_both->($x2,$y2, $dx,$dy) || return 0;     # part 5, 6
  $check_xydxdy_both->($x2,$y2, $rx,$ry) || return 0;     # part 7, 8
  $check_xydxdy     ->($x2,$y2, $nx,$ny) || return 0;     # part    2
  $check_xydxdy_both->($x2,$y2, $lx,$ly) || return 0;     # part 3, 4

  my ($tx2,$ty2) = ($x2+$rx, $y2+$ry);
  $check_xydxdy->($tx2,$ty2, $nx,$ny) || return 0;        # part 9

  return 1;
}

my ($enc_x,$enc_y);

# find fully enclosed triangle
my $skip = 0;
# 4+4 + 10+10 + 2+9+4+15+22+7+7+22+15+4+9+2 + 10+38+32+4;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  xydxdy_is_full($x,$y, $dx,$dy) || next;
  if ($skip-- > 0) { next; }

  my ($x2,$y2) = ($x+$dx, $y+$dy);
  print "  % Enc at N=$n  $x,$y to $x2,$y2\n";
  $enc_x = $x;
  $enc_y = $y;
  last;
}

my $enc_x_min = $enc_x - 2;
my $enc_x_max = $enc_x + 3;
my $enc_y_min = $enc_y - 3;
my $enc_y_max = $enc_y + 3;

sub in_range {
  my ($x,$y) = @_;
  return ($enc_x_min <= $x && $x <= $enc_x_max
          && $enc_y_min <= $y && $y <= $enc_y_max);
}


#          gap 
# x1,y1 *------>* x2,y2
#        \     /
#      gap\   /gap
#          \ /
#           *
#          x3,y3
sub triangle {
  my ($x1,$y1, $x2,$y2) = @_;
  my ($dx,$dy) = ($x2-$x1, $y2-$y1);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  my ($x3,$y3) = ($x1+$dx/2+$rx/2, $y1+$dy/2+$ry/2);

  if (in_range($x1,$y1) && in_range($x2,$y2) && in_range($x3,$y3)) {
    ($x1,$y1) = transform($x1,$y1);
    ($dx,$dy) = transform($dx,$dy);
    printwrap("\\pic at ($x1,$y1) [transform shape] {my triangle={$dx,$dy}};");
  }
}

my @enc;
print "  \\begin{scope}[my grey]\n";
$Printwrap = 4;
foreach my $x ($enc_x_min .. $enc_x_max) {
  foreach my $y ($enc_y_min .. $enc_y_max) {
    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];

      my ($x2,$y2) = ($x+$dx, $y+$dy);
      ### triangle: "$x,$y $x2,$y2"
      my $n = $path->xyxy_to_n($x,$y, $x2,$y2);
      next unless defined $n && $n < $n_hi;

      if (xydxdy_is_full($x,$y, $dx,$dy)) {
        push @enc, [$x,$y, $x2,$y2];
      } else {
        triangle($x,$y, $x2,$y2);
      }
    }
  }
}
printwrap_newline("\n");
print "  \\end{scope}\n";

$Printwrap = 2;
foreach my $elem (@enc) {
  triangle(@$elem);
}
printwrap_newline();

print "  \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

{
  my ($x1,$y1) = transform($enc_x_min,$enc_y_min);
  my ($x2,$y2) = transform($enc_x_max,$enc_y_max);
  ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
  my $xm = ($x1 + $x2) / 2;
  print "  \\coordinate (Enc-minimum) at ($xm,$y1);\n";
}

print "  % end generated\n";
exit 0;
