#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min','max';
use Math::BigRat;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'PartN','transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::CCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $segments = MyLineSegments->new;
my $x_total = 0;
my $y_total = 0;
my $count = 0;
my $PartN    = PartN($level);
my $PartNend = PartN($level+1);
foreach my $n ($PartN .. $PartNend-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  $x_total += $x + $dx/Math::BigRat->new(2);
  $y_total += $y + $dy/Math::BigRat->new(2);
  $count++;

  $segments->add($x,$y, $x+$dx,$y+$dy);
}
my $gx = $x_total / $count;
my $gy = $y_total / $count;

$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);;
printwrap(";\n");

my ($X_axis_x,$X_axis_y) = ($gx + $x_hi, $gy + $y_hi);
my ($Y_axis_x,$Y_axis_y) = ($gx + $y_hi, $gy - $x_hi);

{
  my ($x,$y) = transform($gx,$gy);
  print "% GP-Test-Last  Gpart($level) == $y\n";
  print "  \\node (Gpart) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy(0);
  ($x,$y) = transform($x,$y);
  print "  % \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  print "  % \\coordinate (end) at ($x,$y);\n";
}
print "  % end generated\n";
exit 0;
