#!/usr/bin/perl -w

# Copyright 2015, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min','sum';
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()
use Tie::IxHash;

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'xy_rotate_plus90','xy_rotate_minus90','transform_to_horizontal';
use MyPrintwrap ':all';
use MyLineSegments;

# uncomment this to run the ### lines
# use Smart::Comments;

my $max_level = 13;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";
my $path = Math::PlanePath::CCurve->new;

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;

my @QDs_lists;
foreach my $level (0 .. $max_level) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  my $xydxdy_left_to_square_lower_left = sub {
    my ($x,$y, $dx,$dy) = @_;
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy); # to the left
    my ($x2,$y2) = ($x+$dx, $y+$dy);
    my ($x3,$y3) = ($x+$lx, $y+$ly);
    my ($x4,$y4) = ($x2+$lx, $y2+$ly);
    return (min($x, $x2, $x3, $x4),
            min($y, $y2, $y3, $y4));
  };
  my $xydxdy_right_to_square_lower_left = sub {
    my ($x,$y, $dx,$dy) = @_;
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy); # to the right
    my ($x2,$y2) = ($x+$dx, $y+$dy);
    my ($x3,$y3) = ($x+$rx, $y+$ry);
    my ($x4,$y4) = ($x2+$rx, $y2+$ry);
    return (min($x, $x2, $x3, $x4),
            min($y, $y2, $y3, $y4));
  };

  my @QDs = (undef,0,0,0,0);
  my %seen;
  my $lower_left_sides_count = sub {
    my ($x,$y) = @_;
    return if $seen{"$x,$y"}++;
    my $s = 0;
    foreach my $n_list ([$path->xyxy_to_n_list_either($x,$y,   $x+1,$y)],
                        [$path->xyxy_to_n_list_either($x,$y+1, $x+1,$y+1)],
                        [$path->xyxy_to_n_list_either($x,  $y, $x,  $y+1)],
                        [$path->xyxy_to_n_list_either($x+1,$y, $x+1,$y+1)]) {
      @$n_list = grep {$_ < $n_hi} @$n_list;
      if (@$n_list == 1) { return; }  # 1-segment side, ignore this square
      if (@$n_list == 2) { $s++; }    # 2-segment side
    }
    ### $s
    $QDs[$s]++;
  };

  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my ($sx,$sy) = $xydxdy_left_to_square_lower_left->($x,$y, $dx,$dy);
    $lower_left_sides_count->($sx,$sy);
  }

  if ($QDs[1] || $QDs[3]) {
    die "Oops, expected QD1 and QD3 zero";
  }
  foreach my $s (1 .. 4) {
    push @{$QDs_lists[$s]}, $QDs[$s];
  }
}

foreach my $s (2, 4) {
  my $aref = $QDs_lists[$s];
  print "% GP-DEFINE  ${var}_QD${s}_samples = [",join(',',@$aref),"];\n";
}

print "% end generated\n";
exit 0;
