\\ Copyright 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ THIS IS A GENERATED FILE
\\ Extracted from c-curve.tex by gp-inline.
\\
\\ These are Pari/GP functions and definitions used in c-curve.tex for
\\ various self-tests and formula checking there.  Names and styles are
\\ highly specific to the document, and all might change wildly or
\\ disappear, etc, but some of the functions may be useful for experimenting
\\ or testing.
\\
\\ The sets of configurations, generating functions, etc, use a fair bit of
\\ memory, maybe depending what is actually used etc.  The default gp 2.7 or
\\ 2.9 memory limit could be exceeded.  Try about 50mb in the usual ways, so
\\ from the command line
\\
\\     gp -s 50000000 foo.gp
\\
\\ or as the first thing in a script
\\
\\     default(parisizemax, max(default(parisizemax), 50*10^6));
\\
\\ Might have liked to set or add some suitable size in the defines here,
\\ but a read() of a script which sets default() quietly exits the read at
\\ that point.


/* ------------------------------------------------------------------------- */
/* gp-inline test boilerplate begin */
gp_inline__location = "";
gp_inline__good = 0;
gp_inline__bad = 0;
gp_inline__check(location,bool) =
{
  gp_inline__location = location;
  check(bool);
}
/* this is thinking about a check() for use in loops or "GP-Inline ..."
   testing, but it's not settled so don't rely on it */
check(bool) =
{
  /* use "===" so that a vector like [1] or float 1.0 not reckoned success */
  if(bool===1, gp_inline__good++,
     bool===0, gp_inline__bad++;
               print(gp_inline__location": gp-inline fail"),
     gp_inline__bad++;
     print(gp_inline__location": gp-inline expected result 0 or 1, got ",
           bool));
}
/* gp-inline test boilerplate end */
/* ------------------------------------------------------------------------- */

gp_inline__location="../c-curve.tex:436";
 default(strictargs,1);
 default(realprecision,100);

gp_inline__location="../c-curve.tex:439";
 \\ my http://user42.tuxfamily.org/pari-geom/index.html
 iferr(read("tools/geom.gp"),e, \
    read("../tools/geom.gp"));
 iferr(read("tools/geom-inertia.gp"),e, \
    read("../tools/geom-inertia.gp"));

gp_inline__location="../c-curve.tex:445";
 \\ my http://user42.tuxfamily.org/pari-memoize/index.html
 iferr(read("tools/memoize.gp"),e, \
    read("../tools/memoize.gp"));

gp_inline__location="../c-curve.tex:449";
 \\ my http://user42.tuxfamily.org/pari-pol-pfrac/index.html
 iferr(read("tools/pol-pfrac.gp"),e, \
    read("../tools/pol-pfrac.gp"))

gp_inline__location="../c-curve.tex:453";
 iferr(read("tools/c-curve-oeis-samples.gp"),e, \
    read("../tools/c-curve-oeis-samples.gp"));

gp_inline__location="../c-curve.tex:456";
 nearly_equal(x,y, epsilon=1e-12) = \
   x=abs(x-y); if(type(x)=="t_VEC",x=vecsum(x)); x < epsilon;

gp_inline__location="../c-curve.tex:459";
 \\ v is a vector, return its elements v[1]..v[#v] according
 \\ to i==0 to #v-1 mod #v, so the first element of v is
 \\ i==0.
 vector_modulo(v,i) = v[(i% #v)+1];

gp_inline__location="../c-curve.tex:464";
 \\ g is a polynomial fraction generating function.
 \\ Return a vector of its first n coefficients,
 \\ starting from coefficient of x^0.
 gf_terms(g,n) = {
   my(x = variable(g));
   if(x==0, \\ if g is an integer (polynomial constant only)
      return(vector(n,i, if(i==1,g))));
   my(zeros = min(n,valuation(g,x)),
      v = Vec(g + O(x^n)));
   if(zeros>=0, concat(vector(zeros,i,0), v),
                v[-zeros+1 .. #v]);
 }

gp_inline__location="../c-curve.tex:483";
 gf_term(g,n) = gf_terms(g,n+1)[n+1];

gp_inline__location="../c-curve.tex:485";
 \\ Generating function of all ones sequence 1,1,1,1,1,1,...
 gOnes(x)=1/(1-x);

gp_inline__location="../c-curve.tex:489";
 \\ Generating function of powers of 2, so 1,2,4,8,16,32,...
 gTwoPow(x)=1/(1-2*x);

gp_inline__location="../c-curve.tex:494";
 linear_recurrence_factors_to_poly(v) = x^#v - Pol(v);

gp_inline__location="../c-curve.tex:498";
 \\ g is a "t_RFRAC" polynomial generating function.
 \\ Return the characteristic polynomial of its recurrence.
 \\ This means the denominator with powers reversed.
 gf_to_linear_recurrence_poly(g) = {
   my(p=polrecip(denominator(g)));
   p / pollead(p);
 }

gp_inline__location="../c-curve.tex:508";
 \\ g is a "t_RFRAC" polynomial generating function.
 \\ Return a function f which is to be called f(x) for the
 \\ characteristic polynomial of the recurrence of g.
 gf_to_linear_recurrence_poly_func(g) = {
   my(p=gf_to_linear_recurrence_poly(g), v=variable(p));
   (x)->subst(p,v,x);
 }

gp_inline__location="../c-curve.tex:516";
 \\ Return a vector of length n which are the first n many indices
 \\ i for which f(i) is true.  f is checked at i=0,1,2,etc.
 select_first_n(f,n) = {
   my(l=List([]), i=0);
   while(#l<n, if(f(i),listput(l,i)); i++);
   Vec(l);
 }

gp_inline__location="../c-curve.tex:524";
 digits_join_high_to_low(v,base) = subst(Pol(v),'x,base);
 fromdigits_low_to_high(v,base=10) = subst(Polrev(v),'x,base);

gp_inline__location="../c-curve.tex:530";
 arg_0_to_2pi(z) = my(a=arg(z)); if(a<0,a+=2*Pi); a;

gp_inline__location="../c-curve.tex:532";
 \\ lex_by_abs(a,b) is lex(abs(a),abs(b)) comparing a,b by absolute
 \\ value, including complex numbers compared by magnitude.
 \\ If abs(a)==abs(b) then real(a),real(b) and imag(a),imag(b)
 \\ are compared, which means positives considered bigger.
 lex_by_abs(a,b) = \
   lex([abs(a),real(a),imag(a)],[abs(b),real(b),imag(b)]);
 lex_by_abs_descending(a,b) = - lex_by_abs(a,b);

gp_inline__location="../c-curve.tex:544";
 \\ Return polroots(p) sorted by magnitude per lex_by_abs().
 polroots_ascending(p) = vecsort(polroots(p),lex_by_abs);
 polroots_descending(p) = Vecrev(polroots_ascending(p));

gp_inline__location="../c-curve.tex:548";
 vector_midpoints(v) = if(#v==0,return([])); vector(#v-1,i,(v[i+1]+v[i])/2);

gp_inline__location="../c-curve.tex:553";
 vector_mean(v) = vecsum(v)/#v;
 vector_offset_to_centroid(v) = {
   my(centroid = vector_mean(v));
   apply(z -> z-centroid, v);
 }

gp_inline__location="../c-curve.tex:559";
 \\ gf_shift_down(g,n) takes a polynomial fraction "t_RFRAC" g
 \\ if a generating function of some sequence G(0),...
 \\ The return is a new t_RFRAC which is a generating function
 \\ for sequence G(n),..., ie. shifted down by n places.
 \\ Must have n>=0, currently.
 gf_shift_down(g,n) = {
   my(x = variable(g));
   for(i=1,n, g -= gf_term(g,0); g /= x);
   g;
 }

gp_inline__location="../c-curve.tex:570";
 \\ gf_in_terms_of(g,h) takes two polynomial fractions "t_RFRAC" g
 \\ and h and returns a vector f of coefficients which express g
 \\ in terms of shifts of h, so
 \\   g = f[1] * h
 \\     + f[2] * gf_shift_down(h,1)
 \\     + f[3] * gf_shift_down(h,2)
 \\     + ...
 \\ This is equivalent to expressing a function G in terms of H as
 \\   G(k) = f[1] * H(k)
 \\        + f[2] * H(k+1)
 \\        + f[3] * H(k+2)
 \\        + ...
 \\ Probably need g denominator to be a divisor of h denominator,
 \\ (ie. h a superset of g) otherwise coefficients are an infinite
 \\ sequence.
 gf_in_terms_of(g,h) = {        \\ g in terms of h
   my(gdenv = [denominator(g)]);
   for(len=1,200,
       \\ print("len="len);
       my(hv = vector(len,i, gf_shift_down(h,i-1)),
          d = lcm(concat(gdenv,apply(denominator,hv))),
          hvnums = hv*d,
          gnum = g*d);
       \\ for(i=1,#hv,pol_ascending_print(hv[i])); print(d);
       poldegree(gnum) < len || next();
       vecmax(apply(poldegree,hvnums)) < len || next();
       my(m = matrix(len,len,i,j, polcoeff(hvnums[j],i-1)),
          v = vectorv(len,i, polcoeff(gnum,i-1)),
          f = iferr(matsolve(m,v),e,[]));
       if(f!=[],
          \\ print(sum(i=1,#f, f[i]*hv[i])); print(g);
          while(#f>0 && f[#f]==0, f=f[1..-2]);
          sum(i=1,#f, f[i]*hv[i]) == g || error("oops wrong sum");
          return(f)));
   error("no solution found");
 }

gp_inline__location="../c-curve.tex:613";
 poly_roots_radius_max(p) = {
   my(n = poldegree(p));
   1 + vecmax(vector(n-1,i, abs(polcoeff(p,i) / polcoeff(p,n))));
 }

gp_inline__location="../c-curve.tex:618";
 vector_differences(v) = vector(#v-1,i,v[i+1]-v[i]);
 poly_descartes_sign_rule(p) = {
   1/2*vecsum(abs(vector_differences(
         select(x->x!=0, apply(sign, Vec(p))))));
 }

gp_inline__location="../c-curve.tex:627";
 vector_contains(v,elem) = for(i=1,#v,if(v[i]==elem,return(1))); 0;

gp_inline__location="../c-curve.tex:629";
 poly_reverse(p) = Polrev(Vec(p));

gp_inline__location="../c-curve.tex:631";
 \\ using floor() since it is an error on quads
 \\ whereas round() silently does nothing
 is_integer(n) = (n == floor(n));

gp_inline__location="../c-curve.tex:641";
 \\ v is a vector of fractional decimal digits high to low.
 \\ Eg. [1,2,3] is 123/1000 = 0.123
 \\ Return them as a rational like 12981/10^5.
 \\ If v is longer than the current floating point
 \\ precision() then use only that much of v.
 \\ "point" is how many digits before the decimal point.
 \\ "add" is an amount to add at the low end of the digits.
 fromdigits_to_precision(v,point=0, add=0) = {
   my(p = precision(1.0) - 4);
   if(#v>p, v = v[1..p]);
   (fromdigits(v)+add) / 10^(#v-point);
 }

gp_inline__location="../c-curve.tex:657";
 binary_padded(n,k) =  \
   my(v = binary(n)); \
   concat(vector(max(0,k-#v)),v);

gp_inline__location="../c-curve.tex:665";
 \\ n is an integer.  Return an integer whose decimal digits
 \\ are the bits of n.
 to_binary(n)=fromdigits(binary(n))*sign(n);

gp_inline__location="../c-curve.tex:671";
 sans_high_bit(n) = if(n==0,return(0)); fromdigits(binary(n)[^1],2);

gp_inline__location="../c-curve.tex:678";
 bit_differences(n) = sans_high_bit(bitxor(n, n>>1));

gp_inline__location="../c-curve.tex:682";
 floor_log2(n) = length(binary(n))-1;

gp_inline__location="../c-curve.tex:689";
 sqrt2 = geom_sqrt_as_quad(2);
 sqrt7i = geom_sqrt_as_quad(-7);

gp_inline__location="../c-curve.tex:694";
 \\ omega8 eighth root of unity, but times sqrt2
 w8_times_sqrt2 = 1+I;

gp_inline__location="../c-curve.tex:700";
 \\ table is a matrix where each row is state transitions.
 \\ States and digits are 0-based.
 \\ So table[state+1,digit+1] = new_state.
 \\ Return a vector where v[state] = generating function for how
 \\ many digit strings are in state after k steps.
 \\ initial is the starting state.  The k=0 terms of the gfs are
 \\ 1 in v[initial] and 0 everywhere else.
 state_table_to_gfs(table,initial=1) = {
   my(m=state_table_to_matrix(table,0),
      l=matsize(m)[1]);
   Vec(matsolve(matid(l)-m*x,vectorv(l,i,i==initial)));
 }
 \\ state numbers in table[] are for first state "offset",
 \\ and likewise the resulting matrix first row,column "offset"
 state_table_to_matrix(table,offset=1) = {
   my(num_states=matsize(table)[1],
      radix=matsize(table)[2],
      m=matrix(num_states,num_states));
   for(s=1,num_states,
       for(digit=1,radix,
           m[table[s,digit]+(1-offset), s]++));
   m;
 }

gp_inline__location="../c-curve.tex:728";
 b = 1+I;

gp_inline__location="../c-curve.tex:730";
 c_curve_points(k) = {
   my(v = [0,1], end=1);
   for(i=1,k,
       v = concat(v[^-1], apply((z)->end+z*I, v));
       end *= b);
   v;
 }
 c_curve_midpoints(k) = {
   my(v = [1/2], end=1);
   for(i=1,k,
       v = concat(v, apply((z)->end+z*I, v));
       end *= b);
   v;
 }

gp_inline__location="../c-curve.tex:755";
 \\ Return a vector of the midpoints of C curve k, shifted
 \\ so that the centre of gravity of the points is at the origin 0.
 \\ The direction is unchanged, so still first segment going East.
 c_curve_midpoints_at_centroid(k) = \
   vector_offset_to_centroid(c_curve_midpoints(k));

gp_inline__location="../c-curve.tex:761";
 c_curve_theta_points(k,t) = {
   my(v = [0,1], rotate=exp(I*(Pi-t)));
   for(i=1,k,
       v = concat(v[^-1], apply((z)->v[#v]+z*rotate, v)));
   v;
 }

gp_inline__location="../c-curve.tex:775";
 c_curve_theta_midpoints(k,t) = vector_midpoints(c_curve_theta_points(k,t));

gp_inline__location="../c-curve.tex:785";
 \\ Return a vector of the midpoints of C curve k, shifted
 \\ so that the centre of gravity of the points is at the origin 0.
 \\ The direction is unchanged, so still first segment going East.
 c_curve_theta_midpoints_at_centroid(k,t) = \
   vector_offset_to_centroid(c_curve_theta_midpoints(k,t));

gp_inline__location="../c-curve.tex:794";
 gf_cumulative(g) = 1/(1-variable(g)) * g;

gp_inline__location="../c-curve.tex:797";
 gOdds(x) = 2/(1-x)^2 - 1/(1-x);

gp_inline__location="../c-curve.tex:800";
 sum_over_parts(f,k) = f(k) + 2*sum(j=0,k-1, f(j));
 gf_sum_over_parts(g) = g + 2*x*gf_cumulative(g);
 gf_sum_over_parts2(g) = (1 + variable(g))/(1 - variable(g)) * g;

gp_inline__location="../c-curve.tex:814";
 part_from_whole(f,k) = f(k) + 2*sum(j=1,k, (-1)^j*f(k-j));

gp_inline__location="../c-curve.tex:820";
 gf_part_from_whole(g) = (1 - variable(g))/(1 + variable(g)) * g;

gp_inline__location="../c-curve.tex:826";
 sum_over_parts_overlap1(p,k) = p(k) + 2*sum(j=0,k-1, p(j)-1);
 gf_sum_over_parts_overlap1(g) = g + 2*x*gf_cumulative(g) - 2/(1-x)^2 + 2/(1-x);
 gf_sum_over_parts2_overlap1(g) = {
   (1 + variable(g))/(1 - variable(g)) * g
   - 2*variable(g) / (1 - variable(g))^2;
 }

gp_inline__location="../c-curve.tex:843";
 part_from_whole_overlap1(f,k) = {
   f(k)
   + 2*sum(j=1,k, (-1)^j*f(k-j))
   - (-1)^k*f(0)
   + vector_modulo([2,0],k);
 }
 part_from_whole_overlap1_by_recurrence(f,k) = {
   if(k==0, f(k),
            f(k)-f(k-1) - part_from_whole_overlap1(f,k-1) + 2);
 }

gp_inline__location="../c-curve.tex:856";
 \\ return true if z is an Gaussian integer
 z_is_integer(z) = is_integer(real(z)) && is_integer(imag(z));

gp_inline__location="../c-curve.tex:859";
 z_mod_b(z) = (real(z) + imag(z)) % 2;

gp_inline__location="../c-curve.tex:1497";
 CountOneBits(n) = vecsum(binary(n));
 dir(n) = CountOneBits(n);

gp_inline__location="../c-curve.tex:1502";
 dir_samples = {[0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,1,2,2,3]};

gp_inline__location="../c-curve.tex:1510";
 A000120(n) = dir(n);

gp_inline__location="../c-curve.tex:1561";
 dir_mod(n) = dir(n) % 4;
 dir_mod_samples = {[0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,0,1,2,2,3]};

gp_inline__location="../c-curve.tex:1584";
 \\ dir mod 4 as 1,4,3,2
 A106147(n) = (-hammingweight(n))%4 + 1;
 A106147_state_table = {[2,1;
                         3,2;
                         4,3;
                         1,4]};

gp_inline__location="../c-curve.tex:1616";
 dir_mod2(n) = dir(n) % 2;
 dir_is_horizontal(n) = dir_mod2(n) == 0;
 dir_is_vertical(n) = dir_mod2(n) == 1;
 dir_mod2_samples = {[0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0,1,0,0,1]};

gp_inline__location="../c-curve.tex:1689";
 turn(n) = 1 - CountLowZeros(n);
 CountLowZeros(n) = {
   n>=1 || error("CountLowZeros() is for n>=1");
   valuation(n,2);
 }

gp_inline__location="../c-curve.tex:1696";
 turn_samples = {[1,0,1,-1,1,0,1,-2,1,0,1,-1,1,0,1,-3]};

gp_inline__location="../c-curve.tex:1711";
 CountLowZeros_samples = {[0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4]};

gp_inline__location="../c-curve.tex:1718";
 A007814(n) = CountLowZeros(n);

gp_inline__location="../c-curve.tex:1827";
 A001511(n) = CountLowZeros(n) + 1;

gp_inline__location="../c-curve.tex:1835";
 A094267(n) = valuation(n+2,2) - valuation(n+1,2);

gp_inline__location="../c-curve.tex:1848";
 A011371(n) = n - hammingweight(n);

gp_inline__location="../c-curve.tex:1855";
 \\ 0-->0, 1-->110
 A079559(n) = {
   n>=0 || error("A079559() is for n>=0");
   n+=2; forstep(k=logint(n,2),2,-1, n+=bittest(n,k)); bittest(n,1);
 }
 A005187(n) = 2*n - hammingweight(n);

gp_inline__location="../c-curve.tex:1875";
 A079559_expansions(k) = {
   my(v=[1]);
   for(i=1,k, v=concat(apply(t->if(t==0,[0],[1,1,0]),v)));
   v;
 }

gp_inline__location="../c-curve.tex:1899";
 A327491(n) = if(n==0,0, n%2,2, valuation(n,2));

gp_inline__location="../c-curve.tex:1911";
 \\ like code by Peter L in A327492
 \\ hammingweight excluding lowest bit of n
 A327492(n) = 2*n - hammingweight(n) + n%2;

gp_inline__location="../c-curve.tex:1936";
 CountLowOnes(n) = {
   n>=0 || error("CountLowOnes() is for n>=0");
   valuation(n+1,2);
 }
 next_turn(n) = 1 - CountLowOnes(n);

gp_inline__location="../c-curve.tex:1992";
 turn_mod4(n) = turn(n) % 4;

gp_inline__location="../c-curve.tex:1996";
 turn_mod4_samples = {[1,0,1,3,1,0,1,2,1,0,1,3,1,0,1,1,1,0,1,3,1,0]};

gp_inline__location="../c-curve.tex:2042";
 A092412(n) = (CountLowZeros(n) + 1) % 4;

gp_inline__location="../c-curve.tex:2050";
 A346070(n) = CountLowZeros(n) % 4;

gp_inline__location="../c-curve.tex:2076";
 A035263(n) = {
   n>=1 || error("A035263() is for n>=1");
   1 - valuation(n,2)%2;
 }

gp_inline__location="../c-curve.tex:2086";
 A346070_counts(n) = \
   my(c=vector(4)); for(i=1,n, c[A346070(i)+1]++); c;

gp_inline__location="../c-curve.tex:2100";
 is_8421(v) = v[1]==2*v[2] && v[2]==2*v[3] && v[3]==2*v[4];

gp_inline__location="../c-curve.tex:2132";
 turn_morphism_subst(v) = {
   concat(apply(t->if(t==0,[1,3],
                      t==1,[1,0],
                      t==2,[1,1],
                      t==3,[1,2],
                      error()), v));
 }

gp_inline__location="../c-curve.tex:2151";
 turn_run_length(j) = if(j%14==13,3,1);

gp_inline__location="../c-curve.tex:2165";
 turn_insert = {101310121013101};

gp_inline__location="../c-curve.tex:2708";
 c_curve_point(n) = {
   my(v=binary(n), d=0, ret=0);
   for(i=1,#v,  /* high to low */
       ret += b^(#v-i) * v[i] * I^d;
       d += v[i]);
   ret;
 }

gp_inline__location="../c-curve.tex:2718";
 vector_sans_initial(v,n) = if(n<#v, v[1+n .. #v], []);
 c_curve_point5(n) = {
   my(v=Vecrev(binary_padded(n,5)));  \\ low to high
   sum(i=1,#v,
       b^(i-1) * v[i] * I^vecsum(vector_sans_initial(v,i)));
 }

gp_inline__location="../c-curve.tex:2734";
 point_samples = {[0,1,1+I,1+2*I,2*I,3*I,-1+3*I,-2+3*I,-2+2*I]};

gp_inline__location="../c-curve.tex:2782";
 n_to_bit_positions(n) = {  \\ high to low
   my(v = binary(n));
   apply(k->#v-k, Vec(select(bit->bit!=0, v, 1)));
 }
 bit_positions_to_n(v) = sum(i=1,#v, 2^v[i]);
 c_curve_point_by_t(n) = {
   my(v=n_to_bit_positions(n));
   sum(i=1,#v, b^v[i] * I^(i-1));
 }

gp_inline__location="../c-curve.tex:2812";
 c_curve_point_high_to_low(n) = {
   my(v = binary(n),
      k = #v-1,
      nk1 = fromdigits(v[^1],2));
   b^k*v[1] + c_curve_point(nk1) * I^v[1];
 }

gp_inline__location="../c-curve.tex:2881";
 \\ A332251
 point_x_compact(n) = {
   my(v=binary(n),s=1);
   for(i=2,#v, if(v[i],v[i]=(s*=I)));
   real(subst(Pol(v),'x,I+1));
 }
 point_f_compact(n) = {
   my(v=binary(n),s=1);
   for(i=2,#v, if(v[i],v[i]=(s*=I)));
   subst(Pol(v),'x,I+1);
 }

gp_inline__location="../c-curve.tex:3000";
 c_curve_unpoint_end_z = [ 0, 1, -1, I, -I ];
 c_curve_unpoint_z_is_end(z) = \
   vector_contains(c_curve_unpoint_end_z, z);
 c_curve_unpoint(z,d) = {
   my(v=List([]));  \\ bits low to high
   while(! c_curve_unpoint_z_is_end(z),
     my(bit = z_mod_b(z));
     /* print("at z="z"  d="d" I^d="I^d"  bit="bit" to "z-bit*I^d"  v="Vec(v)); */
     if(bit,
        d = d - 1;
        z = z - I^d;
        );
     z = z/b;
     if(! z_is_integer(z), error("oops, z=",z));
     listput(v,bit);
     );
   d%=4;
   /* print("end z=",z," d=",d," I^d=",I^d,if(z==I^d," (extra bit)","")," bits low to high "Vec(v)); */
   if(d==0 && z==0, fromdigits_low_to_high(Vec(v),2),
      d==1 && z==1, listput(v,1);
                    fromdigits_low_to_high(Vec(v),2),
      -1);
 }

gp_inline__location="../c-curve.tex:3048";
 z_count_visits(z) = sum(d=0,3, c_curve_unpoint(z,d) >= 0);

gp_inline__location="../c-curve.tex:3138";
 unpoint_iterate(v) = {
   my(z=v[1], d=v[2]);
   if(z_mod_b(z), d=(d-1)%4; z=z-I^d); z=z/b; [z,d];
 }
 unpoint_iterations(n,v) = {
   my(result=vector(n));
   for(i=1,n, v = unpoint_iterate(v);
              result[i] = v);
   result;
 }

gp_inline__location="../c-curve.tex:3170";
 unpoint_reaches_zero(v) = {
   for(i=1,100, v=unpoint_iterate(v);
                if(v[1]==0, return(1)));
   0;
 }

gp_inline__location="../c-curve.tex:3289";
 \\ Sort like vecsort(), but act only on the non-negative elements
 \\ v[i]>=0.  Any negatives v[i]<0 are unchanged.
 vecsort_nonnegs(v,cmpf=cmp,flags=0) = {
   my(nonnegs=select(elem->elem>=0,v));
   nonnegs=vecsort(nonnegs,cmpf,flags);
   my(p=1);
   for(i=1,#v,
       if(v[i]>=0, v[i]=nonnegs[p];p++));
   v;
 }

gp_inline__location="../c-curve.tex:3303";
 c_curve_unpoint_list(z) = vector(4,d,d--; c_curve_unpoint(z,d));
 unpoint_orders_n_list = {[
   11, 3, 235, 7, 1, 117, 369, 3003, 2799, 351, 119, 3455, 14081,
   14421, 375, 0, 23, 763, 3701, 1399, 113, 11233, 6001, 30399,
   2989, 1499, 751, 5987, 95583, 3671, 23953, 191, 95, 3651, 11195,
   55, 2999, 1527, 5975, 5997, 6005, 24005, 11181, 1727, 6913,
   11955, 3041, 875, 3505, 13983, 23297, 6907, 1835, 189, 11951,
   22359, 5823, 13805, 14015, 27637, 23291, 56065, 47799, 23927
 ]};

gp_inline__location="../c-curve.tex:3317";
 { my(seek=Map());
   for(i=0,4!-1,
       my(perm=Vec(numtoperm(4,i)));
       forvec(mask=[[0,1],[0,1],[0,1],[0,1]],
          my(v=perm);
          for(j=1,#mask, if(mask[j],v[j]=-1));
          v=vecsort_nonnegs(v,lex,1);
          mapput(seek,v,1)));
   mapdelete(seek,[-1,-1,-1,-1]);
   unpoint_orders_seek_list=vecsort(Mat(seek)[,1]);
 }

gp_inline__location="../c-curve.tex:3428";
 x_y_to_s_m(xy) = my(x=xy[1],y=xy[2]); [x+y, y-x];
 s_m_to_x_y(sm) = my(s=sm[1],m=sm[2]); [(s-m)/2, (s+m)/2];

gp_inline__location="../c-curve.tex:3546";
 Skd_formula(k,d) = 1/4*(2^k + s(k - 2*d) * 2^floor(k/2));
 Skd(k,d) = {
   k>=0 || error("Skd() is for k>=0");
   if(k==0, d%4==0, Skd_formula(k,d));
 }

gp_inline__location="../c-curve.tex:3561";
 Skd_by_b_formula(k,d) = 1/4*(2^k + b^k*(-I)^d + conj(b)^k*(I)^d);

gp_inline__location="../c-curve.tex:3573";
 Skd_by_norm_formula(k,d) = 1/4*( norm(b^k + I^d) - 1);

gp_inline__location="../c-curve.tex:3584";
 s_vector = {[2,2,0,-2,-2,-2,0,2]};
 s(m) = vector_modulo(s_vector,m);

gp_inline__location="../c-curve.tex:3604";
 S0_samples = {[1,1,1,1,2,6,16,36,72,136,256]};

gp_inline__location="../c-curve.tex:3615";
 S1_samples = {[0,1,2,3,4,6,12,28,64,136,272]};

gp_inline__location="../c-curve.tex:3626";
 S2_samples = {[0,0,1,3,6,10,16,28,56,120,256]};

gp_inline__location="../c-curve.tex:3637";
 S3_samples = {[0,0,0,1,4,10,20,36,64,120,240]};

gp_inline__location="../c-curve.tex:3680";
 S_by_binomial(k,d) = sum(j=0,k, binomial(k,4*j+d));

gp_inline__location="../c-curve.tex:3697";
 s_by_b(m) = (b^m + conj(b)^m) / 2^floor(m/2);

gp_inline__location="../c-curve.tex:3703";
 Skd_by_s_by_b(k,d) = {
   if(k==0,return((d%4)==0));
   1/4*(2^k + 2^floor(k/2)*s_by_b(k-2*d));
 }

gp_inline__location="../c-curve.tex:3710";
 Skd_by_b_floor(k,d) = {
   if(k==0,return((d%4)==0));
   1/4*( 2^k + 2^(floor(k/2) - floor((k-2*d)/2))*(b^(k-2*d)
         + conj(b)^(k-2*d)) );
 }

gp_inline__location="../c-curve.tex:3718";
 Skd_by_sqrt2(k,d) = {
   if(k==0,return((d%4)==0));
   1/4*( 2^k + sqrt2^(2*floor(k/2) - 2*floor((k-2*d)/2))*(b^(k-2*d)
         + conj(b)^(k-2*d)) );
 }

gp_inline__location="../c-curve.tex:3725";
 Skd_by_b2d(k,d) = {
   if(k==0,return((d%4)==0));
   1/4*(2^k + 2^d* (b^(k-2*d) + conj(b)^(k-2*d)));
 }

gp_inline__location="../c-curve.tex:3732";
 Skd_by_b_I(k,d) = {
   if(k==0,return((d%4)==0));
   1/4*(2^k + b^k*(-I)^d + conj(b)^k*(I)^d);
 }

gp_inline__location="../c-curve.tex:3739";
 nearly(x) = my(r=round(x)); if(abs(x-r) < 1e-12, r, x);
 S0_by_Cournot(k) = if(k==0,1, nearly(1/4*2^k + 1/2*sqrt2^k*cos(Pi/4*k)));
 S1_by_Cournot(k) = if(k==0,0, nearly(1/4*2^k + 1/2*sqrt2^k*sin(Pi/4*k)));
 S2_by_Cournot(k) = if(k==0,0, nearly(1/4*2^k - 1/2*sqrt2^k*cos(Pi/4*k)));
 S3_by_Cournot(k) = if(k==0,0, nearly(1/4*2^k - 1/2*sqrt2^k*sin(Pi/4*k)));

gp_inline__location="../c-curve.tex:3759";
 S0_by_Ramus(k) = nearly(1/4*2^k + 1/2*sqrt(2)^k * cos(1/4*Pi*k));
 S1_by_Ramus(k) = nearly(1/4*2^k + 1/2*sqrt(2)^k * sin(1/4*Pi*k));
 S2_by_Ramus(k) = nearly(1/4*2^k - 1/2*sqrt(2)^k * cos(1/4*Pi*k));
 S3_by_Ramus(k) = nearly(1/4*2^k - 1/2*sqrt(2)^k * sin(1/4*Pi*k));

gp_inline__location="../c-curve.tex:3796";
 SN(n,d) = {
   1/4 * (n
          + 2*real((-I)^d * c_curve_point(n))
          - if(n%2==1, (-1)^(d+dir(n)))
         );
 }
 SN_by_count(n,d) = d%=4; sum(j=0,n-1, dir(j)%4==d);

gp_inline__location="../c-curve.tex:3808";
 SN0_samples = {[0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2]};

gp_inline__location="../c-curve.tex:3817";
 SN1_samples = {[0,0,1,2,2,3,3,3,3,4,4,4,4,4,4,4,4]};

gp_inline__location="../c-curve.tex:3825";
 SN2_samples = {[0,0,0,0,1,1,2,3,3,3,4,5,5,6,6,6,6]};

gp_inline__location="../c-curve.tex:3834";
 SN3_samples = {[0,0,0,0,0,0,0,0,1,1,1,1,2,2,3,4,4]};

gp_inline__location="../c-curve.tex:3902";
 \\ Return a vector of descending powers of 2 which sum to n.
 \\ So v=[k1,k2,...,kt] with k1 > k2 > ... > kt >= 0
 \\ and n = 2^k1 + 2^k2 + ... + 2^kt.
 n_to_pow2_list(n) = \
   my(v=binary(n)); apply(i->#v-i, Vec(select(bit->bit,v,1)));

gp_inline__location="../c-curve.tex:3917";
 SN_by_Skd(n,d) = {
   my(v=n_to_pow2_list(n));
   sum(i=1,#v, Skd(v[i],d-(i-1)));
 }

gp_inline__location="../c-curve.tex:4054";
 P1(k) = -I*(b^k - I^floor(k/2));

gp_inline__location="../c-curve.tex:4057";
 P1dash(k) = b^k - I^k*conj(P1(k));

gp_inline__location="../c-curve.tex:4064";
 P2(k) = P1(k-1);

gp_inline__location="../c-curve.tex:4067";
 P2dash(k) = b^k - I^k*conj(P2(k));

gp_inline__location="../c-curve.tex:4070";
 P3(k) = P1(k-2);

gp_inline__location="../c-curve.tex:4073";
 P3dash(k) = b^k - I^k*conj(P3(k));

gp_inline__location="../c-curve.tex:4076";
 P4(k) = P1(k-3);

gp_inline__location="../c-curve.tex:4079";
 P4dash(k) = b^k - I^k*conj(P4(k));

gp_inline__location="../c-curve.tex:4082";
 hull(k) = {
   if(k==0, return([0, 1 ]));
   if(k==1, return([0, 1, 1+I ]));
   if(k==2, return([0, 1, 1+2*I, 2*I]));
   if(k==3, return([0, 1, 1+2*I, 3*I, -2+3*I, -2+2*I]));
   if(k==4, return([   1, 1+2*I, 3*I, -4+3*I, -5+2*I, -5]));
   [ P4(k), P3(k), P2(k), P1(k),
     P1dash(k),P2dash(k),P3dash(k),P4dash(k)];
 }

gp_inline__location="../c-curve.tex:4334";
 P2a(k) = b^k + I*P2(k);

gp_inline__location="../c-curve.tex:4339";
 P1_limit = -I;
 P2_limit = -I/b;
 P3_limit = -I/b^2;
 P4_limit = -I/b^3;
 P1dash_limit = 1 - conj(P1_limit);
 P2dash_limit = 1 - conj(P2_limit);
 P3dash_limit = 1 - conj(P3_limit);
 P4dash_limit = 1 - conj(P4_limit);

gp_inline__location="../c-curve.tex:4480";
 HB_formula(k) = (7/2+3/2*sqrt2) * sqrt2^k  - 4*sqrt2;
 HB(k) = if(k==0,2, k==1,2+sqrt2, k==2,6, HB_formula(k));

gp_inline__location="../c-curve.tex:4484";
 HB_by_polygon(k) = geom_length_of_polygon(hull(k));

gp_inline__location="../c-curve.tex:4487";
 HB_by_floor(k) = {
   vector_modulo([7/2+3/2*sqrt2, 3+7/2*sqrt2],k)*2^floor(k/2)
   - 4*sqrt2;
 }

gp_inline__location="../c-curve.tex:4516";
 HB_limit = 7/2 + 3/2*sqrt2;
 HB_limit_sample = {(5621320/1000000)};

gp_inline__location="../c-curve.tex:4537";
 HBrat_initial = {[2,2,6]};

gp_inline__location="../c-curve.tex:4547";
 HBrat_formula(k) = 1/2 * vector_modulo([7, 6],k) * 2^floor(k/2);
 HBrat(k)=if(k<#HBrat_initial,HBrat_initial[k+1], HBrat_formula(k));

gp_inline__location="../c-curve.tex:4552";
 HBrat_samples = {[2,2,6,6,14,12,28,24,56,48,112]};

gp_inline__location="../c-curve.tex:4563";
 HBsqrt_initial = {[0,1,0,3]};

gp_inline__location="../c-curve.tex:4570";
 HBsqrt_formula(k) = HBrat(k-1) - 4;
 HBsqrt(k) = if(k<#HBsqrt_initial,HBsqrt_initial[k+1], \
                HBsqrt_formula(k));

gp_inline__location="../c-curve.tex:4577";
 HBsqrt_samples = {[0,1,0,3,2,10,8,24,20,52,44]};

gp_inline__location="../c-curve.tex:4619";
 HA_formula(k) = {
   35/16 * 2^k
   - 1/2*vector_modulo([10,13],k) * 2^floor(k/2)
   + 2;
 }
 HA(k) = if(k==0,0, k==1,1/2, k==2,2, HA_formula(k));

gp_inline__location="../c-curve.tex:4629";
 HA_samples = {[0,1/2,2,(13)/(2),17,46,102,230,482,1018]};

gp_inline__location="../c-curve.tex:4661";
 MR_formula(k)= 5/2*2^k -13/2*2^floor(k/2) +vector_modulo([4,2],k);
 MR(k) = if(k==0,0, k==1,1, k==2,2, MR_formula(k));

gp_inline__location="../c-curve.tex:4668";
 MR_samples = {[0,1,2,9,18,56,112,270,540,1178]};

gp_inline__location="../c-curve.tex:4765";
 MR_width(k) = \
   sqrt2^k*real( w8_times_sqrt2^(-k) * (P2dash(k) - P2(k)) );

gp_inline__location="../c-curve.tex:4769";
 MR_height(k) = \
   sqrt2^k*imag( w8_times_sqrt2^(-k) * (P4(k) - P1(k)) );

gp_inline__location="../c-curve.tex:4773";
 MR_by_sides(k) = MR_width(k) * MR_height(k);

gp_inline__location="../c-curve.tex:4889";
 MS_side(k) = \
   sqrt2^(k+1)*real( w8_times_sqrt2^(-(k+1)) * (P4dash(k)-P1(k)) );
 MS_by_sides(k) = MS_side(k)^2;

gp_inline__location="../c-curve.tex:4904";
 MS_side_by_powers(k) = \
   5/4*sqrt2*sqrt2^k - vector_modulo([sqrt2,2],k);

gp_inline__location="../c-curve.tex:4922";
 MS_formula(k)= 25/8*2^k - 5*2^ceil(k/2) + vector_modulo([2,4],k);
 MS(k) = if(k==0,1/2, k==1,1, k==2,9/2, MS_formula(k));

gp_inline__location="../c-curve.tex:4931";
 MS_samples = {[(1)/(2),1,(9)/(2),9,32,64,162,324,722,1444]};

gp_inline__location="../c-curve.tex:4954";
 MS_sub_MR(k) = MS(k) - MR(k);

gp_inline__location="../c-curve.tex:4967";
 MS_sub_MR_formula(k) = {
   5/8*2^k
   + vector_modulo([3/2, -7/2],k)*2^floor(k/2)
   + vector_modulo([-2,2],k);
 }

gp_inline__location="../c-curve.tex:5000";
 gen_right_boundary_diag_RStotal = {[ 1,2,4,6,10,14,22,30,46,62,94,
   126,190,254,382,510,766,1022,1534,2046,3070,4094 ]};
 gen_right_boundary_diag_RZtotal = {[ 0,0,0,2,4,10,16,30,44,74,104,
   166,228,354,480,734,988,1498,2008,3030,4052,6098 ]};

gp_inline__location="../c-curve.tex:5099";
 R(k) = {
   if(k<0, error("R() is for k>=0, got ",k));
   vector_modulo([7,10],k) * 2^floor(k/2)  - 2*k - 6;
 }
 R_even(h) = R(2*h);  /* starting h=0,1,2.. for k=1,3,5.. */
 R_odd(h) = R(2*h+1);  /* starting h=0,1,2.. for k=1,3,5.. */

gp_inline__location="../c-curve.tex:5108";
 R_samples = {[1,2,4,8,14,24,38,60,90,136,198,292]};

gp_inline__location="../c-curve.tex:5146";
 g_two_k_minus_6(x) = 4*1/(1-x) + 2*1/(1-x)^2; /* 2*k-6 */
 g_two_power_repeated(x) = (1+x)/(1-2*x^2);
 gR(x) = (7+10*x)/(1-2*x^2) - 4/(1-x) - 2/(1-x)^2;

gp_inline__location="../c-curve.tex:5171";
 Manhattan(z) = abs(real(z)) + abs(imag(z));

gp_inline__location="../c-curve.tex:5175";
 Rpart_by_P(k) = Manhattan(P1dash(k)-P1(k));

gp_inline__location="../c-curve.tex:5182";
 Rpart(k) = {
   if(k<0, error("Rpart() is for k>=0, got ",k));
   2^ceil(k/2) - vector_modulo([0,2],k);
 }

gp_inline__location="../c-curve.tex:5189";
 gRpart(x) = (1 + 2*x)/(1 - 2*x^2) + 1/(1+x) - 1/(1-x);

gp_inline__location="../c-curve.tex:5194";
 Rpart_samples = {[1,0,2,2,4,6,8,14,16,30,32,62]};

gp_inline__location="../c-curve.tex:5211";
 dRpart(k) = Rpart(k+1) - Rpart(k);

gp_inline__location="../c-curve.tex:5304";
 R_by_parts(k) = Rpart(k) + 2*sum(j=0,k-1,Rpart(j));

gp_inline__location="../c-curve.tex:5309";
 Rpart_cumulative(k) = sum(j=0,k,Rpart(j));
 gRpart_cumulative(x) = gf_cumulative(gRpart(x));

gp_inline__location="../c-curve.tex:5519";
 Rpart_by_recurrence(k) = {
   if(k==0, return(1));
   if(k%2==0,   Rpart(k-1) + 2,
              2*Rpart(k-1) - 2);
 }

gp_inline__location="../c-curve.tex:5561";
 RgeomPart(k) = sqrt2^k - vector_modulo([0,sqrt2],k);

gp_inline__location="../c-curve.tex:5583";
 Rgeom_by_parts(k) = RgeomPart(k) + 2*sum(j=0,k-1, RgeomPart(j));

gp_inline__location="../c-curve.tex:5593";
 Rgeom(k) = (3+2*sqrt2)*sqrt2^k - sqrt2*k - 2 - 2*sqrt2;

gp_inline__location="../c-curve.tex:5618";
 Rgeom_limit = 3 + 2*sqrt2;

gp_inline__location="../c-curve.tex:5622";
 Rgeom_limit_sample = {(5828427/1000000)};

gp_inline__location="../c-curve.tex:5703";
 Rgeom_sub_HB_formula(k) = \
   1/2*(sqrt2-1)*sqrt2^k - sqrt2*k - 2 + 2*sqrt2;
 Rgeom_sub_HB(k) = if(k==0,-1, k==1,-sqrt2, k==2,-2, Rgeom_sub_HB_formula(k));

gp_inline__location="../c-curve.tex:5731";
 Rgeom_by_Rat(k) = RgeomRat(k) + sqrt2*RgeomSqrt(k);

gp_inline__location="../c-curve.tex:5735";
 R_by_Rat(k) = RgeomRat(k) + 2*RgeomSqrt(k);

gp_inline__location="../c-curve.tex:5743";
 RgeomRat_by_parts(k) = vector_modulo([1,0],k)*Rpart(k) \
    + 2*sum(j=0,k-1, vector_modulo([1,0],j)*Rpart(j));

gp_inline__location="../c-curve.tex:5752";
 RgeomRat(k) = vector_modulo([3,4],k)*2^floor(k/2) - 2;

gp_inline__location="../c-curve.tex:5757";
 RgeomRat_samples = {[1,2,4,6,10,14,22,30,46,62,94]};

gp_inline__location="../c-curve.tex:5778";
 RgeomSqrt_by_parts(k) = vector_modulo([0,1/2],k)*Rpart(k) \
        + 2*sum(j=0,k-1, vector_modulo([0,1/2],j)*Rpart(j));

gp_inline__location="../c-curve.tex:5786";
 RgeomSqrt(k) = vector_modulo([2,3],k)*2^floor(k/2) - k - 2;

gp_inline__location="../c-curve.tex:5793";
 RgeomSqrt_samples = {[0,0,0,1,2,5,8,15,22,37,52]};

gp_inline__location="../c-curve.tex:5820";
 dR(k) = R(k+1) - R(k);

gp_inline__location="../c-curve.tex:5834";
 RQpart(k) = 2^floor(k/2) - vector_modulo([0,1],k);

gp_inline__location="../c-curve.tex:5838";
 RQpart_samples = {[1,0,2,1,4,3,8,7,16,15,32]};

gp_inline__location="../c-curve.tex:5854";
 RQ(k) = RgeomRat(k) + RgeomSqrt(k);

gp_inline__location="../c-curve.tex:5856";
 RQ_multiply = {[5,7]};

gp_inline__location="../c-curve.tex:5866";
 RQ_samples = {[1,2,4,7,12,19,30,45,68,99,146]};

gp_inline__location="../c-curve.tex:5914";
 PartN_vector = {[-1,1]};

gp_inline__location="../c-curve.tex:5918";
 PartN(k) = 1/3*(2^k + vector_modulo(PartN_vector,k));

gp_inline__location="../c-curve.tex:5920";
 PartN_samples = {[0,1,1,3,5,11,21,43,85,171,341]};

gp_inline__location="../c-curve.tex:5932";
 PartN_by_binary(k) = {
   my(v = if(k==0,[], vector(k-1,i, i%2)));
   if(#v, v[#v]=1);
   fromdigits(v,2);
 }

gp_inline__location="../c-curve.tex:5952";
 PartSegments_increment_vector = {[2,-2]};

gp_inline__location="../c-curve.tex:5962";
 PartSegments_vector = {[2,-2]};

gp_inline__location="../c-curve.tex:5966";
 PartSegments(k) = 1/3*(2^k + vector_modulo(PartSegments_vector,k));

gp_inline__location="../c-curve.tex:5968";
 PartSegments_samples = {[1,0,2,2,6,10,22,42,86,170,342]};

gp_inline__location="../c-curve.tex:6112";
 vector_strip_trailing_2s(v) = {
   forstep(i=#v,1,-1, if(v[i]!=2,return(v[1..i])));
   [];  \\ no non-2 at all
 }

gp_inline__location="../c-curve.tex:6123";
 vector_delete_initial_run(v) = \
   for(i=2,#v, if(v[i]!=v[1], return(v[i..-1]))); [];

gp_inline__location="../c-curve.tex:6172";
 Rpred_k(k,n) = {
   my(v=binary_padded(n,k));
   if(#v>k, error("Rpred_k() n bigger than 2^k"));
   if(k==0, return(1));
   v=vector_delete_initial_run(v);
   my(prev_pos=-1);
   for(pos=1,#v-1,
       if(v[pos]==v[pos+1],
          if(prev_pos!=-1,
             if((pos-prev_pos)%2==1, return(0)));
          prev_pos=pos));
   1;
 }

gp_inline__location="../c-curve.tex:6190";
 Rpred_k_by_alternate(k,n) = {
   my(v=binary_padded(n,k));
   if(#v>k, error("Rpred_k_by_alternate() n bigger than 2^k"));
   if(k==0, return(1));
   v=vector_delete_initial_run(v);
   my(prev_pair=-1);
   for(pos=1,#v-1,
       if(v[pos]==v[pos+1],
          if(v[pos]==prev_pair, return(0));
          prev_pair=v[pos]));
   1;
 }

gp_inline__location="../c-curve.tex:6205";
 Rpred_k_by_pair_distance(k,n) = {
   my(v=binary_padded(n,k));
   if(#v>k, error("Rpred_k_by_pair_distance() n bigger than 2^k"));
   v=vector_delete_initial_run(v);
   my(previous=-1);
   for(i=1,#v-1,
       if(v[i]==v[i+1],
          if(previous > 0 && (i-previous) % 2 == 1, return(0));
          previous = i));
   1;
 }

gp_inline__location="../c-curve.tex:6227";
 RpredInf(n) = Rpred_k(length(binary(n))+1,n);
 NonRpredInf(n) = !RpredInf(n);

gp_inline__location="../c-curve.tex:6232";
 RpredInf_samples = {[1,1,1,1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,1,1]};

gp_inline__location="../c-curve.tex:6245";
 RpredInf_at_samples = {[0,1,2,3,4,5,6,9,10,11,12,13,18]};

gp_inline__location="../c-curve.tex:6255";
 NonRpredInf_at_samples = {[7,8,14,15,16,17,23,24,27,28,29]};

gp_inline__location="../c-curve.tex:6653";
 Rpred_state_table = {[ 1,2;    /* r0 */
                        1,3;    /* r1 */
                        4,2;    /* r2 */
                        4,5;    /* r3 */
                        6,3;    /* r4 */
                        7,8;    /* r5 */
                        8,7;    /* r6 */
                        6,5;    /* r7 */
                        8,8 ]}; /* 8=non */
 Rpred_transition(state,digit) = Rpred_state_table[state+1,digit+1];

gp_inline__location="../c-curve.tex:6747";
 Rpred_transition_run(s,v) = for(i=1,#v, s=Rpred_transition(s,v[i])); s;
 Rpred_k_state_by_high_to_low(k,n) = Rpred_transition_run(0, binary_padded(n,k));
 Rpred_k_by_high_to_low(k,n) = Rpred_k_state_by_high_to_low(k,n)!=8;

gp_inline__location="../c-curve.tex:6787";
 is_binary_run_3_or_more(n) = \
   while(n,if(n%8==7||n%8==0,return(1)); n\=2); 0;

gp_inline__location="../c-curve.tex:6794";
 Run_3_bits_samples = {[7,8,14,15,16,17,23,24,28,29]};

gp_inline__location="../c-curve.tex:6811";
 NonRpredInf_boundary_non_run3(n) = !RpredInf(n) && !is_binary_run_3_or_more(n);

gp_inline__location="../c-curve.tex:6824";
 NonRpredInf_boundary_non_run3_samples = {[27,36,54,73,91,100,107,108,109,146,147,148,155]};

gp_inline__location="../c-curve.tex:6834";
 vector_bitxor_adjacent(v) = vector(#v-1,i, bitxor(v[i],v[i+1]));

gp_inline__location="../c-curve.tex:6836";
 vector_any_zeros_odd_apart(v) = {
   my(i=1);
   while(i<=#v && v[i], i++);  \\ skip 1s
   while(i<=#v,
     if(v[i]!=0, error("oops v[i] zero"));
     i++;  \\ skip 0
     my(run=0);
     while(i<=#v && v[i], i++;run++);  \\ skip 1s
     if(i<=#v && run%2==0, return(1)));
   0;
 }

gp_inline__location="../c-curve.tex:6853";
 any_zeros_even_gap(n) = vector_any_zeros_odd_apart(binary(n));

gp_inline__location="../c-curve.tex:6859";
 vector_delete_initial_zeros(v) = {
   my(i=1);
   while(i<=#v && v[i]==0, i++);  \\ skip 0s
   if(i>#v,[], v[i..-1]);
 }

gp_inline__location="../c-curve.tex:6872";
 Rpred_k_by_differences(n,k) = {
   my(v=binary_padded(n,k));
   v = vector_bitxor_adjacent(v);
   v = vector_delete_initial_zeros(v);
   ! vector_any_zeros_odd_apart(v);
 }

gp_inline__location="../c-curve.tex:6983";
 Rpred_total_by_sum(k) = \
   2*(1     \
      + k-1 \
      + sum(s=0,k-2, (k-s-2)*2^floor(s/2)));

gp_inline__location="../c-curve.tex:6999";
 Rpred_r3(j) = 1/3*(4^j-1);
 Rpred_r4(j) = 2/3*(4^j-1);

gp_inline__location="../c-curve.tex:7004";
 RpredInf_state(n) = Rpred_transition_run(1,binary(n));

gp_inline__location="../c-curve.tex:7029";
 TopMiddlePre_formula(k) = 5/12*2^k - vector_modulo([2/3, 1/3],k);
 TopMiddlePre(k) = if(k<=1,0, TopMiddlePre_formula(k));

gp_inline__location="../c-curve.tex:7037";
 TopMiddlePre_bits(k) = if(k<2,return([])); \
   concat(1, vector(k-2,i, i%2 == 1));
 TopMiddlePre_by_binary(k) = fromdigits(TopMiddlePre_bits(k), 2);

gp_inline__location="../c-curve.tex:7046";
TopMiddlePre_samples = {[0,0,1,3,6,13,26,53,106,213,426]};

gp_inline__location="../c-curve.tex:7067";
 TopMiddlePost_formula(k) = 7/12*2^k - vector_modulo([1/3,2/3],k);
 TopMiddlePost(k) = if(k<=1,k, TopMiddlePost_formula(k));

gp_inline__location="../c-curve.tex:7075";
 TopMiddlePost_from_TopMiddlePre(k) = 2^k-1 - TopMiddlePre(k);

gp_inline__location="../c-curve.tex:7080";
 TopMiddlePost_bits(k) = \
   if(k==0,return([])); \
   if(k==1,return([1])); \
   concat([1,0], vector(k-2,i, i%2 == 0));
 TopMiddlePost_by_binary(k) = fromdigits(TopMiddlePost_bits(k), 2);

gp_inline__location="../c-curve.tex:7089";
TopMiddlePost_samples = {[0,1,2,4,9,18,37,74,149,298,597]};

gp_inline__location="../c-curve.tex:7111";
 \\ Return a vector of the midpoints of the segments on the right
 \\ boundary of C curve k.
 c_curve_R_midpoints(k) = {
   my(v=c_curve_midpoints(k),
      ret=List([]));
   for(n=1,#v, if(Rpred_k(k,n-1), listput(ret,v[n])));
   #ret==R(k) || error("oops got=",#ret," want R=",R(k));
   Vec(ret);
 }

gp_inline__location="../c-curve.tex:7121";
 Gray(n) = bitxor(n, n\2);

gp_inline__location="../c-curve.tex:7130";
 L_samples_generated = [1,2,4,8,16,32,64,124,202,308,450,648,918,1300,1826,2576,3614,5100,7162,10120,14230,20132,28338,40128,56526];
 Lpart_samples_generated = [1,0,2,2,6,10,22,38,40,66,76,122,148,234,292,458,580,906,1156,1802,2308,3594];

gp_inline__location="../c-curve.tex:7151";
 L_formula(k) = \
   1/4*vector_modulo([55,78],k)*2^floor(k/2) + 14*k - 130;
 L(k) = if(k<=5,2^k, L_formula(k));

gp_inline__location="../c-curve.tex:7158";
 gL(x) = {
   469/4 + 197/2*x + 157/2*x^2 + 57*x^3 + 35*x^4 + 14*x^5
   - 144 * 1/(1 - x)
   + 14 * 1/(1 - x)^2
   + (55/4 + 39/2*x)/(1 - 2*x^2);
 };

gp_inline__location="../c-curve.tex:7167";
 L_samples = {[1,2,4,8,16,32,64,124,202,308,450]};

gp_inline__location="../c-curve.tex:7175";
 dL(k) = L(k+1) - L(k);

gp_inline__location="../c-curve.tex:7183";
 Lpart_initial = {[1,0,2,2,6,10]};

gp_inline__location="../c-curve.tex:7189";
 Lpart_vector = {[9,14]};

gp_inline__location="../c-curve.tex:7197";
 Lpart_formula(k) = \
   1/4*vector_modulo(Lpart_vector,k) * 2^floor(k/2) \
   + vector_modulo([4,10],k);
 Lpart(k) = if(k<6,Lpart_initial[k+1], Lpart_formula(k));

gp_inline__location="../c-curve.tex:7204";
 gLpart(x) = {
      -21/4 - 27/2*x - 13/2*x^2 - 15*x^3 - 7*x^4 - 14*x^5
      + 7 * 1/(1 - x)
      - 3 * 1/(1 + x)
      + (9/4 + 7/2*x)/(1 - 2*x^2);
 }

gp_inline__location="../c-curve.tex:7216";
 Lpart_samples = {[1,0,2,2,6,10,22,38,40,66,76]};

gp_inline__location="../c-curve.tex:7224";
 dLpart(k) = Lpart(k+1) - Lpart(k);

gp_inline__location="../c-curve.tex:7510";
 Lcorner_by_recurrence(k) = {
   if(k<2,    error("Lcorner_by_recurrence() is for k>=2, got ",k),
      k==2,   6,
      k%2==0, Lcorner(k-1) + 2,
              2*Lcorner(k-1) - 4);
 }

gp_inline__location="../c-curve.tex:7522";
 Lcorner(k) = 2*2^ceil(k/2) + vector_modulo([2,0],k);

gp_inline__location="../c-curve.tex:7526";
 Lcorner_samples = {[6,8,10,16,18,32,34,64,66,128,130]};

gp_inline__location="../c-curve.tex:7542";
 gA228693(x) = (1+x-x^2-x^3-x^4) / ((1-x)*(1+x)*(1-2*x^2));

gp_inline__location="../c-curve.tex:7729";
 Lpart_end_vector = {[10,16]};
 Lpart_by_sum(k) = {
   my(total = Rpart(k-4) + 2*Rpart(k-5)
              + 2*vector_modulo(Lpart_end_vector,k));
   while(k-6 >= 2,
         total += 2*(Lcorner(k-6) + Rpart(k-7)); k-=2);
   total;
 }

gp_inline__location="../c-curve.tex:7822";
 degrees(a) = a*Pi/180;  /* degrees to radians */

gp_inline__location="../c-curve.tex:7832";
 slope(k) = if(k==0, -2+I, 2*slope(k-1) + 1+I);

gp_inline__location="../c-curve.tex:7844";
 P1a(k) = b^k + I*P1(k);
 P4a(k) = b^k + I*P4(k);

gp_inline__location="../c-curve.tex:7937";
 L_by_parts(k) = Lpart(k) + 2*sum(j=0,k-1,Lpart(j));

gp_inline__location="../c-curve.tex:7945";
 Lpart_sum(k) = sum(j=0,k,Lpart(j));
 gLpart_sum(x) = 1/(1-x) * gLpart(x);

gp_inline__location="../c-curve.tex:8087";
 LgeomCorner(k) = if(k%2==0, Lcorner(k), 1/2*sqrt2*Lcorner(k));

gp_inline__location="../c-curve.tex:8093";
 LgeomCorner_by_sqrt2(k) = 2*sqrt2^k + vector_modulo([2,0],k);

gp_inline__location="../c-curve.tex:8113";
 LgeomPart_end_vector = {[10,16]};
 LgeomPart_by_sum(k) = {
   my(total = RgeomPart(k-4) + 2*RgeomPart(k-5)
              + 2*vector_modulo(LgeomPart_end_vector,k));
   while(k-6 >= 2,
         total += 2*(LgeomCorner(k-6) + RgeomPart(k-7)); k-=2);
   total;
 }

gp_inline__location="../c-curve.tex:8134";
 LgeomPart(k) = {
   if(k<6, return(Lpart(k)));
   (5/4 + 1/2*sqrt2) * sqrt2^k
   + vector_modulo([2 - sqrt2, 0],k) * k
   + vector_modulo([2*sqrt2, 28 - 9*sqrt2],k);
 }

gp_inline__location="../c-curve.tex:8149";
 Lgeom_by_parts(k) = LgeomPart(k) + 2*sum(j=0,k-1, LgeomPart(j));

gp_inline__location="../c-curve.tex:8171";
 Lgeom_formula(k) = {
   (23/4 + 4*sqrt2)*sqrt2^k
   + (1 - 1/2*sqrt2)* k^2
   + (28 - 7*sqrt2)*k
   + 1/2*vector_modulo([-372 + 56*sqrt2, -374 + 57*sqrt2],k);
 }
 Lgeom(k) = if(k<=5, L(k), Lgeom_formula(k));

gp_inline__location="../c-curve.tex:8216";
 Lgeom_limit = 23/4 + 4*sqrt2;

gp_inline__location="../c-curve.tex:8219";
 Lgeom_limit_sample = {(11406854/1000000)};

gp_inline__location="../c-curve.tex:8230";
 LgeomPart_limit = 5/4 + 1/2*sqrt2;

gp_inline__location="../c-curve.tex:8233";
 LgeomPart_limit_sample = {(1957106/1000000)};

gp_inline__location="../c-curve.tex:8254";
 Lgeom_over_Rgeom_limit = Lgeom_limit / Rgeom_limit;

gp_inline__location="../c-curve.tex:8264";
 LgeomPart_limit_approx = 1/4 + 2*1/4*1/sqrt2 + 2*sum(i=0,100, 2/sqrt2^(2*i+6) + 1/sqrt2^(2*i+7));
 Lgeom_limit_approx = LgeomPart_limit_approx * (1 + 2*sum(i=1,100, 1/sqrt2^i));

gp_inline__location="../c-curve.tex:8273";
 \\     --- z+(1+I)*I^d
 \\   |    |
 \\   z ---> z+I^d
 square_z_d_to_LQ_lower_left(z,d) = {
   my(z2 = z + (1+I)*I^d);  \\ opposite corner
   min(real(z),real(z2)) + I*min(imag(z),imag(z2));
 }
 LQ_trace(z,d,z_end,lo,hi) = {
   my(debug=0,
      l=List([]), limit=hi-lo+2);
   if(debug,
      print("LQ_trace z="z" d="d" to z_end="z_end" range "lo" "hi));
   while(z!=z_end,
     d%=4;
     if(debug,print("at d="d" z="z));
     d++;  \\ try to follow to the left first
     my(seen=0);
     for(i=1,4,    \\ turn d perhaps
       for(i=0,1,  \\ forward or backward
         my(n=c_curve_unpoint(z,d));
         if(n>=lo && n<hi, seen++);
         z+=I^d; d+=2);
       if(seen,break());
       d--);  \\ try to the right
     seen || error("oops, next segment not seen");
     listput(l, square_z_d_to_LQ_lower_left(z,d));
     #l<=limit || error("list too long ",#l," ",Vec(l));
     z+=I^d);
   if(debug,
      print("final list ",Vec(l)));
   #Set(l);
 }
 LQ_by_trace(k) = LQ_trace(0,1,b^k, 0,2^k);
 LQpart_by_trace(k) = {
   my(lo=PartN(k), hi=PartN(k+1));
   LQ_trace(c_curve_point(lo),dir(lo),c_curve_point(hi),
            lo,hi);
 }

gp_inline__location="../c-curve.tex:8312";
 LcornerQ(k) = if(k%2==0, Lcorner(k)-1, Lcorner(k)/2);
 LQpart_end_vector = [6,8];
 LQpart_by_sum(k) = {
   my(total = RQpart(k-4) + 2*RQpart(k-5)
              + 2*vector_modulo(LQpart_end_vector,k));
   while(k-6 >= 2,
         total += 2*(LcornerQ(k-6) + RQpart(k-7)); k-=2);
   total;
 }

gp_inline__location="../c-curve.tex:8341";
 LQpart_initial = {[1,0,2,2,4,6]};

gp_inline__location="../c-curve.tex:8346";
 LQpart_multiply = {[7,9]};

gp_inline__location="../c-curve.tex:8352";
 LQpart_constant = {[0,3]};

gp_inline__location="../c-curve.tex:8356";
 LQpart_formula(k) = {
   1/4 * vector_modulo(LQpart_multiply,k)*2^floor(k/2)
   + vector_modulo(LQpart_constant,k);
 }
 LQpart(k) = if(k<=5, LQpart_initial[k+1], LQpart_formula(k));

gp_inline__location="../c-curve.tex:8368";
 LQpart_samples = {[1,0,2,2,4,6,14,21,28,39,56]};

gp_inline__location="../c-curve.tex:8386";
 LQ_by_parts(k) = LQpart(k) + 2 + 2*sum(j=0,k-1, LQpart(j) - 1);

gp_inline__location="../c-curve.tex:8395";
 LQ_initial = {[1,1,2,4,8,16]};

gp_inline__location="../c-curve.tex:8400";
 LQ_multiply = {[39,55]};

gp_inline__location="../c-curve.tex:8408";
 LQ_formula(k) = {
   1/4*vector_modulo(LQ_multiply,k)*2^floor(k/2)
   + k - 50;
 }
 LQ(k) = if(k<=5, LQ_initial[k+1], LQ_formula(k));

gp_inline__location="../c-curve.tex:8420";
 LQ_samples = {[1,1,2,4,8,16,34,67,114,179,272]};

gp_inline__location="../c-curve.tex:8445";
 B_initial = {[2,4,8,16,30,56]};

gp_inline__location="../c-curve.tex:8451";
 B_vector = {[83,118]};

gp_inline__location="../c-curve.tex:8459";
 B_formula(k) = 1/4*vector_modulo(B_vector,k)*2^floor(k/2) \
                + 12*k - 136;
 B(k) = if(k<=5,B_initial[k+1], B_formula(k));

gp_inline__location="../c-curve.tex:8468";
 B_samples = {[2,4,8,16,30,56,102,184,292,444,648]};

gp_inline__location="../c-curve.tex:8485";
 Bpart_initial = {[2,0,4,4,10,16]};

gp_inline__location="../c-curve.tex:8491";
 Bpart_multiply = {[13,22]};

gp_inline__location="../c-curve.tex:8497";
 Bpart_constant = {[4,8]};

gp_inline__location="../c-curve.tex:8502";
 Bpart_formula(k) = \
   1/4*vector_modulo(Bpart_multiply,k) * 2^floor(k/2) \
   + vector_modulo(Bpart_constant,k);
 Bpart(k) = if(k<=5,Bpart_initial[k+1], Bpart_formula(k));

gp_inline__location="../c-curve.tex:8512";
 Bpart_samples = {[2,0,4,4,10,16,30,52,56,96,108]};

gp_inline__location="../c-curve.tex:8527";
 BQ_initial = {[2,3,6,11,20,35]};

gp_inline__location="../c-curve.tex:8533";
 BQ_vector = {[59,83]};

gp_inline__location="../c-curve.tex:8541";
 BQ_formula(k) = 1/4*vector_modulo(BQ_vector,k)*2^floor(k/2) - 54;
 BQ(k) = if(k<=5,BQ_initial[k+1], BQ_formula(k));

gp_inline__location="../c-curve.tex:8549";
 BQ_samples = {[2,3,6,11,20,35,64,112,182,278,418]};

gp_inline__location="../c-curve.tex:8563";
 BQpart_by_sum(k) = RQpart(k) + LQpart(k);

gp_inline__location="../c-curve.tex:8567";
 BQpart_initial = {[2,0,4,3,8,9]};

gp_inline__location="../c-curve.tex:8573";
 BQpart_multiply = {[11,13]};

gp_inline__location="../c-curve.tex:8579";
 BQpart_constant = {[0,2]};

gp_inline__location="../c-curve.tex:8584";
 BQpart_formula(k) = \
   1/4*vector_modulo(BQpart_multiply,k) * 2^floor(k/2) \
   + vector_modulo(BQpart_constant,k);
 BQpart(k) = if(k<=5,BQpart_initial[k+1], BQpart_formula(k));

gp_inline__location="../c-curve.tex:8595";
 BQpart_samples = {[2,0,4,3,8,9,22,28,44,54,88]};

gp_inline__location="../c-curve.tex:8616";
 Bgeom_initial = [ 2, 4, 8, 14+sqrt2, 26+2*sqrt2, 46+5*sqrt2 ];

gp_inline__location="../c-curve.tex:8621";
 Bgeom_limit = 35/4 + 6*sqrt2;

gp_inline__location="../c-curve.tex:8633";
 Bgeom_formula(k) = {
   Bgeom_limit * sqrt2^k
   + (1 - 1/2*sqrt2) * k^2
   + (28 - 8*sqrt2) * k
   + vector_modulo([-188 + 26*sqrt2, -189 + 53/2*sqrt2],k);
 }
 Bgeom(k) = if(k<=5,Bgeom_initial[k+1], Bgeom_formula(k));

gp_inline__location="../c-curve.tex:8666";
 BgeomPart_initial = [2, 0, 4, 2 + sqrt2, 10, 10 + 3*sqrt2];

gp_inline__location="../c-curve.tex:8670";
 BgeomPart_limit = 9/4 + 1/2*sqrt2;

gp_inline__location="../c-curve.tex:8677";
 BgeomPart_formula(k) = {
   BgeomPart_limit * sqrt2^k
   + vector_modulo([2-sqrt2, 0],k) * k
   + vector_modulo([2*sqrt2, 28-10*sqrt2],k);
 }
 BgeomPart(k) =if(k<=5,BgeomPart_initial[k+1],BgeomPart_formula(k));

gp_inline__location="../c-curve.tex:8696";
 BgeomPart_samples = [2,0, 4,2+sqrt2, 10,10+3*sqrt2, 30,36+8*sqrt2];

gp_inline__location="../c-curve.tex:8713";
 \\ Return 1 if segment at complex z direction d mod 4 is traversed,
 \\ either forward or back, by an n in the range 0 <= n < limit.
 z_d_is_traversed_either(z,d,limit) = {
   for(i=0,1,
     my(n=c_curve_unpoint(z,d));
     if(n>=0 && n<limit, return(1));
     z+=I^d; d+=2);
   0;
 }
 \\ Return a vector of segment numbers which are the left boundary
 \\ of curve k.
 \\ Default bothways=1 returns segments both directions forward
 \\ or back on the boundary.
 \\ bothways=0 returns only one segment number of each traversal,
 \\ which means the number of segments is the L boundary length.
 Lpred_k_list_by_trace(k,bothways=1) = {
   my(debug=0,
      z=0,d=1,end=b^k,limit=2^k,
      l=List([]),m=Map());
   while(z!=end,
     d%=4;
     if(debug,print("at d="d" z="z));
     d++;  \\ try to follow to the left first
     my(seen=0);
     for(i=1,4,
       for(i=0,1,
         my(n=c_curve_unpoint(z,d));
         if(n>=0 && n<limit,
            if(!mapisdefined(m,n) && (bothways||!seen),
               listput(l,n);
               mapput(m,n,1));
            seen++);
         z+=I^d; d+=2);
       if(seen,break());
       d--);  \\ try to the right
     seen || error();
     #l<=limit || error("list too long ",#l," ",Vec(l));
     z+=I^d);
   Vec(l);
 }

gp_inline__location="../c-curve.tex:8777";
 Lpred_r3  = 40;
 Lpred_r4  = 41;
 Lpred_r5  = 42;
 Lpred_r6  = 43;
 Lpred_r7  = 44;
 Lpred_d12 = 45;
 Lpred_d21 = 46;
 Lpred_d2  = 47;
 Lpred_d1  = 48;
 Lpred_non = 49;
 \\                 bit 0, 1
 Lpred_state_table = {[ 1, 2;    /* s0 */
                        1, 3;    /* s1 */
                        4, 2;    /* s2 */
                        5 ,6;    /* s3 */
                        7, 8;    /* s4 */

                         7, 9;   /* s5 */
                        10,11;   /* s6 */
                        12,13;   /* s7 */
                        14, 6;   /* s8  */

                        14,15;   /* s9 */
                        16,17;   /* s10 */
                        18,19;   /* s11 */
                        19,20;   /* s12 */
                        21,22;   /* s13 */
                        23, 9;   /* s14 */

                        24,25;                /* s15 */
                        12,26;                /* s16 */
                        Lpred_d2, Lpred_d21;  /* s17 */
                        Lpred_d12, 27;        /* s18 */
                        Lpred_r4,  Lpred_r3;  /* s19 */
                        28,       Lpred_d21;  /* s20 */
                        Lpred_d12, Lpred_d2;  /* s21 */
                        29,11;                /* s22 */
                        30,31;                /* s23 */

                        16,32;          /* s24 */
                        Lpred_d1, 34;   /* s25 */
                        33, Lpred_d1;   /* s26 */
                        34, Lpred_r5;   /* s27 */
                        Lpred_r6, 35;   /* s28 */
                        Lpred_d1, 36;   /* s29 */
                        35, Lpred_d1;   /* s30 */
                        37, 22;         /* s31 */

                        Lpred_d2, Lpred_d1;   /* s32 */
                        Lpred_d12, Lpred_non; /* s33 */
                        Lpred_non, 38;        /* s34 */
                        39, Lpred_non;        /* s35 */
                        Lpred_non, Lpred_d21; /* s36 */
                        Lpred_d1, Lpred_d2;   /* s37 */

                        Lpred_non, 39;  /* s38 */
                        38, Lpred_non;  /* s39 */

                        Lpred_r4, Lpred_r5;    /* 40 = Lpred_r3  */
                        Lpred_r6, Lpred_r3;    /* 41 = Lpred_r4  */
                        Lpred_r7, Lpred_non;   /* 42 = Lpred_r5  */
                        Lpred_non, Lpred_r7;   /* 43 = Lpred_r6  */
                        Lpred_r6, Lpred_r5;    /* 44 = Lpred_r7  */
                        Lpred_non, Lpred_d1;   /* 45 = d12  */
                        Lpred_d1, Lpred_non;   /* 46 = d21 */
                        Lpred_d1, Lpred_d1;    /* 47 = d2  */
                        Lpred_non, Lpred_non;  /* 48 = d1  */
                        Lpred_non, Lpred_non   /* 49 = non  */
 ]};
 Lpred_transition(state,digit) = Lpred_state_table[state+1,digit+1];
 Lpred_transitions(state,v) = {
   for(i=1,#v, state=Lpred_transition(state,v[i]));
   state;
 }
 Lpred_k_state(k,n) = Lpred_transitions(0,binary_padded(n,k));
 Lpred_k(k,n) = Lpred_k_state(k,n) != Lpred_non;
 LpredInf_state(n) = Lpred_transitions(1,binary(n));
 LpredInf(n) = LpredInf_state(n) != Lpred_non;
 NonLpredInf(n) = !LpredInf(n);
 Lpred_k_list_by_states(k) =
   select(n->Lpred_k(k,n), vector(2^k,n,n--; n));

gp_inline__location="../c-curve.tex:8869";
 Lpred_k_state_list(k,n) = {
   my(v=binary_padded(n,k), state=1);
   for(i=1,#v,
       my(bit=v[i]);
       v[i]=state; state=Lpred_transition(state,bit));
   concat(v,[state]);
 }

gp_inline__location="../c-curve.tex:9280";
 NonLpredInf_at_samples = {[55,56,71,72,92,93,94,101,102,103]};

gp_inline__location="../c-curve.tex:9989";
 gNonL(x) = 1/(1-2*x) - gL(x);

gp_inline__location="../c-curve.tex:10001";
 gNonR(x) = 1/(1-2*x) - gR(x);

gp_inline__location="../c-curve.tex:10014";
 LdoublesPart(k) = if(k==4,1, k%2==0 && k>=6, 2, 0);
 Ldoubles(k) = sum_over_parts(LdoublesPart,k);

gp_inline__location="../c-curve.tex:10034";
 Apart_samples_generated = [0,0,0,0,0,1,4,15,56,132,312,670,1432,2962,6104,12410,25176,50762,102232,205290,411992,825642,1654104,3311530,6628696,13264042,26539352];

gp_inline__location="../c-curve.tex:10042";
 A_initial = {[0,0,0,0,0,1]};

gp_inline__location="../c-curve.tex:10050";
 A_half_vector = {[142,201]};

gp_inline__location="../c-curve.tex:10059";
 A_formula(k) = {
   19/16 * 2^k
   - 1/8 * vector_modulo(A_half_vector,k) * 2^floor(k/2)
   + 2*k + 60;
 }
 A(k) = if(k<6,A_initial[k+1], A_formula(k));

gp_inline__location="../c-curve.tex:10069";
 gA(x) = {
   -695/16 - 157/4*x - 133/4*x^2 - 101/4*x^3 - 16*x^4 - 13/2*x^5
   + 58 * 1/(1-x)
   + 2 * 1/(1-x)^2
   + (-71/4 - 201/8*x)/(1 - 2*x^2)
   + 19/16 * 1/(1-2*x);
 }

gp_inline__location="../c-curve.tex:10079";
 A_samples = {[0,0,0,0,0,1,6,25,96,284,728]};

gp_inline__location="../c-curve.tex:10089";
 Apart_initial = {[0,0,0,0,0,1]};

gp_inline__location="../c-curve.tex:10097";
 Apart_half_vector = {[24,35]};

gp_inline__location="../c-curve.tex:10104";
 Apart_constant_vector = {[8,-2]};

gp_inline__location="../c-curve.tex:10109";
 Apart_formula(k) = {
   19/48 * 2^k
   - 1/8 * vector_modulo(Apart_half_vector,k) * 2^floor(k/2)
   + 1/3 * vector_modulo(Apart_constant_vector,k);
 }
 Apart(k) = if(k<6,Apart_initial[k+1], Apart_formula(k));

gp_inline__location="../c-curve.tex:10119";
 Apart_samples = {[0,0,0,0,0,1,4,15,56,132,312]};

gp_inline__location="../c-curve.tex:10302";
 Apart_delta_vector = {[0,8]};

gp_inline__location="../c-curve.tex:10308";
 Apart_by_recurrence(k) = {
   2*Apart(k-1)
   + 1/2*Bpart(k-1)
   - vector_modulo(Apart_delta_vector,k);
 }

gp_inline__location="../c-curve.tex:10324";
 A_from_Apart_sum(k) = Apart(k) + sum(j=0,k-1, 2*Apart(j));

gp_inline__location="../c-curve.tex:10341";
 density_A = 4/19;
 density_A_sample = {(0210526/1000000)};

gp_inline__location="../c-curve.tex:10369";
 QR1_generated_samples = [1,2,2,4,6,12,24,48,102,204,412,824,1642,3284,6552,13104,26206,52412,104852,209704,419442,838884,1677744,3355488,6710886];
 QR2_generated_samples = [0,0,0,0,0,0,2,4,8,16,30,60,116,232,466,932,1872,3744,7494,14988,29964,59928,119834,239668,479336];
 QR3_generated_samples = [0,0,0,0,1,2,4,8,15,30,58,116,233,466,936,1872,3747,7494,14982,29964,59917,119834,239668,479336,958695];
 QR4_generated_samples = [0,0,1,2,4,8,15,30,58,116,233,466,936,1872,3747,7494,14982,29964,59917,119834,239668,479336,958695,1917390,3834802];
 TR1_generated_samples = [1,0,0,0,0,0,0,2,6,10,22,40,80,156,308,622,1242,2494,4994,9988,19988,39952,79904,159786,319550];
 TR2_generated_samples = [0,0,0,0,0,0,0,2,2,6,10,20,40,76,156,310,622,1250,2494,5000,9988,19976,39952,79882,159786];
 TR3_generated_samples = [0,0,0,0,0,0,1,1,3,5,10,20,38,78,155,311,625,1247,2500,4994,9988,19976,39941,79893,159775];
 TR4_generated_samples = [0,0,0,0,0,1,1,3,5,10,20,38,78,155,311,625,1247,2500,4994,9988,19976,39941,79893,159775,319561];
 TR5_generated_samples = [0,0,0,0,1,1,3,5,10,20,38,78,155,311,625,1247,2500,4994,9988,19976,39941,79893,159775,319561,639134];
 TR6_generated_samples = [0,0,0,1,1,3,5,10,20,38,78,155,311,625,1247,2500,4994,9988,19976,39941,79893,159775,319561,639134,1278256];
 TR7_generated_samples = [0,0,1,1,3,5,10,20,38,78,155,311,625,1247,2500,4994,9988,19976,39941,79893,159775,319561,639134,1278256,2556546];
 TR8_generated_samples = [0,1,1,1,1,2,4,8,18,39,79,159,315,628,1250,2494,4988,9977,19965,39941,79893,159798,319584,639156,1278278];
 TR9_generated_samples = [0,0,0,2,4,8,16,30,60,116,232,466,932,1872,3744,7494,14988,29964,59928,119834,239668,479336,958672,1917390,3834780];

gp_inline__location="../c-curve.tex:10445";
 TR(k,a) = {
   if(k<8, return(TR_samples[a][k+1]));
       TR(k-1,a)
   + 2*TR(k-2,a)
   -   TR(k-4,a)
   +   TR(k-5,a)
   + 2*TR(k-7,a)
   + 4*TR(k-8,a);
 }
 TR = memoize(TR);
 TR_recurrence_polynomial(x) = -x^8 + x^7+2*x^6 - x^4+x^3  + 2*x+4;

gp_inline__location="../c-curve.tex:10475";
 TR1_samples = {[1,0,0,0,0,0,0,2,6,10,22,40,80,156,308,622]};

gp_inline__location="../c-curve.tex:10484";
 TR2_samples = {[0,0,0,0,0,0,0,2,2,6,10,20,40,76,156,310]};

gp_inline__location="../c-curve.tex:10493";
 TR3_samples = {[0,0,0,0,0,0,1,1,3,5,10,20,38,78,155,311]};

gp_inline__location="../c-curve.tex:10502";
 TR4_samples = {[0,0,0,0,0,1,1,3,5,10,20,38,78,155,311]};

gp_inline__location="../c-curve.tex:10511";
 TR5_samples = {[0,0,0,0,1,1,3,5,10,20,38,78,155,311]};

gp_inline__location="../c-curve.tex:10519";
 TR6_samples = {[0,0,0,1,1,3,5,10,20,38,78,155,311]};

gp_inline__location="../c-curve.tex:10527";
 TR7_samples = {[0,0,1,1,3,5,10,20,38,78,155,311]};

gp_inline__location="../c-curve.tex:10535";
 TR8_samples = {[0,1,1,1,1,2,4,8,18,39,79,159,315,628,1250]};

gp_inline__location="../c-curve.tex:10543";
 TR9_samples = {[0,0,0,2,4,8,16,30,60,116,232,466,932,1872,3744]};

gp_inline__location="../c-curve.tex:10563";
 gTR1(x) = 2/105*1/(1-2*x) + 1/6*1/(1+x) + 1/20*(7-x)/(1+x^2) \
   + 1/28*(13+5*x-10*x^2-6*x^3)/(1-x^2+2*x^4);

gp_inline__location="../c-curve.tex:10576";
 gTR7(x) = 16/105*1/(1-2*x) + 1/12*1/(1+x) + 1/40*(-3-x)/(1+x^2) \
   + 1/56*(-9 - 11*x + 22*x^2 + 2*x^3)/(1-x^2+2*x^4);

gp_inline__location="../c-curve.tex:10589";
 gTR8(x) = 8/105*1/(1-2*x) - 1/12*1/(1+x) + 1/40*(-9+7*x)/(1+x^2) \
   + 1/56*(13+33*x+18*x^2-6*x^3)/(1-x^2+2*x^4);

gp_inline__location="../c-curve.tex:10601";
 gTR9(x) = 24/105*1/(1-2*x)  + 1/10*(2-x)/(1+x^2) \
   + 1/14*(-6 -5*x -4*x^2 +6*x^3)/(1-x^2+2*x^4);

gp_inline__location="../c-curve.tex:10606";
 gTR2(x) = 2*x * gTR3(x);

gp_inline__location="../c-curve.tex:10610";
 gTR3(x) = x*gTR4(x);

gp_inline__location="../c-curve.tex:10614";
 gTR4(x) = x*gTR5(x);

gp_inline__location="../c-curve.tex:10618";
 gTR5(x) = x*gTR6(x);

gp_inline__location="../c-curve.tex:10622";
 gTR6(x) = x*gTR7(x);

gp_inline__location="../c-curve.tex:10641";
 TR_samples = {[
   TR1_samples,
   TR2_samples,
   TR3_samples,
   TR4_samples,
   TR5_samples,
   TR6_samples,
   TR7_samples,
   TR8_samples,
   TR9_samples
 ]};

gp_inline__location="../c-curve.tex:10844";
 TR_matrix = matrix(9,9);
 TR_matrix_store(from,to,value) = {
   if(TR_matrix[from,to], error("oops, TR_matrix ",from,",",to," already set"));
   TR_matrix[from,to] = value;
 }
 TR_initial = vectorv(9,a, a==1);
 TR_vector(k) = TR_matrix^k * TR_initial;
 TR_vector = memoize(TR_vector);
 TR_by_matrix(k,a) = TR_vector(k)[a];

gp_inline__location="../c-curve.tex:10873";
 TR_matrix_store(1,2, 2);
 TR_matrix_store(1,3, 2);

gp_inline__location="../c-curve.tex:10878";
 TR_matrix_store(2,3, 2);

gp_inline__location="../c-curve.tex:10882";
 TR_matrix_store(3,4, 1);

gp_inline__location="../c-curve.tex:10886";
 TR_matrix_store(4,5, 1);

gp_inline__location="../c-curve.tex:10890";
 TR_matrix_store(5,6, 1);

gp_inline__location="../c-curve.tex:10894";
 TR_matrix_store(6,7, 1);

gp_inline__location="../c-curve.tex:10899";
 TR_matrix_store(7,8, 1);
 TR_matrix_store(7,9, 1);

gp_inline__location="../c-curve.tex:10907";
 TR_matrix_store(8,1, 1);
 TR_matrix_store(8,4, 1);
 TR_matrix_store(8,5, 1);
 TR_matrix_store(8,8, 1);

gp_inline__location="../c-curve.tex:10915";
 TR_matrix_store(9,6, 2);
 TR_matrix_store(9,7, 2);

gp_inline__location="../c-curve.tex:10937";
 TR_gf_column = (matid(9) - TR_matrix*x)^-1 * TR_initial;

gp_inline__location="../c-curve.tex:10998";
 TR_polynomial_root5 = sqrt( (1+sqrt(7)*I)/2 );
 TR_polynomial_root6 = sqrt( (1-sqrt(7)*I)/2 );
 TR_polynomial_root7 = -sqrt( (1+sqrt(7)*I)/2 );
 TR_polynomial_root8 = -sqrt( (1-sqrt(7)*I)/2 );
 TR_polynomial(x) = x^4 - x^2 + 2;  /* reversed */
 TR_polynomial_quadratic(x) = x^2 - x + 2;  /* reversed */
 TR_polynomial_quadratic_root = (1 + sqrt7i)/2;

gp_inline__location="../c-curve.tex:11030";
 TR_polynomial_root_real_sample = {(0978318/1000000)};

gp_inline__location="../c-curve.tex:11037";
 TR_polynomial_root_imag_sample = {(0676096/1000000)};

gp_inline__location="../c-curve.tex:11094";
 TR_polynomial_root_magnitude_sample = {(1189207/1000000)};

gp_inline__location="../c-curve.tex:11358";
 QR1(k) = TR(k,1) + 2*TR(k,8) + TR(k,9);

gp_inline__location="../c-curve.tex:11364";
 QR2(k) = TR(k,2) + 2*TR(k,3);

gp_inline__location="../c-curve.tex:11372";
 QR3(k) = TR(k,4) + TR(k,5);

gp_inline__location="../c-curve.tex:11380";
 QR4(k) = TR(k,6) + TR(k,7);

gp_inline__location="../c-curve.tex:11382";
 QR2_by_mutual(k) = if(k<2,QR2(k), 2*QR3(k-2));
 QR3_by_mutual(k) = if(k<2,QR3(k),   QR4(k-2));

gp_inline__location="../c-curve.tex:11402";
 QR(k,a) = {
   if(k<0, error("QR() is for k>=0, got k="k));
   if(k<7, return(QR_samples[a][k+1]));
     2*QR(k-1,a)
   -   QR(k-4,a)
   + 2*QR(k-5,a)
   - 2*QR(k-6,a)
   + 4*QR(k-7,a);
 }
 QR = memoize(QR);
 QR_recurrence_polynomial(x) = -x^7 + 2*x^6 - x^3 + 2*x^2 - 2*x + 4;

gp_inline__location="../c-curve.tex:11415";
 QR1_samples = {[1,2,2,4,6,12,24,48,102,204,412,824,1642]};

gp_inline__location="../c-curve.tex:11424";
 QR2_samples = {[0,0,0,0,0,0,2,4,8,16,30,60,116]};

gp_inline__location="../c-curve.tex:11442";
QR3_samples = {[0,0,0,0,1,2,4,8,15,30,58,116,233]};

gp_inline__location="../c-curve.tex:11459";
 QR4_samples = {[0,0,1,2,4,8,15,30,58,116,233,466,936]};

gp_inline__location="../c-curve.tex:11478";
 gQR1(x) = (1 - 2*x^2 - x^4) \
   / ((1-2*x) *(1+x^2) *(1 - x^2 + 2*x^4));

gp_inline__location="../c-curve.tex:11488";
 gQR1_partial_fractions(x) = 2/5 * 1/(1-2*x) \
   + 1/10*(1+2*x)/(1+x^2) \
   + 1/2*(1+2*x)/(1 - x^2 + 2*x^4);

gp_inline__location="../c-curve.tex:11495";
 gQR2(x) = 2*x^2*gQR3(x);

gp_inline__location="../c-curve.tex:11499";
 gQR3(x) = x^2*gQR4(x);

gp_inline__location="../c-curve.tex:11505";
 gQR4(x) = x^2  /  ((1-2*x) *(1+x^2) *(1 - x^2 + 2*x^4));

gp_inline__location="../c-curve.tex:11515";
 gQR4_partial_fractions(x) = 8/35 * 1/(1-2*x) \
   - 1/20*(1+2*x)/(1+x^2) \
   - 1/28*(5 + 10*x - 6*x^2 - 12*x^3)/(1 - x^2 + 2*x^4);

gp_inline__location="../c-curve.tex:11527";
 QR_samples = {[
   QR1_samples,
   QR2_samples,
   QR3_samples,
   QR4_samples
 ]};

gp_inline__location="../c-curve.tex:11646";
 poly9(x) = x^9 - 3*x^8 + 3*x^7 - 3*x^6 + 2*x^5 + 4*x^4 - 8*x^3 + 8*x^2 - 16*x + 8;
 poly9_nf = nfinit(poly9(x));

gp_inline__location="../c-curve.tex:11651";
 poly9_roots = polroots_ascending(poly9(x));
 r = real(poly9_roots[9]);
 poly9_nf_basis = subst(poly9_nf.zk, 'x, r);  \\ flonums
 poly9_nfelt_value(e) = poly9_nf_basis * e;  \\ dot product
 r_nfelt = nfeltpow(poly9_nf,x,1);
 zero_nfelt = r_nfelt - r_nfelt;
 \\ one_nfelt as a vector, not just integer 1
 one_nfelt = nfeltmul(poly9_nf, r_nfelt, nfeltpow(poly9_nf,x,-1));

gp_inline__location="../c-curve.tex:11664";
 r_sample = {(1954776/1000000)};

gp_inline__location="../c-curve.tex:11668";
 r_sample_above = r_sample  + 1/10^6;

gp_inline__location="../c-curve.tex:11686";
 dimH_sample = {(1934007/1000000)};
 dimH_by_polroots = log(r)/log(sqrt(2));

gp_inline__location="../c-curve.tex:11709";
 revpoly9(x) = {
   1 - 3*x + 3*x^2 - 3*x^3 + 2*x^4 + 4*x^5 - 8*x^6 + 8*x^7 - 16*x^8
   + 8*x^9;
 }

gp_inline__location="../c-curve.tex:11826";
 DJ(k) = if(k<9,return(DJ_samples[k+1])); \
        3*DJ(k-1) \
     -  3*DJ(k-2) \
     +  3*DJ(k-3) \
     -  2*DJ(k-4) \
     -  4*DJ(k-5) \
     +  8*DJ(k-6) \
     -  8*DJ(k-7) \
     + 16*DJ(k-8) \
     -  8*DJ(k-9);
 DJ = memoize(DJ);

gp_inline__location="../c-curve.tex:11839";
 DJ_samples = {[0,0,0,1,0,1,0,3,4,9,12,39,72,141,264,547,1036]};

gp_inline__location="../c-curve.tex:11855";
 gDJ(x) = x^3 * (1 - 3*x + 4*x^2 - 6*x^3 + 8*x^4 - 4*x^5) / revpoly9(x);

gp_inline__location="../c-curve.tex:11867";
 pair_matrix = matrix(12,12);
 pair_matrix_store(from,to,value) = {
   if(pair_matrix[to,from], error("oops, pair_matrix ",from,",",to," already set"));
   pair_matrix[to,from] = value;
 }

gp_inline__location="../c-curve.tex:11891";
 pair_matrix_store(1,3, 2);
 pair_matrix_store(1,7, 1);

gp_inline__location="../c-curve.tex:11899";
 pair_matrix_store(2,3, 2);

gp_inline__location="../c-curve.tex:11906";
 pair_matrix_store(3,10, 1);

gp_inline__location="../c-curve.tex:11920";
 pair_matrix_store(4,7, 2);
 pair_matrix_store(4,12, 2);

gp_inline__location="../c-curve.tex:11928";
 pair_matrix_store(5,4, 1);
 pair_matrix_store(5,10, 2);

gp_inline__location="../c-curve.tex:11936";
 pair_matrix_store(6,11, 2);

gp_inline__location="../c-curve.tex:11951";
 pair_matrix_store(7,1, 1);
 pair_matrix_store(7,6, 1);
 pair_matrix_store(7,9, 2);

gp_inline__location="../c-curve.tex:11960";
 pair_matrix_store(8,5, 1);
 pair_matrix_store(8,8, 1);

gp_inline__location="../c-curve.tex:11968";
 pair_matrix_store(9,2, 1);
 pair_matrix_store(9,9, 1);

gp_inline__location="../c-curve.tex:11984";
 pair_matrix_store(10,8, 1);

gp_inline__location="../c-curve.tex:11991";
 pair_matrix_store(11,1, 1);

gp_inline__location="../c-curve.tex:11998";
 pair_matrix_store(12,12, 1);

gp_inline__location="../c-curve.tex:12099";
 DJ_matrix = matrix(12,12);
 DJ_matrix_store(from,to,value) = {
   if(DJ_matrix[to,from], error("oops, DJ_matrix ",from,",",to," already set"));
   DJ_matrix[to,from] = value;
 }
 DJ_initial = vectorv(12,p, p==11);
 DJ_vector(k) = DJ_matrix^k * DJ_initial;
 DJ_vector = memoize(DJ_vector);
 DJ_p(k,p) = DJ_vector(k)[p];

gp_inline__location="../c-curve.tex:12141";
 DJ_matrix_store(1,3, 2);
 DJ_matrix_store(1,7, 1);

gp_inline__location="../c-curve.tex:12146";
 DJ_matrix_store(2,3, 2);

gp_inline__location="../c-curve.tex:12150";
 DJ_matrix_store(3,10, 1);

gp_inline__location="../c-curve.tex:12155";
 DJ_matrix_store(4,7, 2);
 DJ_matrix_store(4,12, 2);

gp_inline__location="../c-curve.tex:12161";
 DJ_matrix_store(5,4, 1);
 DJ_matrix_store(5,10, 2);

gp_inline__location="../c-curve.tex:12166";
 DJ_matrix_store(6,11, 2);

gp_inline__location="../c-curve.tex:12172";
 DJ_matrix_store(7,1, 1);
 DJ_matrix_store(7,6, 1);
 DJ_matrix_store(7,9, 2);

gp_inline__location="../c-curve.tex:12179";
 DJ_matrix_store(8,5, 1);
 DJ_matrix_store(8,8, 1);

gp_inline__location="../c-curve.tex:12185";
 DJ_matrix_store(9,2, 1);
 DJ_matrix_store(9,9, 1);

gp_inline__location="../c-curve.tex:12190";
 DJ_matrix_store(10,8, 1);

gp_inline__location="../c-curve.tex:12194";
 DJ_matrix_store(11,1, 1);

gp_inline__location="../c-curve.tex:12199";
 DJ_matrix_store(12,12, 1);

gp_inline__location="../c-curve.tex:12219";
 Strichartz_Wang_matrix = {[
   0,0,2,0,0,0,1,0,0,0,0;
   0,0,2,0,0,0,0,0,0,0,0;
   0,0,0,0,0,0,0,0,0,1,0;
   0,0,0,0,0,0,2,0,0,0,0;
   0,0,0,1,0,0,0,0,0,2,0;
   0,0,0,0,0,0,0,0,0,0,2;
   1,0,0,0,0,1,0,0,2,0,0;
   0,0,0,0,1,0,0,1,0,0,0;
   0,1,0,0,0,0,0,0,1,0,0;
   0,0,0,0,0,0,0,1,0,0,0;
   1,0,0,0,0,0,0,0,0,0,0
 ]};

gp_inline__location="../c-curve.tex:12316";
 DJ_p12_samples = {[0,0,0,0,0,0,0,4,8,16,40,92,176,344]};

gp_inline__location="../c-curve.tex:12322";
 DJ_p12_by_sum(k) = 2*sum(j=0,k-1, DJ_p(j,4));

gp_inline__location="../c-curve.tex:12335";
 DJ_p_gf_column = (matid(12) - DJ_matrix*x)^-1 * DJ_initial;

gp_inline__location="../c-curve.tex:12341";
 gf_denominator(g) = denominator(g) / polcoeff(denominator(g),0);

gp_inline__location="../c-curve.tex:12364";
 D(k) = if(k==0, 0, 2*D(k-1) + DJ(k-1));
 D = memoize(D);

gp_inline__location="../c-curve.tex:12368";
D_by_sum(k) = sum(j=0,k-1, 2^(k-1-j)*DJ(j));

gp_inline__location="../c-curve.tex:12384";
 D_by_recurrence(k) = {
   if(k<10,return(D_samples[k+1]));
     5*D(k-1)
   - 9*D(k-2)
   + 9*D(k-3)
   - 8*D(k-4)
   + 0*D(k-5)
   + 16*D(k-6)
   - 24*D(k-7)
   + 32*D(k-8)
   - 40*D(k-9)
   + 16*D(k-10);
 }
 D_by_recurrence = memoize(D_by_recurrence);

gp_inline__location="../c-curve.tex:12400";
 D_samples = {[0,0,0,0,1,2,5,10,23,50,109,230,499,1070]};

gp_inline__location="../c-curve.tex:12411";
 gD(x) = x * gDJ(x) / (1-2*x);

gp_inline__location="../c-curve.tex:12419";
 gD_from_gS(x) = 1/2*( 1/(1-2*x) - gS(x) );

gp_inline__location="../c-curve.tex:12431";
 S(k) = if(k<9,return(S_samples[k+1])); \
        3*S(k-1) \
     -  3*S(k-2) \
     +  3*S(k-3) \
     -  2*S(k-4) \
     -  4*S(k-5) \
     +  8*S(k-6) \
     -  8*S(k-7) \
     + 16*S(k-8) \
     -  8*S(k-9);
 S = memoize(S);

gp_inline__location="../c-curve.tex:12445";
 S_samples = {[1,2,4,8,14,28,54,108,210,412,806,1588,3098,6052]};

gp_inline__location="../c-curve.tex:12456";
 gS(x) = (1 - x + x^2 - x^3 - 2*x^4 + 6*x^5 - 4*x^6 + 12*x^7 - 8*x^8)/revpoly9(x);

gp_inline__location="../c-curve.tex:12495";
S_plus_D_samples = {[1,2,4,8,15,30,59,118,233,462,915,1818,3597,7122]};

gp_inline__location="../c-curve.tex:12529";
 DC_by_sum(k) = \
   my(i=4,total=0);while(i<=k, total += 2^(k-i); i+=4); total;

gp_inline__location="../c-curve.tex:12533";
 DC(k) = (2^k - 2^(k%4))/15;

gp_inline__location="../c-curve.tex:12536";
 DC_by_powers2(k) = 2^k*1/16*((1/16)^floor(k/4) - 1)/(1/16 - 1);

gp_inline__location="../c-curve.tex:12539";
 gDC(x) = 1/15*( 1/(1-2*x) - (1 + 2*x + 4*x^2 + 8*x^3)/(1-x^4) );

gp_inline__location="../c-curve.tex:12548";
DC_samples = {[0,0,0,0,1,2,4,8,17,34,68,136,273,546,1092]};

gp_inline__location="../c-curve.tex:12556";
 gA083593(x) = 1/((1-2*x)*(1-x^4));

gp_inline__location="../c-curve.tex:12566";
 DC_by_binary(k) = if(k<3,return(0)); fromdigits(vector(k-3,i, i%4 == 1), 2);

gp_inline__location="../c-curve.tex:12579";
 DJC(k) = (k%4==3);
 DC_by_DJC_sum(k) = sum(j=0,k-1, 2^(k-1-j)*DJC(j));

gp_inline__location="../c-curve.tex:12623";
 p1rot(k) = p6rot(k+2);
 p6rot(k) = if(k%2==0,0, 1/3*(2^((k-1)/2) - (-1)^((k-1)/2)));
 p7rot(k) = p6rot(k+1);
 p11rot(k) = if(k==0,1, 2*p6rot(k-1));

gp_inline__location="../c-curve.tex:12632";
 p7rot_explicitly(k) = if(k%2==1,0, 1/3*(2^(k/2) - (-1)^(k/2)));

gp_inline__location="../c-curve.tex:12635";
 p11rot_explicitly(k) = if(k%2==1,0, 1/3*(2^(k/2) + 2*(-1)^(k/2)));

gp_inline__location="../c-curve.tex:12669";
 DJrot(k) = if(k%2==0,0, 1/3*(2^((k-1)/2) - (-1)^((k-1)/2)));

gp_inline__location="../c-curve.tex:12680";
 DJrot_samples = {[0,0,0,1,0,1,0,3,0,5,0,11,0,21,0,43,0,85]};

gp_inline__location="../c-curve.tex:12705";
 Drot_by_sum(k) = sum(j=0,k-1, 2^(k-1-j)*DJrot(j));

gp_inline__location="../c-curve.tex:12717";
Drot_vector = {[1,2,-1,-2]};

gp_inline__location="../c-curve.tex:12721";
 Drot(k) = 1/10*2^k - 1/6*2^ceil(k/2) \
           + 1/15*vector_modulo(Drot_vector,k);

gp_inline__location="../c-curve.tex:12725";
 Drot_2(k) = 1/30*( 3*2^k \
               - 2^(k%2) * (5 * 2^floor(k/2) - 2*(-1)^floor(k/2)));

gp_inline__location="../c-curve.tex:12756";
Drot_samples = {[0,0,0,0,1,2,5,10,23,46,97,194,399,798,1617]};

gp_inline__location="../c-curve.tex:12770";
 DC_over_D_limit = 2/15;
 DC_over_D_sample = {(013333/100000)};

gp_inline__location="../c-curve.tex:12786";
 Drot_over_D_limit = 3/15;

gp_inline__location="../c-curve.tex:12801";
 D_exceeds_S_k = {48};

gp_inline__location="../c-curve.tex:12813";
 two_D_exceeds_S_k = {30};

gp_inline__location="../c-curve.tex:12855";
 S_identity(k) = {
        16 * S(k-5)
     +  22 * S(k-6)
     -   2 * S(k-7)
     -  13 * S(k-8)
     -  26 * S(k-9)
     +  20 * S(k-10)
     + 128 * S(k-11)
     + 184 * S(k-12)
     + 168 * S(k-13)
     +   8 * S(k-14)
     - 128 * S(k-15)
 ;}

gp_inline__location="../c-curve.tex:12871";
 S_identity_recurrence_terms = \
   [-1, 0,0,0,0, 16,22,-2, -13,-26,20, 128,184,168,8,-128 ];

gp_inline__location="../c-curve.tex:12906";
 S_positive_factor(k) = {
        16 * 1/3 * 2^(k-5)
     +  22 * 1/3 * 2^(k-6)
     +  20 * 1/3 * 2^(k-10)
     + 128 * 1/3 * 2^(k-11)
     + 184 * 1/3 * 2^(k-12)
     + 168 * 1/3 * 2^(k-13)
     +   8 * 1/3 * 2^(k-14)
 ;}

gp_inline__location="../c-curve.tex:12918";
 S_positive_terms(k) = {
        16 * S(k-5)
     +  22 * S(k-6)
     +  20 * S(k-10)
     + 128 * S(k-11)
     + 184 * S(k-12)
     + 168 * S(k-13)
     +   8 * S(k-14)
 ;}

gp_inline__location="../c-curve.tex:12974";
 {
   my(m = matrix(9,9, i,j, poly9_roots[j]^(i-1)),
      v = vectorv(9,k,k--; S(k)),
      f = matsolve(m,v));
   S_power_coefficients_complex = Vec(f);
 }
 S_power_coefficients = S_power_coefficients_complex;
 /*real*/ S_power_coefficients[1] = real(S_power_coefficients[1]);

gp_inline__location="../c-curve.tex:12984";
 S_by_powers(k) = \
   sum(i=1,9, S_power_coefficients[i]*poly9_roots[i]^k);

gp_inline__location="../c-curve.tex:12988";
 S_magnitudes(k) = {
   sum(i=1,#poly9_roots,
       abs(S_power_coefficients[i]) * abs(poly9_roots[i]/2)^k );
 }

gp_inline__location="../c-curve.tex:13054";
 ThreeB(k) = S(k) + S(k+1);
 gThreeB(x) = gS(x) + 1/x*(gS(x)-1);

gp_inline__location="../c-curve.tex:13058";
 ThreeB_samples = {[3,6,12,22,42,82,162,318,622,1218]};

gp_inline__location="../c-curve.tex:13128";
 ThreeNonB(k) = 2*D(k) + 2*D(k+1);
 gThreeNonB(x) = 2*gD(x) + 2*1/x*gD(x);

gp_inline__location="../c-curve.tex:13132";
 ThreeNonB_samples = {[0,0,0,2,6,14,30,66,146,318]};

gp_inline__location="../c-curve.tex:13249";
 QO2(k) = if(k==0,0, S(k-1));

gp_inline__location="../c-curve.tex:13256";
 QO4(k) = if(k==0,0, D(k-1));

gp_inline__location="../c-curve.tex:13260";
 QO1(k) = if(k==0,1, 0);
 QO3(k) = 0;
 QO(k) = QO1(k) + QO2(k) + QO3(k) + QO4(k);

gp_inline__location="../c-curve.tex:13344";
 turn3_count(k) = sum(i=4,k-1, if(i%4==0, 2^(k-1-i)));

gp_inline__location="../c-curve.tex:13359";
 conf_to_string(conf) = Str(conf);
 conf_print(conf) = {
   print("     .",if(conf[9],"--->","    "),".");
   print("    ",
         if(conf[7],"|"," "),
         if(conf[8],"^"," "),
         "    ",
         if(conf[10],"|"," "),
         if(conf[11],"^"," "));
   for(i=1,1,
      print("    ",
            if(conf[7],"|"," "),
            if(conf[8],"|"," "),
            "    ",
            if(conf[10],"|"," "),
            if(conf[11],"|"," ")));
   print("    ",
         if(conf[7],"v"," "),
         if(conf[8],"|"," "),
         "    ",
         if(conf[10],"v"," "),
         if(conf[11],"|"," "));

   print(if(conf[6],"<---","    ")," .",
         if(conf[1],"<---","    "),". ",
         if(conf[12],"<---","    "),
         "  ",conf_to_string(conf),
         /* ", sides=",conf_count_QI_sides(conf), */
         if(conf_is_double_seg(conf),", double",""));
   print(if(conf[5],"--->","    "),"  ",
         if(conf[2],"--->","    "),"  ",
         if(conf[13],"--->","    "));

   print("    ",
         if(conf[4],"|"," "),
         if(conf[3],"^"," "),
         "    ",
         if(conf[15],"|"," "),
         if(conf[14],"^"," "));
   for(i=1,1,
      print("    ",
            if(conf[4],"|"," "),
            if(conf[3],"|"," "),
            "    ",
            if(conf[15],"|"," "),
            if(conf[14],"|"," ")));
   print("    ",
         if(conf[4],"v"," "),
         if(conf[3],"|","."),
         "    ",
         if(conf[15],"v"," "),
         if(conf[14],"|","."));
   print("-----------------------------------------------");
 }

gp_inline__location="../c-curve.tex:13436";
 \\ expanded                1  2  3  4  5   6  7 8    9 19 11  12,13,14,15
 conf_left_permutation  = [ 8, 1, 9, 8,10,  9, 1,10, 15, 3, 2,  5, 4, 7, 6 ];
 conf_right_permutation = [10, 1,12,11,14, 13, 2,15,  3, 8, 1,  9, 8,10, 9 ];

gp_inline__location="../c-curve.tex:13441";
 conf_permutations_vector = \
   [conf_left_permutation, conf_right_permutation];
 conf_permute(conf,perm) = vector(#conf,i, conf[perm[i]]);

gp_inline__location="../c-curve.tex:13445";
 \\              1  2   3  4  5   6  7  8  9 19 11  12,13,14,15
 conf_mirror_permutation = [ 1, 2, 15,14,13, 12,11,10, 9, 8, 7,  6, 5, 4, 3 ];
 conf_identity_permutation = [1..#conf_mirror_permutation];
 conf_mirror(conf) = conf_permute(conf,conf_mirror_permutation);

gp_inline__location="../c-curve.tex:13451";
 \\ v is a vector of conf triangle positions, so each 1 to 15.
 \\ Return a conf which has those triangles filled.
 conf_make(v) = {
   my(conf=vector(15));
   for(i=1,#v,
       if(conf[v[i]],error("conf_make() duplicate part"));
       conf[v[i]]=1);
   conf;
 }

gp_inline__location="../c-curve.tex:13461";
 \\ vector of initial non-empty configurations, being a single
 \\ triangle in any position 1 to 15
 conf_initial_vector = vector(15,i, conf_make([i]));

gp_inline__location="../c-curve.tex:13465";
 \\ A vector suitable for use with forvec to iterate all 2^15
 \\ possible confs.  So for example
 \\    forvec(conf=conf_forvec_specification, conf_print(conf));
 \\
 conf_forvec_specification = vector(15,i, [0,1]);

gp_inline__location="../c-curve.tex:13471";
 \\ Configurations occurring in the curve have an index.
 \\ conf_to_index_map is a Map() from conf to that index.
 \\ conf_to_index(conf) returns the index of conf, or error
 \\ if conf does not occur.
 conf_to_index_map = Map();
 conf_to_index(conf) = mapget(conf_to_index_map,conf);

gp_inline__location="../c-curve.tex:13478";
 \\ conf_occurring_vector[i] is the conf within index i.
 conf_occurring_vector = List([]);

gp_inline__location="../c-curve.tex:13481";
 {
   my(l = List(conf_initial_vector));
   while(#l,
         my(conf = l[1]); listpop(l,1);
         if(vecsum(conf) == 0, next());
         if(mapisdefined(conf_to_index_map,conf),
            next());  /* already seen */

         listput(conf_occurring_vector, conf);
         mapput(conf_to_index_map, conf, #conf_occurring_vector);
         listput(l, conf_permute(conf,conf_left_permutation));
         listput(l, conf_permute(conf,conf_right_permutation)));
 }
 /*vectorize*/ conf_occurring_vector = Vec(conf_occurring_vector);

gp_inline__location="../c-curve.tex:13496";
 \\ conf_occurs(conf) returns 1 if conf occurs in the curve.
 conf_occurs(conf) = mapisdefined(conf_to_index_map,conf);

gp_inline__location="../c-curve.tex:13499";
 conf_occurring_mirror_vector = List([]);
 { my(m = Map());
   for(i=1,#conf_occurring_vector,
       my(conf = conf_occurring_vector[i]);
       if(mapisdefined(m,conf), next());
       listput(conf_occurring_mirror_vector, conf);
       mapput(m, conf, 1);
       mapput(m, conf_mirror(conf), 1));
   count;
 }
 /*vectorize*/ conf_occurring_mirror_vector = Vec(conf_occurring_mirror_vector);

gp_inline__location="../c-curve.tex:13511";
 iferr(read("tools/configurations-gfs-generated.gp"),e,read("../tools/configurations-gfs-generated.gp"));

gp_inline__location="../c-curve.tex:13514";
 \\ Return the generating function of how many times conf occurs
 \\ in the curve (of successive expansion levels), or error if
 \\ conf does not occur.
 conf_to_gf(conf) = conf_index_to_gf[conf_to_index(conf)];

gp_inline__location="../c-curve.tex:13519";
 \\ pred is a predicate function called pred(conf).
 \\ Return a generating function polynomial in 'x which is how many
 \\ times all confs matching pred occur.
 conf_predicate_to_gf(pred) = {
   vecsum(apply(conf-> if(pred(conf),conf_to_gf(conf)),
                conf_occurring_vector));
 }

gp_inline__location="../c-curve.tex:13527";
 \\ func is a conf count function func(conf).
 \\ Return a generating function polynomial in 'x which is how many
 \\ times all confs which have func(conf)==n occur.
 conf_predicate_n_to_gf(func,n) = \
   conf_predicate_to_gf(conf->func(conf)==n);

gp_inline__location="../c-curve.tex:13533";
 conf_matrix = matrix(#conf_index_to_gf,#conf_index_to_gf);
 { for(i=1,#conf_occurring_vector,
       my(conf = conf_occurring_vector[i]);
       for(j=1,#conf_permutations_vector,
           my(perm = conf_permutations_vector[j]);
           my(new_conf = conf_permute(conf,perm));
           if(vecsum(new_conf),
              conf_matrix[conf_to_index(new_conf),i]++)));
 }

gp_inline__location="../c-curve.tex:13543";
 \\ conf_index_to_initial_count[i] is the initial count of how many
 \\ occurrences of conf index i are in curve k=0.
 \\ This is 1 at each conf of conf_initial_vector[] and 0 elsewhere.
 conf_index_to_initial_count = vector(#conf_index_to_gf);
 for(i=1,#conf_initial_vector, \
     conf_index_to_initial_count[conf_to_index(conf_initial_vector[i])]++);

gp_inline__location="../c-curve.tex:13882";
 conf_count_occurring_sample = {753};

gp_inline__location="../c-curve.tex:13894";
 conf_occurring_at_k(k) = sum(i=1,#conf_occurring_vector, \
                             gf_term(conf_index_to_gf[i],k)!=0);

gp_inline__location="../c-curve.tex:13899";
 conf_count_predicate(pred) = {
   my(count=0);
   forvec(conf=conf_forvec_specification,
          if(pred(conf), count++));
   count;
 }
 conf_curve_start_or_end_vector = {[
    conf_make([1,4]),  \\ start
    conf_make([2,11]),
    conf_make([14,1]),  \\ end
    conf_make([7,2])
 ]};
 conf_is_curve_start_or_end(conf) = {
   if(vecsum(conf)==1 && !conf[9], return(1));  \\ single segment
   for(i=1,#conf_curve_start_or_end_vector,
       if(conf == conf_curve_start_or_end_vector[i],return(1)));
   0;
 }

gp_inline__location="../c-curve.tex:13918";
 vector_is_all_same(v) = \
   for(i=1,#v-1, if(v[i]!=v[i+1], return(0))); 1;

gp_inline__location="../c-curve.tex:13937";
 conf_left_enter_locations = {[1,3,5,7]};

gp_inline__location="../c-curve.tex:13942";
 conf_left_exit_locations = {[2,4,6,8]};

gp_inline__location="../c-curve.tex:13949";
 conf_right_enter_locations = {[2,10,12,14]};

gp_inline__location="../c-curve.tex:13954";
 conf_right_exit_locations = {[1,11,13,15]};

gp_inline__location="../c-curve.tex:13959";
 conf_count_locations(conf,v) = sum(i=1,#v, conf[v[i]]);
 \\ Return 1 if conf has its left point valid enter=leave counts
 conf_is_ELcount_valid_left(conf) = {
       conf_count_locations(conf,conf_left_enter_locations)
    == conf_count_locations(conf,conf_left_exit_locations);
 }
 \\ Return 1 if conf has its right point valid enter=leave counts
 conf_is_ELcount_valid_right(conf) = {
      conf_count_locations(conf,conf_right_enter_locations)
   == conf_count_locations(conf,conf_right_exit_locations);
 }
 \\ Return 1 if conf is valid for enter=leave counts
 conf_is_ELcount_valid(conf) = {
      conf_is_ELcount_valid_left(conf)
   && conf_is_ELcount_valid_right(conf);
 }

gp_inline__location="../c-curve.tex:13976";
 conf_left_exit_directed  = [6,8,2,4];
 conf_right_exit_directed = [13,15,1,11];

gp_inline__location="../c-curve.tex:13982";
 conf_turns_e_e(conf,enter,exit) = {
   my(l=List([]));
   for(i=1,#enter,
       for(j=1,#exit,
           if(conf[enter[i]] && conf[exit[j]],
              listput(l, (i-j) % 4))));
   Vec(l);
 }
 conf_turns_left(conf) = conf_turns_e_e(conf, \
        conf_left_enter_locations,conf_left_exit_directed);
 conf_turns_right(conf) = conf_turns_e_e(conf, \
        conf_right_enter_locations,conf_right_exit_directed);

gp_inline__location="../c-curve.tex:14003";
conf_is_left_even(conf) = {
  my(v = conf_turns_left(conf));
  #v!=0 && ! vector_contains(v, 1);
}
conf_is_right_even(conf) = {
  my(v = conf_turns_right(conf));
  #v!=0 && ! vector_contains(v, 1);
}

gp_inline__location="../c-curve.tex:14041";
 \\ Return 1 if conf is valid for left point being turn=t.
 conf_is_ELturn_valid_left(conf,t) = {
   t = 2-t;   \\ so t=0 is straight ahead, t=1 left, etc
   for(i=0,3,
       my(enter=vector_modulo(conf_left_enter_locations,i));
       my(exit=vector_modulo(conf_left_exit_locations,i+t));
       if(conf[enter]!=conf[exit], return(0)));
   1;
 }

gp_inline__location="../c-curve.tex:14054";
 conf_is_left_odd_valid(conf) = {
      conf[1] == conf[4]
   && conf[3] == conf[6]
   && conf[5] == conf[8]
   && conf[7] == conf[2];
 }

gp_inline__location="../c-curve.tex:14065";
 \\ Return 1 if conf is valid for right point being turn=t.
 conf_is_ELturn_valid_right(conf,t) = {
   t = t+2;   \\ so t=0 is straight ahead, t=1 left, etc
   for(i=0,3,
       my(exit=vector_modulo(conf_right_exit_locations,i));
       my(enter=vector_modulo(conf_right_enter_locations,i+t));
       if(conf[enter]!=conf[exit], return(0)));
   1;
 }

gp_inline__location="../c-curve.tex:14078";
 conf_is_right_odd_valid(conf) = {
      conf[2]  == conf[11]
   && conf[10] == conf[13]
   && conf[12] == conf[15]
   && conf[14] == conf[1];
 }

gp_inline__location="../c-curve.tex:14089";
 \\ Return 1 if conf is valid for turns.
 \\ Either left or right valid left turn t=1,
 \\ both valid for some turn.
 conf_is_ELturn_valid(conf) = {
   my(l=vector(4,t, conf_is_ELturn_valid_left(conf,t)),
      r=vector(4,t, conf_is_ELturn_valid_right(conf,t)));
   (l[1] || r[1]) && vecsum(l) && vecsum(r);
 }

gp_inline__location="../c-curve.tex:14103";
 conf_occurs_with_9_flipped(conf) = {
   conf[9] = 1-conf[9];
   vecsum(conf)==0 || conf_occurs(conf);
 }

gp_inline__location="../c-curve.tex:14118";
 conf_count_mirror_sample = {393};

gp_inline__location="../c-curve.tex:14165";
 poly8alt(x) = x^8 - 2*x^6 - x^4 + 2*x^2 - 4;

gp_inline__location="../c-curve.tex:14170";
 revpoly8alt(x) = poly_reverse(poly8alt(x));

gp_inline__location="../c-curve.tex:14176";
 poly8alt_largest_root = sqrt(1/2 + sqrt(5/4 + sqrt(5)));
 poly8alt_largest_root_by_polroots = polroots_descending(poly8alt(x))[1];

gp_inline__location="../c-curve.tex:14190";
 poly8alt_root_sample = {(1538538/1000000)};

gp_inline__location="../c-curve.tex:14211";
 poly8(x) =  x^8 - x^7 + x^6 - 2*x^5 + x^4 - x^3 + x^2 - 2;

gp_inline__location="../c-curve.tex:14218";
 poly8_root_sample = {(1320638/1000000)};

gp_inline__location="../c-curve.tex:14224";
 poly8_largest_root = polroots_descending(poly8(x))[1];

gp_inline__location="../c-curve.tex:14239";
 poly11(x) = {
     x^11 - 2*x^10 + 3*x^9
   - 5*x^8 + 6*x^7 - 9*x^6 + 10*x^5
   - 14*x^4 + 12*x^3 - 14*x^2 + 8*x - 8;
 }
 revpoly11(x) = poly_reverse(poly11(x));

gp_inline__location="../c-curve.tex:14249";
 poly11_root_sample = {(1642339/1000000)};

gp_inline__location="../c-curve.tex:14255";
 poly11_largest_root = polroots_descending(poly11(x))[1];

gp_inline__location="../c-curve.tex:14267";
 poly14(x) = x^14 - 2*x^10 - 5*x^6 - 6*x^4 - 4*x^2 - 8;

gp_inline__location="../c-curve.tex:14272";
 poly14_root_sample = {(1438110/1000000)};

gp_inline__location="../c-curve.tex:14280";
 poly14_largest_root = polroots_descending(poly14(x))[1];

gp_inline__location="../c-curve.tex:14300";
 poly18(x) = {
   x^18 - 5*x^17 + 12*x^16 - 21*x^15 + 28*x^14
   - 25*x^13 + 8*x^12 + 19*x^11 - 53*x^10
   + 90*x^9 - 112*x^8 + 118*x^7 - 108*x^6 + 88*x^5
   - 48*x^4 + 16*x^3 + 8*x^2 - 32*x + 32;
 }
 revpoly18(x) = poly_reverse(poly18(x));

gp_inline__location="../c-curve.tex:14312";
 poly18_r18_sample = {(1848349/1000000)};

gp_inline__location="../c-curve.tex:14319";
 poly18_roots = polroots_ascending(poly18(x));
 poly18_r18 = real(poly18_roots[18]); /* largest root */

gp_inline__location="../c-curve.tex:14335";
 gen_Enc_values = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,44,172,554,1656,4714,12832,33798,86756];

gp_inline__location="../c-curve.tex:14346";
 Enc_samples = {[8,44,172,554,1656,4714]};
 /*append*/ Enc_samples = concat(vector(14),Enc_samples);

gp_inline__location="../c-curve.tex:14375";
 EncPoly(x) = {
   (x-1)^2 * (x+1) * (x-2) * (x^2 + 1)
   * (x^4 + 1)^3 * (x^4 - 2)^2 * poly8(x) * poly8alt(x)
   * poly9(x) * poly11(x) * poly14(x) * poly18(x);
 }
 Enc_linear_recurrence_factors = {[
   14, -93, 399, -1264, 3155, -6331, 10024, -11316, 4211, 20227,
   -71426, 154220, -261835, 367331, -415968, 322933, 21513,
   -733582, 1895227, -3495150, 5358532, -7071828, 7941210,
   -7025760, 3250370, 4373962, -16408752, 32614928, -51607318,
   70551590, -85092320, 89643681, -77941144, 43977055, 16590125,
   -104987428, 217961315, -346368491, 474573956, -581114980,
   640107359, -624068137, 508562062, -276968364, -74978871,
   534289583, -1068161080, 1625050667, -2137499167, 2528238492,
   -2721196999, 2652570770, -2280166842, 1593834682, -621975798,
   -569789848, 1881894252, -3187362052, 4349170900, -5238370752,
   5746468952, -5802006776, 5384684200, -4524006704, 3297502496,
   -1829761968, 271705376, 1223138144, -2509938688, 3473484672,
   -4050548928, 4225288576, -4025455488, 3526095616, -2830091008,
   2044368384, -1274964480, 609164288, -94017536, -253966336,
   438857728, -488501248, 443699200, -343703552, 227893248,
   -127287296, 53002240, -4390912, -20742144, 27721728, -23855104,
   14680064, -5767168, 1048576
 ]};

gp_inline__location="../c-curve.tex:14405";
 gEnc(x) = {
 (8*x^14 - 68*x^15 + 300*x^16 - 954*x^17 + 2452*x^18 - 5200*x^19
  + 9034*x^20 - 12224*x^21 + 9792*x^22 + 7838*x^23 - 54172*x^24
  + 143306*x^25 - 283180*x^26 + 465990*x^27 - 662030*x^28
  + 820336*x^29 - 874602*x^30 + 758130*x^31 - 421378*x^32
  - 161426*x^33 + 995062*x^34 - 2070390*x^35 + 3388130*x^36
  - 4969042*x^37 + 6831766*x^38 - 8963930*x^39 + 11286454*x^40
  - 13600150*x^41 + 15581166*x^42 - 16798314*x^43 + 16720358*x^44
  - 14754138*x^45 + 10269470*x^46 - 2669470*x^47 - 8589918*x^48
  + 23887984*x^49 - 43290638*x^50 + 66479454*x^51 - 92540492*x^52
  + 119774034*x^53 - 145383702*x^54 + 165477380*x^55
  - 175076394*x^56 + 168826388*x^57 - 141997138*x^58
  + 91586332*x^59 - 16944584*x^60 - 79986494*x^61 + 194584868*x^62
  - 320333240*x^63 + 449109732*x^64 - 571731084*x^65
  + 678819732*x^66 - 762050064*x^67 + 815023248*x^68
  - 833899096*x^69 + 818155680*x^70 - 769781728*x^71
  + 692122144*x^72 - 590557600*x^73 + 472826560*x^74
  - 348473024*x^75 + 227695296*x^76 - 120595712*x^77
  + 33487744*x^78 + 30801664*x^79 - 71324416*x^80 + 89231872*x^81
  - 88659968*x^82 + 74721280*x^83 - 53735424*x^84 + 32323584*x^85
  - 14917632*x^86 + 2482176*x^87 + 4501504*x^88 - 7208960*x^89
  + 7266304*x^90 - 5881856*x^91 + 3719168*x^92 - 1867776*x^93
  + 720896*x^94 - 131072*x^95)
 / (1 - 14*x + 93*x^2 - 399*x^3 + 1264*x^4 - 3155*x^5 + 6331*x^6
  - 10024*x^7 + 11316*x^8 - 4211*x^9 - 20227*x^10 + 71426*x^11
  - 154220*x^12 + 261835*x^13 - 367331*x^14 + 415968*x^15
  - 322933*x^16 - 21513*x^17 + 733582*x^18 - 1895227*x^19
  + 3495150*x^20 - 5358532*x^21 + 7071828*x^22 - 7941210*x^23
  + 7025760*x^24 - 3250370*x^25 - 4373962*x^26 + 16408752*x^27
  - 32614928*x^28 + 51607318*x^29 - 70551590*x^30 + 85092320*x^31
  - 89643681*x^32 + 77941144*x^33 - 43977055*x^34 - 16590125*x^35
  + 104987428*x^36 - 217961315*x^37 + 346368491*x^38
  - 474573956*x^39 + 581114980*x^40 - 640107359*x^41
  + 624068137*x^42 - 508562062*x^43 + 276968364*x^44
  + 74978871*x^45 - 534289583*x^46 + 1068161080*x^47
  - 1625050667*x^48 + 2137499167*x^49 - 2528238492*x^50
  + 2721196999*x^51 - 2652570770*x^52 + 2280166842*x^53
  - 1593834682*x^54 + 621975798*x^55 + 569789848*x^56
  - 1881894252*x^57 + 3187362052*x^58 - 4349170900*x^59
  + 5238370752*x^60 - 5746468952*x^61 + 5802006776*x^62
  - 5384684200*x^63 + 4524006704*x^64 - 3297502496*x^65
  + 1829761968*x^66 - 271705376*x^67 - 1223138144*x^68
  + 2509938688*x^69 - 3473484672*x^70 + 4050548928*x^71
  - 4225288576*x^72 + 4025455488*x^73 - 3526095616*x^74
  + 2830091008*x^75 - 2044368384*x^76 + 1274964480*x^77
  - 609164288*x^78 + 94017536*x^79 + 253966336*x^80
  - 438857728*x^81 + 488501248*x^82 - 443699200*x^83
  + 343703552*x^84 - 227893248*x^85 + 127287296*x^86
  - 53002240*x^87 + 4390912*x^88 + 20742144*x^89 - 27721728*x^90
  + 23855104*x^91 - 14680064*x^92 + 5767168*x^93 - 1048576*x^94);
 }
 Enc_conf = vector(15,i, 1);  /* all filled */
 Enc_conf_index = conf_to_index(Enc_conf);
 Enc(k) = gf_term(gEnc(x),k);

gp_inline__location="../c-curve.tex:14596";
 gEnc_sans_2(x) = gEnc(x) - 1/(1-2*x);

gp_inline__location="../c-curve.tex:14644";
 conf_count_three(conf) = if(conf[1], conf[2]+conf[8]+conf[10], -1);
 gThree0(x) = conf_predicate_n_to_gf(conf_count_three,0);
 gThree1(x) = conf_predicate_n_to_gf(conf_count_three,1);
 gThree2(x) = conf_predicate_n_to_gf(conf_count_three,2);
 gThree3(x) = conf_predicate_n_to_gf(conf_count_three,3);
 gThree0 = memoize(gThree0);
 gThree1 = memoize(gThree1);
 gThree2 = memoize(gThree2);
 gThree3 = memoize(gThree3);
 Three0(k) = gf_term(gThree0(x), k);
 Three1(k) = gf_term(gThree1(x), k);
 Three2(k) = gf_term(gThree2(x), k);
 Three3(k) = gf_term(gThree3(x), k);

gp_inline__location="../c-curve.tex:14671";
 Three0_samples = {[1,2,4,6,10,18,36,68,126,234,448,844]};

gp_inline__location="../c-curve.tex:14678";
 Three1_samples = {[0,0,0,2,6,14,26,54,114,238,476,960]};

gp_inline__location="../c-curve.tex:14687";
 Three2_samples = {[0,0,0,0,0,0,2,6,16,40,98,234]};

gp_inline__location="../c-curve.tex:14703";
 Three3_zeros = {10};

gp_inline__location="../c-curve.tex:14707";
Three3_samples_sans_zeros = {[2,10,36,120,350]};
 Three3_samples = concat(vector(Three3_zeros), Three3_samples_sans_zeros);

gp_inline__location="../c-curve.tex:14715";
 PolyThree0(x) = gf_to_linear_recurrence_poly(gThree0(x));
 PolyThree1(x) = gf_to_linear_recurrence_poly(gThree1(x));
 PolyThree2(x) = gf_to_linear_recurrence_poly(gThree2(x));
 PolyThree3(x) = gf_to_linear_recurrence_poly(gThree3(x));

gp_inline__location="../c-curve.tex:14913";
 conf_is_single_seg(conf) = conf[1] && ! conf[2];
 conf_is_double_seg(conf) = conf[1] && conf[2];

gp_inline__location="../c-curve.tex:14921";
 conf_count_single_sample = {147};

gp_inline__location="../c-curve.tex:14927";
 conf_count_single_mirror_sample = {78};

gp_inline__location="../c-curve.tex:14940";
 conf_is_single_opposite_seg(conf) = ! conf[1] && conf[2];

gp_inline__location="../c-curve.tex:14950";
 conf_count_double_sample = {224};

gp_inline__location="../c-curve.tex:14956";
 conf_count_double_mirror_sample = {116};

gp_inline__location="../c-curve.tex:14979";
 conf_is_double34_seg(conf) =conf[3] && conf[4];

gp_inline__location="../c-curve.tex:14988";
 Tee_parts_vector = {[1,2,4,7]};

gp_inline__location="../c-curve.tex:14994";
 Tee_conf = conf_make(Tee_parts_vector);

gp_inline__location="../c-curve.tex:15048";
Tee_samples = {[0,0,0,0,1,2,4,6,13,22,40,64,119,205]};

gp_inline__location="../c-curve.tex:15056";
 TeePoly(x) = {
   (x-1)^2 * (x+1)^2 * (x^2 + 1)
   * (x^4 + 1)^3 * (x^4 - 2)^2 * poly8(x) * poly8alt(x)
   * poly9(x) * poly11(x) * poly14(x) * poly18(x);
 }

gp_inline__location="../c-curve.tex:15064";
 Tee_linear_recurrence_factors = {[11, -57, 192, -481, 929, -1318,
   1057, 822, -5461, 13516, -24167, 34460, -38695, 28106,
   7575, -77885, 188676, -334717, 492211, -615501, 632380,
   -448536, -27690, 860070, -2055250, 3513832, -5007126,
   6191924, -6608542, 5727188, -3086354, -1621347,
   8459843, -17044403, 26478374, -35440555, 42092777,
   -44221622, 39762221, -27016582, 4959215, 25957652,
   -63590771, 104412156, -143122923, 173064866,
   -187741765, 181406057, -149636386, 91466553, -10025401,
   -88677031, 195049866, -296232428, 379394028,
   -432977590, 446149224, -413169352, 335470144,
   -218259564, 71579072, 87620320, -242081936, 376513624,
   -473476960, 520786608, -516483376, 461876768,
   -363047008, 237451968, -102160320, -29580992,
   140671104, -218017664, 259969280, -265784064,
   237835776, -190128640, 134889472, -78204928, 28481536,
   7319552, -30162944, 39669760, -36470784, 27664384,
   -18956288, 10698752, -3735552, -491520, 2883584,
   -3407872, 2097152, -524288 ]};

gp_inline__location="../c-curve.tex:15087";
 gTee(x) = (x^4 - 9*x^5 + 39*x^6 - 116*x^7 + 272*x^8 - 514*x^9 + 771*x^10 - 869*x^11 + 486*x^12 + 866*x^13 - 3672*x^14 + 8281*x^15 - 14767*x^16 + 22274*x^17 - 28606*x^18 + 30305*x^19 - 22046*x^20 - 2832*x^21 + 49519*x^22 - 120575*x^23 + 214279*x^24 - 318779*x^25 + 412240*x^26 - 466244*x^27 + 441744*x^28 - 294066*x^29 - 7734*x^30 + 481180*x^31 - 1125081*x^32 + 1881710*x^33 - 2645265*x^34 + 3281680*x^35 - 3599184*x^36 + 3370939*x^37 - 2448998*x^38 + 738058*x^39 + 1785903*x^40 - 4908612*x^41 + 8242376*x^42 - 11346023*x^43 + 13593395*x^44 - 14209010*x^45 + 12703847*x^46 - 8675043*x^47 + 1854638*x^48 + 7396850*x^49 - 18083769*x^50 + 29164859*x^51 - 39065611*x^52 + 45678998*x^53 - 47609688*x^54 + 43811993*x^55 - 33223446*x^56 + 15921977*x^57 + 6225093*x^58 - 31751408*x^59 + 58094407*x^60 - 81360766*x^61 + 98564783*x^62 - 108416482*x^63 + 108420538*x^64 - 97925466*x^65 + 79368008*x^66 - 54174220*x^67 + 24254376*x^68 + 5426472*x^69 - 30716224*x^70 + 51327872*x^71 - 63878960*x^72 + 67285280*x^73 - 63710272*x^74 + 55061888*x^75 - 40491008*x^76 + 24909440*x^77 - 11230848*x^78 - 742144*x^79 + 9696768*x^80 - 13697536*x^81 + 14907392*x^82 - 14686208*x^83 + 11451392*x^84 - 7497728*x^85 + 4423680*x^86 - 2031616*x^87 - 106496*x^88 + 671744*x^89 - 827392*x^90 + 1015808*x^91 - 819200*x^92 + 262144*x^93)/(1 - 11*x + 57*x^2 - 192*x^3 + 481*x^4 - 929*x^5 + 1318*x^6 - 1057*x^7 - 822*x^8 + 5461*x^9 - 13516*x^10 + 24167*x^11 - 34460*x^12 + 38695*x^13 - 28106*x^14 - 7575*x^15 + 77885*x^16 - 188676*x^17 + 334717*x^18 - 492211*x^19 + 615501*x^20 - 632380*x^21 + 448536*x^22 + 27690*x^23 - 860070*x^24 + 2055250*x^25 - 3513832*x^26 + 5007126*x^27 - 6191924*x^28 + 6608542*x^29 - 5727188*x^30 + 3086354*x^31 + 1621347*x^32 - 8459843*x^33 + 17044403*x^34 - 26478374*x^35 + 35440555*x^36 - 42092777*x^37 + 44221622*x^38 - 39762221*x^39 + 27016582*x^40 - 4959215*x^41 - 25957652*x^42 + 63590771*x^43 - 104412156*x^44 + 143122923*x^45 - 173064866*x^46 + 187741765*x^47 - 181406057*x^48 + 149636386*x^49 - 91466553*x^50 + 10025401*x^51 + 88677031*x^52 - 195049866*x^53 + 296232428*x^54 - 379394028*x^55 + 432977590*x^56 - 446149224*x^57 + 413169352*x^58 - 335470144*x^59 + 218259564*x^60 - 71579072*x^61 - 87620320*x^62 + 242081936*x^63 - 376513624*x^64 + 473476960*x^65 - 520786608*x^66 + 516483376*x^67 - 461876768*x^68 + 363047008*x^69 - 237451968*x^70 + 102160320*x^71 + 29580992*x^72 - 140671104*x^73 + 218017664*x^74 - 259969280*x^75 + 265784064*x^76 - 237835776*x^77 + 190128640*x^78 - 134889472*x^79 + 78204928*x^80 - 28481536*x^81 - 7319552*x^82 + 30162944*x^83 - 39669760*x^84 + 36470784*x^85 - 27664384*x^86 + 18956288*x^87 - 10698752*x^88 + 3735552*x^89 + 491520*x^90 - 2883584*x^91 + 3407872*x^92 - 2097152*x^93 + 524288*x^94);

gp_inline__location="../c-curve.tex:15100";
 conf_count_visits(conf) = if(conf[1],1+conf[11]+conf[13]+conf[15]);
 gV1(x) = conf_predicate_n_to_gf(conf_count_visits,1) + gOnes(x);
 gV2(x) = conf_predicate_n_to_gf(conf_count_visits,2)/2;
 gV3(x) = conf_predicate_n_to_gf(conf_count_visits,3)/3;
 gV4(x) = conf_predicate_n_to_gf(conf_count_visits,4)/4;
 gV1 = memoize(gV1);
 gV2 = memoize(gV2);
 gV3 = memoize(gV3);
 gV4 = memoize(gV4);
 V1(k) = gf_term(gV1(x), k);
 V2(k) = gf_term(gV2(x), k);
 V3(k) = gf_term(gV3(x), k);
 V4(k) = gf_term(gV4(x), k);

gp_inline__location="../c-curve.tex:15114";
 conf_count_visits_to(conf) = if(conf[1],1+conf[3]+conf[5]+conf[7]);

gp_inline__location="../c-curve.tex:15120";
 conf_count_visits2(conf) = if(conf[2],1+conf[10]+conf[12]+conf[14]);

gp_inline__location="../c-curve.tex:15126";
 conf_count_visits_to2(conf) = if(conf[2],1+conf[4]+conf[6]+conf[8]);

gp_inline__location="../c-curve.tex:15178";
V1_samples = {[2,3,5,9,15,27,49,93,171,321,601,1137]};

gp_inline__location="../c-curve.tex:15185";
V2_samples = {[0,0,0,0,1,3,8,18,43,93,200,414]};

gp_inline__location="../c-curve.tex:15202";
 V3_zeros = {9};

gp_inline__location="../c-curve.tex:15206";
V3_samples_sans_zeros = {[2,8,28,84,238,596]};
 V3_samples = concat(vector(V3_zeros), V3_samples_sans_zeros);

gp_inline__location="../c-curve.tex:15215";
 V4_zeros = {12};

gp_inline__location="../c-curve.tex:15219";
 V4_samples_sans_zeros = {[1,5,24,78,232,626,1648]};
 V4_samples = concat(vector(V4_zeros), V4_samples_sans_zeros);

gp_inline__location="../c-curve.tex:15230";
 V1_recurrence_initial = {[
   2, 3, 5, 9, 15, 27, 49, 93, 171, 321, 601, 1137, 2135,
   4045, 7653, 14537, 27575, 52489, 99889, 190505, 363383,
   694421, 1327661, 2541785, 4868727, 9336417, 17914121,
   34403913, 66110567, 127140621, 244653877, 471119241,
   907733095, 1750124361, 3376138209
 ]};
 V1_recurrence_factors = {[
   8, -28, 59, -83, 56, 90, -377, 731, -1012, 982, -399,
   -701, 2072, -3370, 4037, -3766, 2900, -1706, 256, 1052,
   -2248, 4000, -6104, 7824, -9968, 11872, -11872, 11136,
   -9920, 6144, -2176, -512, 2816, -3072, 1024
 ]};

gp_inline__location="../c-curve.tex:15256";
 PolyV1(x) = gf_to_linear_recurrence_poly(gV1(x));
 PolyV2(x) = gf_to_linear_recurrence_poly(gV2(x));
 PolyV3(x) = gf_to_linear_recurrence_poly(gV3(x));
 PolyV4(x) = gf_to_linear_recurrence_poly(gV4(x));

gp_inline__location="../c-curve.tex:15313";
 gV4_sans_2(x) = gV4(x) - 1/4 * 1/(1-2*x);

gp_inline__location="../c-curve.tex:15317";
 my_pol_partial_fractions_part(g,p) = {
   my(v=pol_partial_fractions(g,[p]));
   for(i=1,#v, if(denominator(v[i])%p==0, return(v[i])));
   0;
 }

gp_inline__location="../c-curve.tex:15323";
 gf_poly8alt_part(g) = my_pol_partial_fractions_part(g,revpoly8alt(x));
 gV1_poly8alt_part(x) = gf_poly8alt_part(gV1(x));
 gV2_poly8alt_part(x) = gf_poly8alt_part(gV2(x));
 gV3_poly8alt_part(x) = gf_poly8alt_part(gV3(x));
 gV4_poly8alt_part(x) = gf_poly8alt_part(gV4(x));

gp_inline__location="../c-curve.tex:15329";
 gf_poly9_part(g) = my_pol_partial_fractions_part(g,revpoly9(x));
 gV1_poly9_part(x) = gf_poly9_part(gV1(x));
 gV2_poly9_part(x) = gf_poly9_part(gV2(x));
 gV3_poly9_part(x) = gf_poly9_part(gV3(x));
 gV4_poly9_part(x) = gf_poly9_part(gV4(x));

gp_inline__location="../c-curve.tex:15335";
 gf_poly11_part(g) = my_pol_partial_fractions_part(g,revpoly11(x));

gp_inline__location="../c-curve.tex:15337";
 gf_poly18_part(g) = my_pol_partial_fractions_part(g,revpoly18(x));
 gV1_poly18_part(x) = gf_poly18_part(gV1(x));
 gV2_poly18_part(x) = gf_poly18_part(gV2(x));
 gV3_poly18_part(x) = gf_poly18_part(gV3(x));
 gV4_poly18_part(x) = gf_poly18_part(gV4(x));

gp_inline__location="../c-curve.tex:15367";
 V1_poly9_part(k) = gf_term(gV1_poly9_part(x),k);
 V2_poly9_part(k) = gf_term(gV2_poly9_part(x),k);
 V3_poly9_part(k) = gf_term(gV3_poly9_part(x),k);
 V1_poly8alt_part(k) = gf_term(gV1_poly8alt_part(x),k);
 V2_poly8alt_part(k) = gf_term(gV2_poly8alt_part(x),k);
 V3_poly8alt_part(k) = gf_term(gV3_poly8alt_part(x),k);

gp_inline__location="../c-curve.tex:15398";
 V1sub3(k) = V1(k) - V3(k);
 gV1sub3(x) = gV1(x) - gV3(x);

gp_inline__location="../c-curve.tex:15402";
 V1sub3_samples = {[2,3,5,9,15,27,49,93,171,319,593,1109]};

gp_inline__location="../c-curve.tex:15420";
 V1sub3_recurrence_factors = {[
   5, -12, 21, -28, 25, -8, -19, 53, -90,
   112, -118, 108, -88, 48, -16, -8, 32, -32
 ]};

gp_inline__location="../c-curve.tex:15427";
 V1sub3_by_recurrence(k) = {
   if(k<18, V1sub3(k),
      sum(i=1,#V1sub3_recurrence_factors,
          V1sub3_recurrence_factors[i] * V1sub3(k-i)));
 }

gp_inline__location="../c-curve.tex:15450";
 {
   my(m = matrix(18,18, i,j, poly18_roots[j]^(i-1)),
      v = vectorv(18,k,k--; V1sub3(k)),
      f = matsolve(m,v));
   V1sub3_power_coefficients_complex = Vec(f);
 }
 V1sub3_power_coefficients = V1sub3_power_coefficients_complex;
 /*real*/ V1sub3_power_coefficients[1] = real(V1sub3_power_coefficients[1]);
 V1sub3_by_powers(k) = \
   sum(i=1,18, V1sub3_power_coefficients[i]*poly18_roots[i]^k);

gp_inline__location="../c-curve.tex:15473";
 poly18_f18_sample = {(1319372/1000000)};
 poly18_f18 = real(V1sub3_power_coefficients[18]);

gp_inline__location="../c-curve.tex:15494";
 V1sub3_other_magnitudes(k) = {
   sum(i=1,17,
       abs(V1sub3_power_coefficients[i])
       * abs(poly18_roots[i]/poly18_r18)^k );
 }

gp_inline__location="../c-curve.tex:15503";
 V1sub3_by_powers_sans_r18(k) = \
   sum(i=1,17, V1sub3_power_coefficients[i]*poly18_roots[i]^k);

gp_inline__location="../c-curve.tex:15535";
 V2_over_V1_coefficients = \
   [-34/15, 1/12, -1/10, 43/120, 1/4, -31/240, 19/80, -3/16, 3/80];
 V2_over_V1_limit = {
   sum(i=1,#V2_over_V1_coefficients,
       V2_over_V1_coefficients[i] * r^(i-1));
 }
 V2_over_V1_nfelt = {
   sum(i=1,#V2_over_V1_coefficients,
       V2_over_V1_coefficients[i] * nfeltpow(poly9_nf,r_nfelt,i-1));
 }

gp_inline__location="../c-curve.tex:15550";
 V2_over_V1_limit_sample = {(0950615/1000000)};

gp_inline__location="../c-curve.tex:15568";
 V1_over_V2_nfelt = nfeltdiv(poly9_nf, 1, V2_over_V1_nfelt);

gp_inline__location="../c-curve.tex:15587";
 gV(x) = gV1(x) + gV2(x) + gV3(x) + gV4(x);
 V(k) = gf_term(gV(x), k);
 PolyV(x) = gf_to_linear_recurrence_poly(gV(x));

gp_inline__location="../c-curve.tex:15593";
V_samples = {[2,3,5,9,16,30,57,111,214,416,809,1579]};

gp_inline__location="../c-curve.tex:15658";
 SD(k) = S(k) + D(k);
 SDpart(k) = part_from_whole(SD,k);

gp_inline__location="../c-curve.tex:15661";
 Vpart(k) = part_from_whole_overlap1(V,k);

gp_inline__location="../c-curve.tex:15701";
 conf_count_QI_sides(conf) = if(conf[1],1+conf[8]+conf[9]+conf[10]);
 gQI1(x) = conf_predicate_n_to_gf(conf_count_QI_sides,1);
 gQI2(x) = conf_predicate_n_to_gf(conf_count_QI_sides,2)/2;
 gQI3(x) = conf_predicate_n_to_gf(conf_count_QI_sides,3)/3;
 gQI4(x) = conf_predicate_n_to_gf(conf_count_QI_sides,4)/4;
 gQI1 = memoize(gQI1);
 gQI2 = memoize(gQI2);
 gQI3 = memoize(gQI3);
 gQI4 = memoize(gQI4);
 QI1(k) = gf_term(gQI1(x), k);
 QI2(k) = gf_term(gQI2(x), k);
 QI3(k) = gf_term(gQI3(x), k);
 QI4(k) = gf_term(gQI4(x), k);
 PolyQI1(x) = gf_to_linear_recurrence_poly(gQI1(x));
 PolyQI2(x) = gf_to_linear_recurrence_poly(gQI2(x));
 PolyQI3(x) = gf_to_linear_recurrence_poly(gQI3(x));
 PolyQI4(x) = gf_to_linear_recurrence_poly(gQI4(x));

gp_inline__location="../c-curve.tex:15772";
 QI1_samples = {[1,2,4,6,12,22,44,78,150,280,536,998]};

gp_inline__location="../c-curve.tex:15779";
QI2_samples = {[0,0,0,1,2,5,10,25,50,107,214,439]};

gp_inline__location="../c-curve.tex:15788";
 QI3_zeros = {8};

gp_inline__location="../c-curve.tex:15792";
QI3_samples_sans_zeros = {[2,6,20,56,154,358,848]};
 QI3_samples = concat(vector(QI3_zeros), QI3_samples_sans_zeros);

gp_inline__location="../c-curve.tex:15801";
 QI4_zeros = {11};

gp_inline__location="../c-curve.tex:15805";
 QI4_samples_sans_zeros = {[1,4,19,54,154,394,1022,2492]};
 QI4_samples = concat(vector(QI4_zeros), QI4_samples_sans_zeros);

gp_inline__location="../c-curve.tex:15980";
 gQI(x) = gQI1(x) + gQI2(x) + gQI3(x) + gQI4(x);
 QI(k) = gf_term(gQI(x), k);
 PolyQI(x) = gf_to_linear_recurrence_poly(gQI(x));

gp_inline__location="../c-curve.tex:15985";
QI_samples = {[1,2,4,7,14,27,54,103,202,393,770,1494]};

gp_inline__location="../c-curve.tex:16084";
 conf_is_QI2a(conf) = conf[1] && conf[8] && !conf[9] && !conf[10];
 conf_is_QI2o(conf) = conf[1] && !conf[8] && conf[9] && !conf[10];
 gQI2a(x) = conf_predicate_to_gf(conf_is_QI2a);
 gQI2o(x) = conf_predicate_to_gf(conf_is_QI2o)/2;
 gQI2a = memoize(gQI2a);
 gQI2o = memoize(gQI2o);
 QI2a(k) = gf_term(gQI2a(x), k);
 QI2o(k) = gf_term(gQI2o(x), k);
 PolyQI2a(x) = gf_to_linear_recurrence_poly(gQI2a(x));
 PolyQI2o(x) = gf_to_linear_recurrence_poly(gQI2o(x));

gp_inline__location="../c-curve.tex:16136";
 QI2a_samples = {[0,0,0,1,2,5,10,23,46,97,190,383]};

gp_inline__location="../c-curve.tex:16143";
QI2o_samples = {[0,0,0,0,0,0,0,2,4,10,24,56]};

gp_inline__location="../c-curve.tex:16172";
 QI2a_over_QI2_coefficients = {
   gf_in_terms_of(gf_poly9_part(gQI2a(x)),
                  gf_poly9_part(gQI2(x)));
 }
 QI2a_over_QI2_limit = {
   sum(i=1,#QI2a_over_QI2_coefficients,
       QI2a_over_QI2_coefficients[i]*r^(i-1));
 }
 QI2a_over_QI2_nfelt = {
   sum(i=1,#QI2a_over_QI2_coefficients,
       QI2a_over_QI2_coefficients[i]
       * nfeltpow(poly9_nf,r_nfelt,i-1));
 }

gp_inline__location="../c-curve.tex:16227";
 QI2a_over_QI2_limit_sample = {(0856584/1000000)};

gp_inline__location="../c-curve.tex:16242";
 QI2o_over_QI2_coefficients = {
   gf_in_terms_of(gf_poly9_part(gQI2o(x)),
                  gf_poly9_part(gQI2(x)));
 }
 QI2o_over_QI2_limit = {
   sum(i=1,#QI2o_over_QI2_coefficients,
       QI2o_over_QI2_coefficients[i]*r^(i-1));
 }
 QI2o_over_QI2_nfelt = {
   sum(i=1,#QI2o_over_QI2_coefficients,
       QI2o_over_QI2_coefficients[i]
       * nfeltpow(poly9_nf,r_nfelt,i-1));
 }

gp_inline__location="../c-curve.tex:16267";
 QI2o_over_QI2_limit_sample = {(0143415/1000000)};

gp_inline__location="../c-curve.tex:16277";
 QI2a_over_QI2o_coefficients = {
   gf_in_terms_of(gf_poly9_part(gQI2a(x)),
                  gf_poly9_part(gQI2o(x)));
 }
 QI2a_over_QI2o_limit = {
   sum(i=1,#QI2a_over_QI2o_coefficients,
       QI2a_over_QI2o_coefficients[i]*r^(i-1));
 }

gp_inline__location="../c-curve.tex:16292";
 QI2a_over_QI2o_limit_sample = {(5972726/1000000)};

gp_inline__location="../c-curve.tex:16520";
 gV2leftA(x) = x/(1-x^4)*gQI2a(x);

gp_inline__location="../c-curve.tex:16522";
V2leftA_samples = {[0,0,0,0,1,2,5,10,24,48,102,200,407,794,1591]};

gp_inline__location="../c-curve.tex:16532";
 gV2leftO(x) = x/(1-x^4)*gQI2o(x);

gp_inline__location="../c-curve.tex:16536";
V2leftO_samples = {[0,0,0,0,0,0,0,0,2,4,10,24,58,112,238]};

gp_inline__location="../c-curve.tex:16550";
 gV2strA(x) = x*gV2leftA(x);

gp_inline__location="../c-curve.tex:16553";
 gV2strO(x) = x*gV2leftO(x);

gp_inline__location="../c-curve.tex:16555";
 gV2rightA(x) = x^2*gV2leftA(x);

gp_inline__location="../c-curve.tex:16562";
 gV2rightO(x) = x^2*gV2leftO(x);

gp_inline__location="../c-curve.tex:16565";
 gV2revA(x) = x^3*gV2leftA(x);

gp_inline__location="../c-curve.tex:16568";
 gV2revO(x) = x^3*gV2leftO(x);

gp_inline__location="../c-curve.tex:16593";
conf_is_V2leftA(conf) = {
  conf[1] && conf[2] && conf[4] && conf[7]
  && !conf[3] && !conf[5] && !conf[6] && !conf[8];
}

gp_inline__location="../c-curve.tex:16602";
conf_is_V2rightA(conf) = {
  conf[1] && conf[2] && conf[3] && conf[8]
  && !conf[4] && !conf[5] && !conf[6] && !conf[7];
}

gp_inline__location="../c-curve.tex:16609";
conf_is_V2strA(conf) = {
 conf[1] && conf[6] && conf[3] && conf[8]
 && !conf[2] && !conf[4]  && !conf[5] && !conf[7];
}

gp_inline__location="../c-curve.tex:16616";
conf_is_V2revA(conf) = {
 conf[1] && conf[2] && conf[7] && conf[8]
 && !conf[3] && !conf[4]  && !conf[5] && !conf[6];
}

gp_inline__location="../c-curve.tex:16632";
conf_is_V2strO_or_V2revO(conf) = {
  conf[1] && conf[6] && conf[2] && conf[5]
  && !conf[3] && !conf[8] && !conf[4] && !conf[7];
}

gp_inline__location="../c-curve.tex:16639";
conf_is_V2leftO_or_V2rightO(conf) = {
  conf[1] && conf[8] && conf[4] && conf[5]
  && !conf[6] && !conf[7] && !conf[2] && !conf[3];
}

gp_inline__location="../c-curve.tex:16662";
 VisitsN(k,n) = {
   n<=2^k || error("VisitsN k=",k," n=",n);
   VisitsZ(k,c_curve_point(n));
 }
 VisitsZ(k,z) = {
   my(limit=2^k);
   sum(d=0,3,
       my(n=c_curve_unpoint(z,d));
       n>=0 && n<=limit);
 }

gp_inline__location="../c-curve.tex:16691";
 gen_Q_values_Q1_samples = [2,2,4,6,10,16,32,54,98,166,312,552,1024];
 gen_Q_values_Q2_samples = [0,1,2,5,10,17,30,60,116,222,410,789,1496];
 gen_Q_values_Q3_samples = [0,0,0,0,0,2,6,14,32,74,166,350,726];
 gen_Q_values_Q4_samples = [0,0,0,0,0,1,2,5,10,23,50,114,250];
 gen_Q_values_Q_samples = [2,3,6,11,20,36,70,133,256,485,938,1805,3496];

gp_inline__location="../c-curve.tex:16698";
 conf_count_Q_sides(conf) = {
   \\ if(! (conf[1] || conf[2]), return(0));
       (conf[1] || conf[2])
     + (conf[7] || conf[8])
     + (conf[9] || ((conf[2]+conf[7]+conf[11])%2))
     + (conf[10] || conf[11]);
 }
 gQ1(x) = {
   my(g = conf_predicate_n_to_gf(conf_count_Q_sides,1)/4);
   g - gf_term(g,0) + 2;   \\ force Q1(0)==2
 }
 gQ2(x) = {
   my(g = conf_predicate_n_to_gf(conf_count_Q_sides,2)/4);
   g - gf_term(g,0);       \\ force Q2(0)==0
 }
 gQ3(x) = conf_predicate_n_to_gf(conf_count_Q_sides,3)/4;
 gQ4(x) = conf_predicate_n_to_gf(conf_count_Q_sides,4)/4;
 gQ1 = memoize(gQ1);
 gQ2 = memoize(gQ2);
 gQ3 = memoize(gQ3);
 gQ4 = memoize(gQ4);
 Q1(k) = gf_term(gQ1(x), k);
 Q2(k) = gf_term(gQ2(x), k);
 Q3(k) = gf_term(gQ3(x), k);
 Q4(k) = gf_term(gQ4(x), k);
 Q1 = memoize(Q1);
 Q2 = memoize(Q2);
 Q3 = memoize(Q3);
 Q4 = memoize(Q4);

gp_inline__location="../c-curve.tex:16794";
 Q1_samples = {[2,2,4,6,10,16,32,54,98,166,312,552]};

gp_inline__location="../c-curve.tex:16803";
Q2_samples = {[0,1,2,5,10,17,30,60,116,222,410,789]};

gp_inline__location="../c-curve.tex:16812";
Q3_samples = {[0,0,0,0,0,2,6,14,32,74,166,350]};

gp_inline__location="../c-curve.tex:16826";
 Q4_samples = {[0,0,0,0,0,1,2,5,10,23,50,114]};

gp_inline__location="../c-curve.tex:16846";
 PolyQ1(x) = gf_to_linear_recurrence_poly(gQ1(x));
 PolyQ2(x) = gf_to_linear_recurrence_poly(gQ2(x));
 PolyQ3(x) = gf_to_linear_recurrence_poly(gQ3(x));
 PolyQ4(x) = gf_to_linear_recurrence_poly(gQ4(x));

gp_inline__location="../c-curve.tex:16899";
 Q1_recurrence_factors = - Vec(PolyQ1(x))[2..-1];
 Q1_by_recurrence(k) = \
  sum(i=1,#Q1_recurrence_factors,Q1_recurrence_factors[i]*Q1(k-i));

gp_inline__location="../c-curve.tex:16906";
 Q2_by_recurrence(k) = /* same factors as Q1 */ \
  sum(i=1,#Q1_recurrence_factors,Q1_recurrence_factors[i]*Q2(k-i));

gp_inline__location="../c-curve.tex:16941";
 Q4_mod_2_vector = {[0,1,0,0,0,0,0,1]};

gp_inline__location="../c-curve.tex:16946";
 Q4_mod_2_formula(k) = vector_modulo(Q4_mod_2_vector,k);
 Q4_mod_2(k) = if(k==1,0, k==5,1, Q4_mod_2_formula(k));

gp_inline__location="../c-curve.tex:16956";
 A173858_samples_mangled = A173858_samples;
 /* */ A173858_samples_mangled[1] = 0;
 /* */ A173858_samples_mangled[2] = 1;

gp_inline__location="../c-curve.tex:16965";
 gQ4_mod_2_formula(x) = Polrev(Q4_mod_2_vector)/(1-x^8);
 \\ Bruno Berselli in A173858
 gA173858(x) = x*(1-x^2+x^3+x^4-x^6)/(1-x^2+x^4-x^6);

gp_inline__location="../c-curve.tex:16973";
 sqrt5 = geom_sqrt_as_quad(5);
 phi = (1 + sqrt5)/2;

gp_inline__location="../c-curve.tex:16981";
 A236677(n) = if(n==0,1, (1-A236677(floor_log2(n))) * A236677(sans_high_bit(n)));

gp_inline__location="../c-curve.tex:17006";
 gQ(x) = gQ1(x) + gQ2(x) + gQ3(x) + gQ4(x);
 Q(k) = gf_term(gQ(x), k);
 PolyQ(x) = gf_to_linear_recurrence_poly(gQ(x));

gp_inline__location="../c-curve.tex:17011";
Q_samples = {[2,3,6,11,20,36,70,133,256,485,938,1805]};

gp_inline__location="../c-curve.tex:17134";
 NonQ(k) = S(k)+D(k) - V(k) - Q4(k) + 1;
 gNonQ(x) = gS(x)+gD(x) - gV(x) - gQ4(x) + gOnes(x);

gp_inline__location="../c-curve.tex:17140";
NonQ_samples = {[0,0,0,0,0,0,1,3,10,24,57,126]};

gp_inline__location="../c-curve.tex:17154";
 PolyNonQ(x) = gf_to_linear_recurrence_poly(gNonQ(x));

gp_inline__location="../c-curve.tex:17165";
 Acount(k) = Q(k) + NonQ(k);
Acount_samples = {[0,0,0,0,0,1,3,8,20,47,107,240]};

gp_inline__location="../c-curve.tex:17173";
 gAcount(x) = gQ(x) + gNonQ(x);
 PolyAcount(x) = gf_to_linear_recurrence_poly(gAcount(x));

gp_inline__location="../c-curve.tex:17237";
 gen_QD_values_QD2_samples = [0,0,0,0,0,0,0,0,0,2,4,10,24,58,112,240,484,998,1944,3922,7716];
 gen_QD_values_QD4_samples = [0,0,0,0,0,0,0,0,0,0,0,0,0,2,10,40,122,350,936,2480,6260];

gp_inline__location="../c-curve.tex:17241";
 conf_count_QD_sides(conf) = {
   my(e = (conf[2]+conf[7]+conf[11]) % 2);
   if(   conf[1] != conf[2]
      || conf[7] != conf[8]
      || conf[10] != conf[11]
      || conf[9] != e,
      return(0));
    conf[1] + conf[7] + conf[9] + conf[10];
 }
 gQD1(x) = conf_predicate_n_to_gf(conf_count_QD_sides,1)/4;
 gQD2(x) = conf_predicate_n_to_gf(conf_count_QD_sides,2)/4;
 gQD3(x) = conf_predicate_n_to_gf(conf_count_QD_sides,3)/4;
 gQD4(x) = conf_predicate_n_to_gf(conf_count_QD_sides,4)/4;
 gQD2 = memoize(gQD2);
 gQD4 = memoize(gQD4);
 QD2(k) = gf_term(gQD2(x), k);
 QD4(k) = gf_term(gQD4(x), k);
 QD2 = memoize(QD2);
 QD4 = memoize(QD4);

gp_inline__location="../c-curve.tex:17263";
 conf_is_QD2s(conf) = {
   my(e = (conf[2]+conf[7]+conf[11]) % 2);
      conf[1] && conf[2]
   && conf[7] && conf[8]
   && !conf[10] && !conf[11]
   && !conf[9] && !e;
 }
 gQD2s(x) = conf_predicate_to_gf(conf_is_QD2s);
 conf_is_QD2o(conf) = {
   my(e = (conf[2]+conf[7]+conf[11]) % 2);
      conf[1] && conf[2]
   && !conf[7] && !conf[8]
   && !conf[10] && !conf[11]
   && conf[9] && e;
 }
 gQD2o(x) = conf_predicate_to_gf(conf_is_QD2o)/2;

gp_inline__location="../c-curve.tex:17325";
 QD2_zeros = {9};

gp_inline__location="../c-curve.tex:17329";
QD2_samples_sans_zeros = {[2,4,10,24,58,112,240,484]};
 QD2_samples = concat(vector(QD2_zeros), QD2_samples_sans_zeros);

gp_inline__location="../c-curve.tex:17339";
 QD4_zeros = {13};

gp_inline__location="../c-curve.tex:17343";
 QD4_samples_sans_zeros = {[2,10,40,122,350,936,2480]};
 QD4_samples = concat(vector(QD4_zeros), QD4_samples_sans_zeros);

gp_inline__location="../c-curve.tex:17356";
 PolyQD2(x) = gf_to_linear_recurrence_poly(gQD2(x));
 PolyQD4(x) = gf_to_linear_recurrence_poly(gQD4(x));

gp_inline__location="../c-curve.tex:17491";
 Trigons_generated_samples = [1,2,4,7,13,25,49,95,183,353,685,1320,2532,4865,9355,17975,34529,66401,127753,245938,473582];

gp_inline__location="../c-curve.tex:17495";
 Trigons_part_generated_samples = [1,0,2,1,5,7,17,29,59,111,221,414,798,1535,2955,5665,10889,20983,40369,77816,149828];

gp_inline__location="../c-curve.tex:17552";
 conf_1_empty_enclosed = vector(15,n, n!=1);

gp_inline__location="../c-curve.tex:17557";
 conf_1_empty_enclosed_sans_9 = vector(15,n, n!=1 && n!=9);

gp_inline__location="../c-curve.tex:17686";
 Trigons(k) = \
   1/2*ThreeB(k) - 1/4*(V1(k+1) + 2*V2(k+1) + 3*V3(k+1) - 1);
 gTrigons(x) = {
   1/2*gThreeB(x)
   - 1/4*( 1/x*(gV1(x)-V1(0)) + 2*1/x*gV2(x)
                              + 3*1/x*gV3(x) - gOnes(x) );
 }

gp_inline__location="../c-curve.tex:17702";
 Trigons_samples = {[1,2,4,7,13,25,49,95,183,353,685,1320]};

gp_inline__location="../c-curve.tex:17717";
 TrigonsPoly(x) = poly8alt(x) * poly9(x) * poly18(x);

gp_inline__location="../c-curve.tex:17736";
 \\ v is a vector of 0s and 1s.
 \\ Return the number of runs of 1s occurring.
 \\ The vector is taken as wrapping around so the last element is
 \\ followed by the first.
 vector_cycle_count_run_1s(v) = \
   if(#v==0,0, sum(i=1,#v-1, v[i+1] > v[i]) + (v[1] > v[#v]));

gp_inline__location="../c-curve.tex:17746";
 conf_count_turn_end(conf) = {
   my(v = conf[1..8], runs = vector_cycle_count_run_1s(v));
   if(runs, 4*runs - vecsum(v));
 }

gp_inline__location="../c-curve.tex:17763";
 conf_count_turn_middle(conf) = {
   my(v = [conf[1],conf[8],conf[9],conf[10]],
      runs = vector_cycle_count_run_1s(v));
   if(runs, 4*runs - 2*vecsum(v));
 }

gp_inline__location="../c-curve.tex:17773";
 conf_polygons_net_end(conf) = {
   my(reps = conf[1]+conf[3]+conf[5]+conf[7]);
   if(0<reps && reps<8, conf_count_turn_end(conf) / reps);
 }
 conf_polygons_net_middle(conf) = {
   my(reps = conf[1]+conf[8]+conf[9]+conf[10]);
   if(0<reps && reps<4, conf_count_turn_middle(conf) / reps);
 }
 conf_polygons_net(conf) = \
   conf_polygons_net_end(conf) + conf_polygons_net_middle(conf);

gp_inline__location="../c-curve.tex:17788";
 gTrigons_net_turn(x) = {
   3*gOnes(x) +
   sum(i=1,#conf_occurring_vector,
       my(conf = conf_occurring_vector[i]);
       if(conf[1],
          conf_to_gf(conf) * conf_polygons_net(conf)));
 }

gp_inline__location="../c-curve.tex:17874";
 Trigons_by_Euler(k) = 2^k - 1/2*ThreeNonB(k) + V4(k) + QI4(k);

gp_inline__location="../c-curve.tex:17931";
 Untrigons_part(k) = part_from_whole_overlap1(Untrigons,k);

gp_inline__location="../c-curve.tex:17965";
 Untrigons(k) = \
   1/2*ThreeB(k) - 1/4*(3*V1(k+1) + 2*V2(k+1) + V3(k+1) - 3);

gp_inline__location="../c-curve.tex:17968";
 gUntrigons(x) = {
   1/2*gThreeB(x)
   - 1/4*( 3*1/x*(gV1(x)-V1(0)) + 2*1/x*gV2(x) + 1/x*gV3(x)
           - 3*gOnes(x));
 }

gp_inline__location="../c-curve.tex:17981";
 Untrigons_samples = {[0,0,0,0,0,1,3,10,24,57,131,295]};

gp_inline__location="../c-curve.tex:18010";
 UntrigonsPoly = gf_to_linear_recurrence_poly_func(gTrigons(x));

gp_inline__location="../c-curve.tex:18029";
 Trigons_sub_Untrigons_samples = {[1,2,4,7,13,24,46,85,159,296,554,1025]};

gp_inline__location="../c-curve.tex:18070";
 gen_Touches_Trigons_samples = [1,2,4,7,13,25,49,95,183,353,685,1320,2532,4865,9355,17975,34529];
 gen_Touches_VTouch1_samples = [2,2,2,3,5,8,13,24,46,89,168,323,627,1218,2351,4570,8892];
 gen_Touches_VTouch2_samples = [0,1,3,6,10,19,37,72,134,255,491,954,1838,3571,6937,13500,26230];
 gen_Touches_VTouch3_samples = [0,0,0,0,1,3,7,15,34,72,150,302,607,1201,2385,4705,9296];
 gen_Touches_V4_samples = [0,0,0,0,0,0,0,0,0,0,0,0,1,5,24,78,232];
 gen_Touches_QTouch1_samples = [1,2,4,7,14,27,54,101,198,383,746,1437,2810,5455,10646,20717,40494];
 gen_Touches_QTouch2_samples = [0,0,0,0,0,0,0,2,4,10,24,56,108,228,456,926,1804];
 gen_Touches_QI4_samples = [0,0,0,0,0,0,0,0,0,0,0,1,4,19,54,154,394];

gp_inline__location="../c-curve.tex:18088";
 vector_count_low_zeros(v) = {
   for(i=1,#v, if(v[i]!=0,return(i-1)));
   error("no non-zero in vector");
 }
 gf_count_low_zeros(g) = vector_count_low_zeros(gf_terms(g,80));

gp_inline__location="../c-curve.tex:18094";
 InverseTee_conf = apply(fill->1-fill, Tee_conf);

gp_inline__location="../c-curve.tex:18403";
 gV1left(x) = x/(1-x^4)*gQI1(x);

gp_inline__location="../c-curve.tex:18405";
V1left_samples = {[0,1,2,4,6,13,24,48,84,163,304,584,1082]};

gp_inline__location="../c-curve.tex:18414";
 gV3left(x) = x/(1-x^4)*gQI3(x);

gp_inline__location="../c-curve.tex:18416";
V3left_samples = {[0,0,0,0,0,0,0,0,0,2,6,20,56,156]};

gp_inline__location="../c-curve.tex:18429";
 gV1str(x)   = x*gV1left(x);

gp_inline__location="../c-curve.tex:18432";
 gV3str(x)   = x*gV3left(x);

gp_inline__location="../c-curve.tex:18436";
 gV1right(x) = x^2*gV1left(x);

gp_inline__location="../c-curve.tex:18439";
 gV3right(x) = x^2*gV3left(x);

gp_inline__location="../c-curve.tex:18443";
 gV1rev(x)   = x^3*gV1left(x);

gp_inline__location="../c-curve.tex:18446";
 gV3rev(x)   = x^3*gV3left(x);

gp_inline__location="../c-curve.tex:18483";
 gV2o(x) = x/(1-x) * gQI2o(x);

gp_inline__location="../c-curve.tex:18488";
V2o_samples = {[0,0,0,0,0,0,0,0,2,6,16,40,96,204]};

gp_inline__location="../c-curve.tex:18509";
 gVTouch1(x) = {
     gV1right(x) + gV1rev(x)
   + gV2rightA(x) + gV2revA(x)
   + gV3right(x) + gV3rev(x)
   + 2*gOnes(x);
 }

gp_inline__location="../c-curve.tex:18516";
VTouch1_samples = {[2,2,2,3,5,8,13,24,46,89]};

gp_inline__location="../c-curve.tex:18530";
 gVTouch2(x) = {
     gV1left(x) + gV1str(x)
   + gV2o(x)
   + gV3left(x) + gV3str(x);
 }

gp_inline__location="../c-curve.tex:18536";
VTouch2_samples = {[0,1,3,6,10,19,37,72,134,255]};

gp_inline__location="../c-curve.tex:18548";
 gVTouch3(x) = {
     gV2leftA(x) + gV2strA(x);
 }

gp_inline__location="../c-curve.tex:18552";
VTouch3_samples = {[0,0,0,0,1,3,7,15,34,72]};

gp_inline__location="../c-curve.tex:18573";
 gQTouch1(x) = gQI1(x) + gQI2a(x) + gQI3(x);

gp_inline__location="../c-curve.tex:18575";
QTouch1_samples = {[1,2,4,7,14,27,54,101,198,383]};

gp_inline__location="../c-curve.tex:18587";
 gQTouch2(x) = gQI2o(x);

gp_inline__location="../c-curve.tex:18600";
 gTouch1(x) = gVTouch1(x) + gQTouch1(x);
 Touch1(k) = gf_term(gTouch1(x),k);

gp_inline__location="../c-curve.tex:18603";
 VTouch1(k) = gf_term(gVTouch1(x),k);

gp_inline__location="../c-curve.tex:18606";
Touch1_samples = {[3,4,6,10,19,35,67,125,244,472]};

gp_inline__location="../c-curve.tex:18617";
 gTouch2(x) = gVTouch2(x) + gQTouch2(x);
 Touch2(k) = gf_term(gTouch2(x),k);

gp_inline__location="../c-curve.tex:18620";
 VTouch2(k) = gf_term(gVTouch2(x),k);
 gTouch3(x) = gVTouch3(x);

gp_inline__location="../c-curve.tex:18626";
Touch2_samples = {[0,1,3,6,10,19,37,74,138,265]};

gp_inline__location="../c-curve.tex:18644";
 Touch3(k) = VTouch3(k);
 VTouch3(k) = gf_term(gVTouch3(x),k);

gp_inline__location="../c-curve.tex:18654";
 PolyTouch3(x) = gf_to_linear_recurrence_poly(gTouch3(x));

gp_inline__location="../c-curve.tex:18692";
 \\ Return count of how many polygon pieces touch left end of
 \\ triangle 1.
 conf_VPoly_count(conf) = vector_cycle_count_run_1s(conf[1..8]);

gp_inline__location="../c-curve.tex:18699";
 conf_VPoly_net_count(conf) = {
   \\ if(vecsum(conf[1..8])%2, return(0));
   if(!conf[1],return(0));
   my(reps = conf[1]+conf[3]+conf[5]+conf[7]);
   if(reps==0,0, conf_VPoly_count(conf) / reps);
 }

gp_inline__location="../c-curve.tex:18709";
 gVPolyN_by_confs(n,x) = {
   if(n==1,gOnes(x))
   + sum(i=1,#conf_occurring_vector,
       my(conf = conf_occurring_vector[i]);
       if(conf[1] && conf_VPoly_count(conf)==n,
          my(reps = conf[1]+conf[3]+conf[5]+conf[7]);
          conf_to_gf(conf) / reps));
 }

gp_inline__location="../c-curve.tex:18819";
 Trigon_touches_mean_coefficients_numerators = {[
   371728, 1109244, -1142344, 1003322, -677148, 270311,
   -67483, -68763, 44657
 ]};
 Trigon_touches_mean_coefficients = {
   Trigon_touches_mean_coefficients_numerators / 488504;
 }
 Trigon_touches_mean_limit = {
   sum(i=1,#Trigon_touches_mean_coefficients,
       Trigon_touches_mean_coefficients[i] * r^(i-1));
 }
 Trigon_touches_mean_nfelt = {
   sum(i=1,#Trigon_touches_mean_coefficients,
       Trigon_touches_mean_coefficients[i]
       * nfeltpow(poly9_nf,r_nfelt,i-1));
 }

gp_inline__location="../c-curve.tex:18842";
 Trigon_touches_mean_limit_sample = {(3589245/1000000)};

gp_inline__location="../c-curve.tex:18888";
 Trigon_touches_proportion_coefficients_numerators = {[
   675976, 367080, -664352,  431384, -176436, -3018,
   164727, -182548, 56145
 ]};
 Trigon_touches_proportion_coefficients = {
   Trigon_touches_proportion_coefficients_numerators / 1052288;
 }
 Trigon_touches_proportion_limit = {
   sum(i=1,#Trigon_touches_proportion_coefficients,
       Trigon_touches_proportion_coefficients[i] * r^(i-1));
 }
 Trigon_touches_proportion_nfelt = {
   sum(i=1,#Trigon_touches_proportion_coefficients,
       Trigon_touches_proportion_coefficients[i]
       * nfeltpow(poly9_nf,r_nfelt,i-1));
 }

gp_inline__location="../c-curve.tex:18911";
 Trigon_touches_proportion_limit_sample = {(0632922/1000000)};

gp_inline__location="../c-curve.tex:18999";
 gen_Touches_14_1_count = 16;
 gen_Touches_14_1_bbox = [-105+I*-47,-102+I*-45];

gp_inline__location="../c-curve.tex:19003";
 gen_Touches_14_2_count = 4;
 gen_Touches_14_2_bbox = [-104+I*-48,-102+I*-47];

gp_inline__location="../c-curve.tex:19012";
 gen_Touches_14_2_middle = -103 - 48*I;

gp_inline__location="../c-curve.tex:19044";
 gen_Touches_16_1_count = 14;
 gen_Touches_16_1_bbox = [93+I*-208,95+I*-204];

gp_inline__location="../c-curve.tex:19052";
 gen_Touches_16_2_count = 44;
 gen_Touches_16_2_bbox = [89+I*-209,93+I*-204];

gp_inline__location="../c-curve.tex:19064";
 gen_Touches_16_1_middle = 95 - 206*I;

gp_inline__location="../c-curve.tex:19128";
 G(k) = if(k<0, error("G() is for k>=0")); \
        1/2*(sqrt2^k - 1/sqrt2^k);
 gG(x) = 1/2*(1/(1-sqrt2*x) - 1/(1-1/sqrt2*x));

gp_inline__location="../c-curve.tex:19134";
 Gtotal(k) = G(k)*2^k;
 gGtotal(x) = 1/2*( 1/(1-2*sqrt2*x) - 1/(1-sqrt2*x) );

gp_inline__location="../c-curve.tex:19138";
 Gtotal_by_segments(k) = {
   my(z = vecsum(c_curve_midpoints(k)) * conj(b)^(k-4));
   real(z) == -1/8*4^k || error("oops, real not at middle");
   imag(z) / sqrt2^(k-4);
 }
 G_by_segments(k) = Gtotal_by_segments(k)/2^k;

gp_inline__location="../c-curve.tex:19156";
 G_samples = {[ 0,
      sqrt2 / 4,
    3       / 4,
    7*sqrt2 / 8,
   15       / 8,
   31*sqrt2 / 16,
   63       / 16
 ]};

gp_inline__location="../c-curve.tex:19226";
 G_by_recurrence(k) = if(k==0, 0, 1/4*sqrt2^k + 1/sqrt2*G(k-1));

gp_inline__location="../c-curve.tex:19233";
 G_by_sum(k) = 1/4*sum(j=0,k-1, sqrt2^(k-j) * 1/sqrt2^j);

gp_inline__location="../c-curve.tex:19239";
 G_by_fraction(k) = 1/4*sqrt2*(sqrt2^k - (1/sqrt2)^k)/(sqrt2 - 1/sqrt2);

gp_inline__location="../c-curve.tex:19250";
 G_expand_offset(d) = 1/4*I^(d-1);

gp_inline__location="../c-curve.tex:19264";
 G_by_expand_inexact(k) =                    \
   sqrt(2) * (                               \
     G(k-1)*1.0                              \
     + 1/2^(k-1)                             \
       * (conj(b)/sqrt(2))^(k-3)             \
       * sum(d=0,3, 1/4*I^(d-1)*Skd(k-1,d))  \
   );

gp_inline__location="../c-curve.tex:19273";
 G_by_expand(k) =                    \
   my(z = (conj(b))^(k-3)            \
           * sum(d=0,3, 1/4*I^(d-1) * Skd(k-1,d))); \
   sqrt2*( G(k-1) + real(z) / sqrt2^(k-3) * 1/2^(k-1) );

gp_inline__location="../c-curve.tex:19315";
 G_fractal(k) = G(k) / sqrt2^k;
 G_fractal_limit = 1/2;

gp_inline__location="../c-curve.tex:19324";
 b_theta_rotate(t) = my(w=b_theta(t)); w/abs(w);
 G_theta_by_segments(k,t) = -imag(vector_mean(c_curve_theta_midpoints(k,t)) * conj(b_theta_rotate(t))^k);

gp_inline__location="../c-curve.tex:19328";
 G_theta(k,t) = 1/2*cos(t/2)*sin(t/2)^(k-1)*(2^k-1);

gp_inline__location="../c-curve.tex:19332";
 b_theta(t) = 1 + exp((Pi-t)*I);

gp_inline__location="../c-curve.tex:19346";
 b_theta_abs(t) = sqrt(2*(1-cos(t)));

gp_inline__location="../c-curve.tex:19351";
 b_theta_abs_half_angle(t) = 2*abs(sin(t/2));

gp_inline__location="../c-curve.tex:19361";
 G_theta_by_recurrence(k,t) = if(k==0, 0, 1/2*cos(t/2)*b_theta_abs(t)^(k-1) + sin(t/2)*G_theta(k-1,t));

gp_inline__location="../c-curve.tex:19371";
 G_theta_by_sum(k,t) = 1/2*cos(t/2) * sum(j=0,k-1, (2*sin(t/2))^(k-1-j) * sin(t/2)^j);

gp_inline__location="../c-curve.tex:19379";
 G_theta_by_fraction(k,t) = 1/(2*tan(t/2))*sin(t/2)^k*(2^k-1);

gp_inline__location="../c-curve.tex:19387";
 G_theta_fractal(k,t) = G_theta(k,t) / b_theta_abs(t)^k;
 G_theta_fractal_limit(t) = 1/(2*tan(t/2));

gp_inline__location="../c-curve.tex:19446";
 c_curve_part_midpoints(k) = \
   if(PartSegments(k)==0, [], c_curve_midpoints(k)[PartN(k)+1 .. PartN(k+1)]);

gp_inline__location="../c-curve.tex:19453";
 GpartTotal_by_segments(k) = {
   my(z = vecsum(c_curve_part_midpoints(k)) * conj(b)^(k-4));
   real(z) == -1/8*2^k*PartSegments(k) || error("oops, real not at middle");
   imag(z) / sqrt2^(k-4);
 }

gp_inline__location="../c-curve.tex:19483";
 Gpart(k) = {
   11/14 * sqrt2^k
   + 1/14 * ( - vector_modulo([15, 27],k) * sqrt2^k
              + vector_modulo([-18, 27*sqrt2],k)
            ) / (2^k + vector_modulo([2,-2],k) );
 }

gp_inline__location="../c-curve.tex:19498";
 Gpart_samples = {[
    0, -99,
    1, 5/4*sqrt2, 17/6, 59/20*sqrt2,
    135/22, 173/28*sqrt2
 ]};

gp_inline__location="../c-curve.tex:19512";
 GpartTotal_half = {[1,-7]};

gp_inline__location="../c-curve.tex:19521";
 GpartTotal_constant = [-2, 3*sqrt2];
 GpartTotal(k) = {
   11/42 * (2*sqrt2)^k
   + 1/6 * vector_modulo(GpartTotal_half,k) * (sqrt2)^k
   + 3/14*vector_modulo(GpartTotal_constant,k);
 }

gp_inline__location="../c-curve.tex:19537";
 GpartTotal_samples = [ 0, 0,  2, 5/2*sqrt2, 17, 59/2*sqrt2, \
                    135, 519/2*sqrt2 ];

gp_inline__location="../c-curve.tex:19668";
 GpartTotal_by_recurrence(k) = {
   if(k<4, GpartTotal(k),
      Gtotal(k)
      - 2*( PartSegments(k-1) * 1/2*sqrt2^(k-2)
            +     GpartTotal(k-1) * 1/2*sqrt2               )
      - 2*  PartSegments(k-2) * 1/2*sqrt2^(k-2)
      - 2*( PartSegments(k-3) * 1/2*sqrt2^(k-4)
            +     GpartTotal(k-3) * -1/2*sqrt2              )
      - (-Gtotal(k-4) - GpartTotal(k-4))
     );
 }

gp_inline__location="../c-curve.tex:19690";
 Gpart_limit = 11/14;

gp_inline__location="../c-curve.tex:20047";
 Ix_by_points(k) = {
   my(v = c_curve_midpoints_at_centroid(k));
   v *= conj(b)^(k-2);
   sum(i=1,#v, real(v[i])^2) / 2^(k-2);
 }
 Iy_by_points(k) = {
   my(v = c_curve_midpoints_at_centroid(k));
   v *= conj(b)^(k-2);
   sum(i=1,#v, imag(v[i])^2) / 2^(k-2);
 }
 Iz_by_points(k) = {
   my(v = c_curve_midpoints_at_centroid(k));
   sum(i=1,#v, abs(v[i]^2));
 }
 inertia_by_points(k) = {
   my(m = geom_inertia_of_points(c_curve_midpoints_at_centroid(k)));
   geom_inertia_rotate_xydir(m, b^k * -1);  \\ endpoint to West
 }

gp_inline__location="../c-curve.tex:20173";
 Ix(k) = 1/8*(4^k - (2*k-1)*2^k - 2);

gp_inline__location="../c-curve.tex:20176";
 Ix_samples = {[0,0,(1)/(4),(11)/(4),(71)/(4),(367)/(4),(1695)/(4),(7359)/(4)]};

gp_inline__location="../c-curve.tex:20207";
 Iy(k) = Ix(k+1) - Ix(k);

gp_inline__location="../c-curve.tex:20222";
 Iy_samples = {[0,(1)/(4),(5)/(2),15,74,332,1416,5872]};

gp_inline__location="../c-curve.tex:20251";
 Iz(k) = Ix(k) + Iy(k);

gp_inline__location="../c-curve.tex:20259";
 inertia(k) = {
   my(x=Ix(k), y=Iy(k), z=Iz(k));
   [ x,  0,  0;
     0,  y,  0;
     0,  0,  z ];
 }

gp_inline__location="../c-curve.tex:20331";
 I_t_of_k_minus_1(k) = 1/sqrt2*G(k-1) + 1/4*sqrt2^k;
 I_t(k) = I_t_of_k_minus_1(k+1);

gp_inline__location="../c-curve.tex:20337";
 I_t_of_k_minus_1_by_powers(k) = 1/2*(2^k-1)/sqrt2^k;
 I_t_by_powers(k) = 1/2*(2^(k+1)-1)/sqrt2^(k+1);

gp_inline__location="../c-curve.tex:20366";
 Ix_by_mutual(k) = Ix(k-1) + Iy(k-1);

gp_inline__location="../c-curve.tex:20374";
 Iy_by_mutual(k) = Ix(k-1) + Iy(k-1) + 2*2^(k-1)*I_t(k-1)^2;

gp_inline__location="../c-curve.tex:20385";
 I_t_squared(k) = 1/2 * (2^k - 1 + 1/4*1/2^k);

gp_inline__location="../c-curve.tex:20395";
 I_2k_t_squared(k) = 2*2^(k)*I_t(k)^2;

gp_inline__location="../c-curve.tex:20402";
 Iz_by_summation(k) = sum(j=0,k-1, 2^j*(4^(k-1-j) - 2^(k-1-j) + 1/4));

gp_inline__location="../c-curve.tex:20409";
 Iz_unsimplified(k) = (4^k - 2^k)/(4-2) - k*2^(k-1) + 1/4*(2^k-1);

gp_inline__location="../c-curve.tex:20419";
 Iz_by_expression(k) = 1/4*(2*4^k - (2*k + 1)*2^k - 1);

gp_inline__location="../c-curve.tex:20431";
 Iz_by_points2(k) = {
   my(g=G(k), v=c_curve_midpoints(k));
   sum(i=1,#v,
       my(p = (-1)*(v[i]/b^k - 1/2)); /* upward pointing */
       (real(p)*sqrt2^k)^2 + (imag(p)*sqrt2^k - g)^2 );
 }

gp_inline__location="../c-curve.tex:20439";
 Ix_by_points2(k) = {
   my(g=G(k), v=c_curve_midpoints(k));
   sum(i=1,#v,
       my(p = (-1)*(v[i]/b^k - 1/2)); /* upward pointing */
       (imag(p)*sqrt2^k - g)^2 );
 }

gp_inline__location="../c-curve.tex:20447";
 Iy_by_points2(k) = {
   my(g=G(k), v=c_curve_midpoints(k));
   sum(i=1,#v,
       my(p = (-1)*(v[i]/b^k - 1/2)); /* upward pointing */
       (real(p)*sqrt2^k)^2 );
 }

gp_inline__location="../c-curve.tex:20468";
 I_t_theta_of_k_minus_1(k,t) = G_theta(k-1,t) * cos(t/2) \
                               + 1/4 * b_theta_abs(t)^k;
 I_t_theta(k,t) = I_t_theta_of_k_minus_1(k+1,t);

gp_inline__location="../c-curve.tex:20478";
 I_t_theta_of_k_minus_1_by_powers(k,t) = \
   1/2*( 2^(k-1) - cos(t/2)^2 ) * sin(t/2)^(k-2);

gp_inline__location="../c-curve.tex:20495";
 I_z_theta_by_recurrence(k,t) = {
   if(k==0,return(0));
   2*I_z_theta(k-1,t) + 2 * 2^(k-1) * I_t_theta(k-1,t)^2;
 }

gp_inline__location="../c-curve.tex:20504";
 I_z_theta_by_sum(k,t) = \
   2^k * sum(j=0,k-1, I_t_theta(k-j-1,t)^2);

gp_inline__location="../c-curve.tex:20507";
 I_z_theta_by_sum_unsimplified(k,t) = \
   sum(j=0,k-1, 2^j * 2 * 2^(k-1-j) * I_t_theta(k-j-1,t)^2);

gp_inline__location="../c-curve.tex:20516";
 I_z_theta_pi_on_2(k) = Iz(k);

gp_inline__location="../c-curve.tex:20526";
 I_z_theta_pi_on_3(k) = (k-9/4)*2^k + 3 - 3/4*(1/2)^k;

gp_inline__location="../c-curve.tex:20551";
 I_z_theta_general(k,t) = {
     2/(1-2*cos(t))     * (8*sin(t/2)^2)^(k-1)
   + (1+cos(t))/cos(t)  * (4*sin(t/2)^2)^(k-1)
   - 1/4 * (1+cos(t))   * (2*sin(t/2)^2)^(k-1)
   - 1/4 * (1 + 2/((1-2*cos(t))*cos(t))) * 2^k;
 }
 I_z_theta(k,t) = {
   if(nearly_equal(cos(t),0), return(I_z_theta_pi_on_2(k)));
   if(nearly_equal(1-2*cos(t),0), return(I_z_theta_pi_on_3(k)));
   I_z_theta_general(k,t);
 }

gp_inline__location="../c-curve.tex:20578";
 inertia_theta_by_points(k,t) = {
   my(m = geom_inertia_of_points(c_curve_theta_midpoints_at_centroid(k,t)));
   geom_inertia_rotate_xydir(m, b_theta(t)^k * -1);
 }

gp_inline__location="../c-curve.tex:20640";
 I_z_theta_Pi(k) = 1/12 * 2^k * (4^k - 1);

gp_inline__location="../c-curve.tex:20643";
 I_z_theta_Pi_by_points(k) = \
   geom_inertia_Iz(geom_inertia_of_points(vector_offset_to_centroid(vector(2^k,i, i))));

kill(gp_inline__location);
